@@ -13,13 +13,14 @@ namespace ESSImport
     {
         while (esm.isNextSub("NPCO"))
         {
-            ESM::ContItem contItem;
+            ContItem contItem;
             esm.getHT(contItem);
 
             InventoryItem item;
             item.mId = contItem.mItem.toString();
             item.mCount = contItem.mCount;
             item.mRelativeEquipmentSlot = -1;
+            item.mLockLevel = 0;
 
             unsigned int itemCount = std::abs(item.mCount);
             bool separateStacks = false;
