@@ -230,9 +230,19 @@ MwIniImporter::MwIniImporter()
         "Blood:Texture 0",
         "Blood:Texture 1",
         "Blood:Texture 2",
+        "Blood:Texture 3",
+        "Blood:Texture 4",
+        "Blood:Texture 5",
+        "Blood:Texture 6",
+        "Blood:Texture 7",
         "Blood:Texture Name 0",
         "Blood:Texture Name 1",
         "Blood:Texture Name 2",
+        "Blood:Texture Name 3",
+        "Blood:Texture Name 4",
+        "Blood:Texture Name 5",
+        "Blood:Texture Name 6",
+        "Blood:Texture Name 7",
 
         // movies
         "Movies:Company Logo",
@@ -624,17 +634,6 @@ MwIniImporter::MwIniImporter()
         "Moons:Masser Fade Out Finish",
         "Moons:Script Color",
 
-        // blood
-        "Blood:Model 0",
-        "Blood:Model 1",
-        "Blood:Model 2",
-        "Blood:Texture 0",
-        "Blood:Texture 1",
-        "Blood:Texture 2",
-        "Blood:Texture Name 0",
-        "Blood:Texture Name 1",
-        "Blood:Texture Name 2",
-
         // werewolf (Bloodmoon)
         "General:Werewolf FOV",
 
@@ -860,11 +859,12 @@ std::vector<std::string>::iterator MwIniImporter::findString(std::vector<std::st
 }
 
 void MwIniImporter::addPaths(std::vector<boost::filesystem::path>& output, std::vector<std::string> input) {
-    for (auto& path : input) {
+    for (auto& path : input)
+    {
         if (path.front() == '"')
         {
-            path.erase(path.begin());
-            path.erase(path.end() - 1);
+            // Drop first and last characters - quotation marks
+            path = path.substr(1, path.size() - 2);
         }
         output.emplace_back(path);
     }
@@ -874,7 +874,6 @@ void MwIniImporter::importGameFiles(multistrmap &cfg, const multistrmap &ini, co
 {
     std::vector<std::pair<std::time_t, boost::filesystem::path>> contentFiles;
     std::string baseGameFile("Game Files:GameFile");
-    std::string gameFile("");
     std::time_t defaultTime = 0;
     ToUTF8::Utf8Encoder encoder(mEncoding);
 
@@ -890,7 +889,7 @@ void MwIniImporter::importGameFiles(multistrmap &cfg, const multistrmap &ini, co
     multistrmap::const_iterator it = ini.begin();
     for (int i=0; it != ini.end(); i++)
     {
-        gameFile = baseGameFile;
+        std::string gameFile = baseGameFile;
         gameFile.append(std::to_string(i));
 
         it = ini.find(gameFile);
