@@ -5,8 +5,9 @@
 #include <QLocalSocket>
 #include <QMessageBox>
 
+#include <components/debug/debuglog.hpp>
 #include <components/fallback/validate.hpp>
-
+#include <components/misc/rng.hpp>
 #include <components/nifosg/nifloader.hpp>
 
 #include "model/doc/document.hpp"
@@ -18,15 +19,19 @@
 
 using namespace Fallback;
 
-CS::Editor::Editor ()
+CS::Editor::Editor (int argc, char **argv)
 : mSettingsState (mCfgMgr), mDocumentManager (mCfgMgr),
-  mViewManager (mDocumentManager), mPid(""),
-  mLock(), mMerge (mDocumentManager),
-  mIpcServerName ("org.openmw.OpenCS"), mServer(NULL), mClientSocket(NULL)
+  mPid(""), mLock(), mMerge (mDocumentManager),
+  mIpcServerName ("org.openmw.OpenCS"), mServer(nullptr), mClientSocket(nullptr)
 {
     std::pair<Files::PathContainer, std::vector<std::string> > config = readConfig();
 
-    setupDataFiles (config.first);
+    mViewManager = new CSVDoc::ViewManager(mDocumentManager);
+    if (argc > 1)
+    {
+        mFileToLoad = argv[1];
+        mDataDirs = config.first;
+    }
 
     NifOsg::Loader::setShowMarkers(true);
 
@@ -43,11 +48,11 @@ CS::Editor::Editor ()
     connect (&mDocumentManager, SIGNAL (lastDocumentDeleted()),
         this, SLOT (lastDocumentDeleted()));
 
-    connect (&mViewManager, SIGNAL (newGameRequest ()), this, SLOT (createGame ()));
-    connect (&mViewManager, SIGNAL (newAddonRequest ()), this, SLOT (createAddon ()));
-    connect (&mViewManager, SIGNAL (loadDocumentRequest ()), this, SLOT (loadDocument ()));
-    connect (&mViewManager, SIGNAL (editSettingsRequest()), this, SLOT (showSettings ()));
-    connect (&mViewManager, SIGNAL (mergeDocument (CSMDoc::Document *)), this, SLOT (mergeDocument (CSMDoc::Document *)));
+    connect (mViewManager, SIGNAL (newGameRequest ()), this, SLOT (createGame ()));
+    connect (mViewManager, SIGNAL (newAddonRequest ()), this, SLOT (createAddon ()));
+    connect (mViewManager, SIGNAL (loadDocumentRequest ()), this, SLOT (loadDocument ()));
+    connect (mViewManager, SIGNAL (editSettingsRequest()), this, SLOT (showSettings ()));
+    connect (mViewManager, SIGNAL (mergeDocument (CSMDoc::Document *)), this, SLOT (mergeDocument (CSMDoc::Document *)));
 
     connect (&mStartup, SIGNAL (createGame()), this, SLOT (createGame ()));
     connect (&mStartup, SIGNAL (createAddon()), this, SLOT (createAddon ()));
@@ -68,6 +73,8 @@ CS::Editor::Editor ()
 
 CS::Editor::~Editor ()
 {
+    delete mViewManager;
+
     mPidFile.close();
 
     if(mServer && boost::filesystem::exists(mPid))
@@ -75,15 +82,6 @@ CS::Editor::~Editor ()
         remove(mPid.string().c_str())); // ignore any error
 }
 
-void CS::Editor::setupDataFiles (const Files::PathContainer& dataDirs)
-{
-    for (Files::PathContainer::const_iterator iter = dataDirs.begin(); iter != dataDirs.end(); ++iter)
-    {
-        QString path = QString::fromUtf8 (iter->string().c_str());
-        mFileDialog.addFiles(path);
-    }
-}
-
 std::pair<Files::PathContainer, std::vector<std::string> > CS::Editor::readConfig(bool quiet)
 {
     boost::program_options::variables_map variables;
@@ -106,15 +104,16 @@ std::pair<Files::PathContainer, std::vector<std::string> > CS::Editor::readConfi
 
     boost::program_options::notify(variables);
 
-    mCfgMgr.readConfiguration(variables, desc, quiet);
+    mCfgMgr.readConfiguration(variables, desc, false);
 
-    mDocumentManager.setEncoding (
-        ToUTF8::calculateEncoding (variables["encoding"].as<Files::EscapeHashString>().toStdString()));
+    Fallback::Map::init(variables["fallback"].as<FallbackMap>().mMap);
+
+    mEncodingName = variables["encoding"].as<Files::EscapeHashString>().toStdString();
+    mDocumentManager.setEncoding(ToUTF8::calculateEncoding(mEncodingName));
+    mFileDialog.setEncoding (QString::fromUtf8(mEncodingName.c_str()));
 
     mDocumentManager.setResourceDir (mResources = variables["resources"].as<Files::EscapeHashString>().toStdString());
 
-    mDocumentManager.setFallbackMap (variables["fallback"].as<FallbackMap>().mMap);
-
     if (variables["script-blacklist-use"].as<bool>())
         mDocumentManager.setBlacklistedScripts (
             variables["script-blacklist"].as<Files::EscapeStringVector>().toStdStringVector());
@@ -155,7 +154,7 @@ std::pair<Files::PathContainer, std::vector<std::string> > CS::Editor::readConfi
     dataDirs.insert (dataDirs.end(), dataLocal.begin(), dataLocal.end());
 
     //iterate the data directories and add them to the file dialog for loading
-    for (Files::PathContainer::const_iterator iter = dataDirs.begin(); iter != dataDirs.end(); ++iter)
+    for (Files::PathContainer::const_reverse_iterator iter = dataDirs.rbegin(); iter != dataDirs.rend(); ++iter)
     {
         QString path = QString::fromUtf8 (iter->string().c_str());
         mFileDialog.addFiles(path);
@@ -194,8 +193,7 @@ void CS::Editor::createAddon()
     mStartup.hide();
 
     mFileDialog.clearFiles();
-    std::pair<Files::PathContainer, std::vector<std::string> > config = readConfig(/*quiet*/true);
-    setupDataFiles (config.first);
+    readConfig(/*quiet*/true);
 
     mFileDialog.showDialog (CSVDoc::ContentAction_New);
 }
@@ -219,18 +217,24 @@ void CS::Editor::loadDocument()
     mStartup.hide();
 
     mFileDialog.clearFiles();
-    std::pair<Files::PathContainer, std::vector<std::string> > config = readConfig(/*quiet*/true);
-    setupDataFiles (config.first);
+    readConfig(/*quiet*/true);
 
     mFileDialog.showDialog (CSVDoc::ContentAction_Edit);
 }
 
-void CS::Editor::openFiles (const boost::filesystem::path &savePath)
+void CS::Editor::openFiles (const boost::filesystem::path &savePath, const std::vector<boost::filesystem::path> &discoveredFiles)
 {
     std::vector<boost::filesystem::path> files;
 
-    foreach (const QString &path, mFileDialog.selectedFilePaths())
-        files.push_back(path.toUtf8().constData());
+    if(discoveredFiles.empty())
+    {
+        for (const QString &path : mFileDialog.selectedFilePaths())
+            files.push_back(path.toUtf8().constData());
+    }
+    else
+    {
+        files = discoveredFiles;
+    }
 
     mDocumentManager.addDocument (files, savePath, false);
 
@@ -241,7 +245,7 @@ void CS::Editor::createNewFile (const boost::filesystem::path &savePath)
 {
     std::vector<boost::filesystem::path> files;
 
-    foreach (const QString &path, mFileDialog.selectedFilePaths()) {
+    for (const QString &path : mFileDialog.selectedFilePaths()) {
         files.push_back(path.toUtf8().constData());
     }
 
@@ -294,7 +298,7 @@ bool CS::Editor::makeIPCServer()
         mLock = boost::interprocess::file_lock(mPid.string().c_str());
         if(!mLock.try_lock())
         {
-            std::cerr << "OpenCS already running."  << std::endl;
+            Log(Debug::Error) << "Error: OpenMW-CS is already running.";
             return false;
         }
 
@@ -317,17 +321,17 @@ bool CS::Editor::makeIPCServer()
             if(boost::filesystem::exists(fullPath.toUtf8().constData()))
             {
                 // TODO: compare pid of the current process with that in the file
-                std::cout << "Detected unclean shutdown." << std::endl;
+                Log(Debug::Info) << "Detected unclean shutdown.";
                 // delete the stale file
                 if(remove(fullPath.toUtf8().constData()))
-                    std::cerr << "ERROR removing stale connection file" << std::endl;
+                    Log(Debug::Error) << "Error: can not remove stale connection file.";
             }
         }
     }
 
     catch(const std::exception& e)
     {
-        std::cerr << "ERROR " << e.what() << std::endl;
+        Log(Debug::Error) << "Error: " << e.what();
         return false;
     }
 
@@ -338,7 +342,7 @@ bool CS::Editor::makeIPCServer()
     }
 
     mServer->close();
-    mServer = NULL;
+    mServer = nullptr;
     return false;
 }
 
@@ -354,16 +358,62 @@ int CS::Editor::run()
     if (mLocal.empty())
         return 1;
 
-    mStartup.show();
+    Misc::Rng::init();
 
-    QApplication::setQuitOnLastWindowClosed (true);
+    QApplication::setQuitOnLastWindowClosed(true);
+
+    if (mFileToLoad.empty())
+    {
+        mStartup.show();
+    }
+    else
+    {
+        ESM::ESMReader fileReader;
+        ToUTF8::Utf8Encoder encoder = ToUTF8::calculateEncoding(mEncodingName);
+        fileReader.setEncoder(&encoder);
+        fileReader.open(mFileToLoad.string());
+
+        std::vector<boost::filesystem::path> discoveredFiles;
+
+        for (std::vector<ESM::Header::MasterData>::const_iterator itemIter = fileReader.getGameFiles().begin();
+            itemIter != fileReader.getGameFiles().end(); ++itemIter)
+        {
+            for (Files::PathContainer::const_iterator pathIter = mDataDirs.begin();
+                pathIter != mDataDirs.end(); ++pathIter)
+            {
+                const boost::filesystem::path masterPath = *pathIter / itemIter->name;
+                if (boost::filesystem::exists(masterPath))
+                {
+                    discoveredFiles.push_back(masterPath);
+                    break;
+                }
+            }
+        }
+        discoveredFiles.push_back(mFileToLoad);
+
+        QString extension = QString::fromStdString(mFileToLoad.extension().string()).toLower();
+        if (extension == ".esm")
+        {
+            mFileToLoad.replace_extension(".omwgame");
+            mDocumentManager.addDocument(discoveredFiles, mFileToLoad, false);
+        }
+        else if (extension == ".esp")
+        {
+            mFileToLoad.replace_extension(".omwaddon");
+            mDocumentManager.addDocument(discoveredFiles, mFileToLoad, false);
+        }
+        else
+        {
+            openFiles(mFileToLoad, discoveredFiles);
+        }
+    }
 
     return QApplication::exec();
 }
 
 void CS::Editor::documentAdded (CSMDoc::Document *document)
 {
-    mViewManager.addView (document);
+    mViewManager->addView (document);
 }
 
 void CS::Editor::documentAboutToBeRemoved (CSMDoc::Document *document)
