@@ -3,10 +3,9 @@
 
 #include <string>
 #include <map>
+#include <memory>
 #include <vector>
 
-#include <boost/shared_ptr.hpp>
-
 #include <QMetaType>
 
 namespace CSMWorld
@@ -48,6 +47,6 @@ namespace CSMFilter
     };
 }
 
-Q_DECLARE_METATYPE (boost::shared_ptr<CSMFilter::Node>)
+Q_DECLARE_METATYPE (std::shared_ptr<CSMFilter::Node>)
 
 #endif
