@@ -19,10 +19,11 @@ namespace CSMTools
 
         private:
             const CSMWorld::RefCollection& mReferences;
-            const CSMWorld::RefIdCollection& mReferencables;
+            const CSMWorld::RefIdCollection& mObjects;
             const CSMWorld::RefIdData& mDataSet;
             const CSMWorld::IdCollection<CSMWorld::Cell>& mCells;
             const CSMWorld::IdCollection<ESM::Faction>& mFactions;
+            bool mIgnoreBaseRecords;
     };
 }
 
