@@ -136,7 +136,7 @@ namespace CSMWorld
     struct VarTypeColumn : public Column<ESXRecordT>
     {
         VarTypeColumn (ColumnBase::Display display)
-        : Column<ESXRecordT> (Columns::ColumnId_ValueType, display)
+            : Column<ESXRecordT> (Columns::ColumnId_ValueType, display, ColumnBase::Flag_Table | ColumnBase::Flag_Dialogue | ColumnBase::Flag_Dialogue_Refresh)
         {}
 
         virtual QVariant get (const Record<ESXRecordT>& record) const
@@ -161,7 +161,7 @@ namespace CSMWorld
     template<typename ESXRecordT>
     struct VarValueColumn : public Column<ESXRecordT>
     {
-        VarValueColumn() : Column<ESXRecordT> (Columns::ColumnId_Value, ColumnBase::Display_Var) {}
+        VarValueColumn() : Column<ESXRecordT> (Columns::ColumnId_Value, ColumnBase::Display_Var, ColumnBase::Flag_Table | ColumnBase::Flag_Dialogue | ColumnBase::Flag_Dialogue_Refresh) {}
 
         virtual QVariant get (const Record<ESXRecordT>& record) const
         {
@@ -1371,12 +1371,12 @@ namespace CSMWorld
         RotColumn (ESM::Position ESXRecordT::* position, int index, bool door)
         : Column<ESXRecordT> (
           (door ? Columns::ColumnId_DoorPositionXRot : Columns::ColumnId_PositionXRot)+index,
-          ColumnBase::Display_Float), mPosition (position), mIndex (index) {}
+          ColumnBase::Display_Double), mPosition (position), mIndex (index) {}
 
         virtual QVariant get (const Record<ESXRecordT>& record) const
         {
             const ESM::Position& position = record.get().*mPosition;
-            return position.rot[mIndex];
+            return osg::RadiansToDegrees(position.rot[mIndex]);
         }
 
         virtual void set (Record<ESXRecordT>& record, const QVariant& data)
@@ -1385,7 +1385,7 @@ namespace CSMWorld
 
             ESM::Position& position = record2.*mPosition;
 
-            position.rot[mIndex] = data.toFloat();
+            position.rot[mIndex] = osg::DegreesToRadians(data.toFloat());
 
             record.setModified (record2);
         }
@@ -1770,7 +1770,7 @@ namespace CSMWorld
     struct GenderNpcColumn : public Column<ESXRecordT>
     {
         GenderNpcColumn()
-            : Column<ESXRecordT>(Columns::ColumnId_GenderNpc, ColumnBase::Display_GenderNpc)
+            : Column<ESXRecordT>(Columns::ColumnId_Gender, ColumnBase::Display_GenderNpc)
         {}
 
         virtual QVariant get(const Record<ESXRecordT>& record) const
@@ -2461,17 +2461,6 @@ namespace CSMWorld
         bool isEditable() const override;
     };
 
-    struct LandMapLodColumn : public Column<Land>
-    {
-        using DataType = QVector<signed char>;
-
-        LandMapLodColumn();
-
-        QVariant get(const Record<Land>& record) const override;
-        void set(Record<Land>& record, const QVariant& data) override;
-        bool isEditable() const override;
-    };
-
     struct LandNormalsColumn : public Column<Land>
     {
         using DataType = QVector<signed char>;
@@ -2529,8 +2518,7 @@ namespace CSMWorld
 }
 
 // This is required to access the type as a QVariant.
-Q_DECLARE_METATYPE(CSMWorld::LandMapLodColumn::DataType)
-//Q_DECLARE_METATYPE(CSMWorld::LandNormalsColumn::DataType) // Same as LandMapLodColumn::DataType
+Q_DECLARE_METATYPE(CSMWorld::LandNormalsColumn::DataType)
 Q_DECLARE_METATYPE(CSMWorld::LandHeightsColumn::DataType)
 Q_DECLARE_METATYPE(CSMWorld::LandColoursColumn::DataType)
 Q_DECLARE_METATYPE(CSMWorld::LandTexturesColumn::DataType)
