@@ -25,21 +25,6 @@ namespace CSMWorld
     struct Pathgrid;
     struct Info;
 
-    struct PathgridPointsWrap : public NestedTableWrapperBase
-    {
-        ESM::Pathgrid mRecord;
-
-        PathgridPointsWrap(ESM::Pathgrid pathgrid)
-            : mRecord(pathgrid) {}
-
-        virtual ~PathgridPointsWrap() {}
-
-        virtual int size() const
-        {
-            return mRecord.mPoints.size(); // used in IdTree::setNestedTable()
-        }
-    };
-
     class PathgridPointListAdapter : public NestedColumnAdapter<Pathgrid>
     {
     public:
@@ -115,6 +100,31 @@ namespace CSMWorld
         virtual int getRowsCount(const Record<ESM::Faction>& record) const;
     };
 
+    class FactionRanksAdapter : public NestedColumnAdapter<ESM::Faction>
+    {
+    public:
+        FactionRanksAdapter ();
+
+        virtual void addRow(Record<ESM::Faction>& record, int position) const;
+
+        virtual void removeRow(Record<ESM::Faction>& record, int rowToRemove) const;
+
+        virtual void setTable(Record<ESM::Faction>& record,
+                const NestedTableWrapperBase& nestedTable) const;
+
+        virtual NestedTableWrapperBase* table(const Record<ESM::Faction>& record) const;
+
+        virtual QVariant getData(const Record<ESM::Faction>& record,
+                int subRowIndex, int subColIndex) const;
+
+        virtual void setData(Record<ESM::Faction>& record,
+                const QVariant& value, int subRowIndex, int subColIndex) const;
+
+        virtual int getColumnsCount(const Record<ESM::Faction>& record) const;
+
+        virtual int getRowsCount(const Record<ESM::Faction>& record) const;
+    };
+
     class RegionSoundListAdapter : public NestedColumnAdapter<ESM::Region>
     {
     public:
@@ -540,6 +550,31 @@ namespace CSMWorld
 
         virtual int getRowsCount(const Record<CSMWorld::Cell>& record) const;
     };
+
+    class RegionWeatherAdapter : public NestedColumnAdapter<ESM::Region>
+    {
+    public:
+        RegionWeatherAdapter ();
+
+        virtual void addRow(Record<ESM::Region>& record, int position) const;
+
+        virtual void removeRow(Record<ESM::Region>& record, int rowToRemove) const;
+
+        virtual void setTable(Record<ESM::Region>& record,
+                const NestedTableWrapperBase& nestedTable) const;
+
+        virtual NestedTableWrapperBase* table(const Record<ESM::Region>& record) const;
+
+        virtual QVariant getData(const Record<ESM::Region>& record,
+                int subRowIndex, int subColIndex) const;
+
+        virtual void setData(Record<ESM::Region>& record,
+                const QVariant& value, int subRowIndex, int subColIndex) const;
+
+        virtual int getColumnsCount(const Record<ESM::Region>& record) const;
+
+        virtual int getRowsCount(const Record<ESM::Region>& record) const;
+    };
 }
 
 #endif // CSM_WOLRD_NESTEDCOLADAPTERIMP_H
