@@ -4,6 +4,8 @@
 #include <QDialog>
 #include <QModelIndex>
 
+#ifndef Q_MOC_RUN
+
 #include <boost/filesystem/path.hpp>
 #include "adjusterwidget.hpp"
 
@@ -12,6 +14,8 @@
 Q_DECLARE_METATYPE (boost::filesystem::path)
 #endif
 
+#endif
+
 #include "ui_filedialog.h"
 
 namespace ContentSelectorView
@@ -42,6 +46,7 @@ namespace CSVDoc
         void showDialog (ContentAction action);
 
         void addFiles (const QString &path);
+        void setEncoding (const QString &encoding);
         void clearFiles ();
 
         QString filename() const;
