@@ -38,7 +38,7 @@ namespace CSVRender
         , mCameraSensitivity(1/650.f)
         , mSecondaryMoveMult(50)
         , mWheelMoveMult(8)
-        , mCamera(NULL)
+        , mCamera(nullptr)
     {
     }
 
@@ -81,7 +81,7 @@ namespace CSVRender
         bool wasActive = mActive;
 
         mCamera = camera;
-        mActive = (mCamera != NULL);
+        mActive = (mCamera != nullptr);
 
         if (mActive != wasActive)
         {
@@ -463,6 +463,7 @@ namespace CSVRender
         , mDistance(0)
         , mOrbitSpeed(osg::PI / 4)
         , mOrbitSpeedMult(4)
+        , mConstRoll(false)
     {
         CSMPrefs::Shortcut* naviPrimaryShortcut = new CSMPrefs::Shortcut("scene-navi-primary", widget);
         naviPrimaryShortcut->enable(false);
