@@ -10,10 +10,11 @@
 #include "../../model/prefs/state.hpp"
 #include "../../model/prefs/shortcutmanager.hpp"
 
-#include "mask.hpp"
+#include <components/misc/constants.hpp>
+#include <components/sceneutil/vismask.hpp>
 
 CSVRender::CellArrowTag::CellArrowTag (CellArrow *arrow)
-: TagBase (Mask_CellArrow), mArrow (arrow)
+: TagBase (SceneUtil::Mask_EditorCellArrow), mArrow (arrow)
 {}
 
 CSVRender::CellArrow *CSVRender::CellArrowTag::getCellArrow() const
@@ -57,7 +58,7 @@ QString CSVRender::CellArrowTag::getToolTip (bool hideBasics) const
 void CSVRender::CellArrow::adjustTransform()
 {
     // position
-    const int cellSize = 8192;
+    const int cellSize = Constants::CellSizeInUnits;
     const int offset = cellSize / 2 + 800;
 
     int x = mCoordinates.getX()*cellSize + cellSize/2;
@@ -173,7 +174,7 @@ CSVRender::CellArrow::CellArrow (osg::Group *cellNode, Direction direction,
 
     mParentNode->addChild (mBaseNode);
 
-    mBaseNode->setNodeMask (Mask_CellArrow);
+    mBaseNode->setNodeMask (SceneUtil::Mask_EditorCellArrow);
 
     adjustTransform();
     buildShape();
