@@ -11,12 +11,12 @@
 #include <components/resource/imagemanager.hpp>
 #include <components/resource/resourcesystem.hpp>
 #include <components/sceneutil/waterutil.hpp>
+#include <components/sceneutil/vismask.hpp>
 
 #include "../../model/world/cell.hpp"
 #include "../../model/world/cellcoordinates.hpp"
 #include "../../model/world/data.hpp"
 
-#include "mask.hpp"
 
 namespace CSVRender
 {
@@ -38,7 +38,7 @@ namespace CSVRender
         mWaterTransform->setPosition(osg::Vec3f(cellCoords.getX() * CellSize + CellSize / 2.f,
             cellCoords.getY() * CellSize + CellSize / 2.f, 0));
 
-        mWaterTransform->setNodeMask(Mask_Water);
+        mWaterTransform->setNodeMask(SceneUtil::Mask_Water);
         mParentNode->addChild(mWaterTransform);
 
         mWaterNode = new osg::Geode();
@@ -50,7 +50,7 @@ namespace CSVRender
             updateCellData(mData.getCells().getRecord(cellIndex));
         }
 
-        // Keep water existance/height up to date
+        // Keep water existence/height up to date
         QAbstractItemModel* cells = mData.getTableModel(CSMWorld::UniversalId::Type_Cells);
         connect(cells, SIGNAL(dataChanged(const QModelIndex&, const QModelIndex&)),
             this, SLOT(cellDataChanged(const QModelIndex&, const QModelIndex&)));
@@ -92,6 +92,11 @@ namespace CSVRender
         }
     }
 
+    void CellWater::reloadAssets()
+    {
+        recreate();
+    }
+
     void CellWater::cellDataChanged(const QModelIndex& topLeft, const QModelIndex& bottomRight)
     {
         const CSMWorld::Collection<CSMWorld::Cell>& cells = mData.getCells();
@@ -159,7 +164,7 @@ namespace CSVRender
         mWaterGeometry->setStateSet(SceneUtil::createSimpleWaterStateSet(Alpha, RenderBin));
 
         // Add water texture
-        std::string textureName = mData.getFallbackMap()->getFallbackString("Water_SurfaceTexture");
+        std::string textureName = Fallback::Map::getString("Water_SurfaceTexture");
         textureName = "textures/water/" + textureName + "00.dds";
 
         Resource::ImageManager* imageManager = mData.getResourceSystem()->getImageManager();
