@@ -19,7 +19,7 @@ namespace CSVRender
             Lighting() : mRootNode(0) {}
             virtual ~Lighting();
 
-            virtual void activate (osg::Group* rootNode) = 0;
+            virtual void activate (osg::Group* rootNode, bool isExterior) = 0;
 
             virtual void deactivate() = 0;
 
@@ -27,6 +27,8 @@ namespace CSVRender
 
         protected:
 
+            void updateDayNightMode(int index);
+
             osg::ref_ptr<osg::LightSource> mLightSource;
             osg::Group* mRootNode;
     };
