@@ -11,7 +11,7 @@ namespace CSVRender
 
             LightingDay();
 
-            virtual void activate (osg::Group* rootNode);
+            virtual void activate (osg::Group* rootNode, bool /*isExterior*/);
 
             virtual void deactivate();
 
