@@ -64,6 +64,8 @@ namespace CSVRender
             osg::ref_ptr<osgViewer::View> mView;
             osg::ref_ptr<osg::Group> mRootNode;
 
+            void updateCameraParameters(double overrideAspect = -1.0);
+
             QTimer mTimer;
 
         protected slots:
@@ -87,6 +89,8 @@ namespace CSVRender
             void setDefaultAmbient (const osg::Vec4f& colour);
             ///< \note The actual ambient colour may differ based on lighting settings.
 
+            void setExterior (bool isExterior);
+
         protected:
             void setLighting (Lighting *lighting);
             ///< \attention The ownership of \a lighting is not transferred to *this.
@@ -102,6 +106,7 @@ namespace CSVRender
 
             osg::Vec4f mDefaultAmbient;
             bool mHasDefaultAmbient;
+            bool mIsExterior;
             LightingDay mLightingDay;
             LightingNight mLightingNight;
             LightingBright mLightingBright;
