@@ -5,19 +5,19 @@
 
 #include <QString>
 
-#include "elements.hpp"
+#include <components/sceneutil/vismask.hpp>
 
 namespace CSVRender
 {
     class TagBase : public osg::Referenced
     {
-            Elements mElement;
+            SceneUtil::VisMask mMask;
 
         public:
 
-            TagBase (Elements element);
+            TagBase (SceneUtil::VisMask mask);
 
-            Elements getElement() const;
+            SceneUtil::VisMask getMask() const;
 
             virtual QString getToolTip (bool hideBasics) const;
 
