@@ -17,6 +17,7 @@ namespace CSMDoc
 namespace CSMWorld
 {
     class IdTable;
+    class CellCoordinates;
 }
 
 namespace CSVRender
@@ -29,7 +30,7 @@ namespace CSVRender
             std::string mCellId;
             CSMWorld::IdTable *mCellsModel;
             CSMWorld::IdTable *mReferenceablesModel;
-            std::auto_ptr<Cell> mCell;
+            std::unique_ptr<Cell> mCell;
 
             void update();
 
@@ -63,6 +64,8 @@ namespace CSVRender
 
             virtual Cell* getCell(const osg::Vec3d& point) const;
 
+            virtual Cell* getCell(const CSMWorld::CellCoordinates& coords) const;
+
             virtual std::vector<osg::ref_ptr<TagBase> > getSelection (unsigned int elementMask)
                 const;
 
@@ -108,6 +111,8 @@ namespace CSVRender
 
             void cellRowsAboutToBeRemoved (const QModelIndex& parent, int start, int end);
 
+            void assetTablesChanged ();
+
         signals:
 
             void cellChanged(const CSMWorld::UniversalId& id);
