@@ -26,8 +26,9 @@
 #include "../widget/scenetooltoggle2.hpp"
 #include "../widget/scenetoolrun.hpp"
 
+#include <components/sceneutil/vismask.hpp>
+
 #include "object.hpp"
-#include "mask.hpp"
 #include "instancemode.hpp"
 #include "pathgridmode.hpp"
 #include "cameracontroller.hpp"
@@ -101,6 +102,9 @@ CSVRender::WorldspaceWidget::WorldspaceWidget (CSMDoc::Document& document, QWidg
     // Shortcuts
     CSMPrefs::Shortcut* primaryEditShortcut = new CSMPrefs::Shortcut("scene-edit-primary", "scene-speed-modifier",
             CSMPrefs::Shortcut::SM_Detach, this);
+    CSMPrefs::Shortcut* primaryOpenShortcut = new CSMPrefs::Shortcut("scene-open-primary", this);
+
+    connect(primaryOpenShortcut, SIGNAL(activated(bool)), this, SLOT(primaryOpen(bool)));
     connect(primaryEditShortcut, SIGNAL(activated(bool)), this, SLOT(primaryEdit(bool)));
     connect(primaryEditShortcut, SIGNAL(secondary(bool)), this, SLOT(speedMode(bool)));
 
@@ -135,7 +139,7 @@ void CSVRender::WorldspaceWidget::settingChanged (const CSMPrefs::Setting *setti
     {
         float alpha = setting->toDouble();
         // getSelection is virtual, thus this can not be called from the constructor
-        auto selection = getSelection(Mask_Reference);
+        auto selection = getSelection(SceneUtil::Mask_EditorReference);
         for (osg::ref_ptr<TagBase> tag : selection)
         {
             if (auto objTag = dynamic_cast<ObjectTag*>(tag.get()))
@@ -342,7 +346,7 @@ unsigned int CSVRender::WorldspaceWidget::getVisibilityMask() const
 
 void CSVRender::WorldspaceWidget::setInteractionMask (unsigned int mask)
 {
-    mInteractionMask = mask | Mask_CellMarker | Mask_CellArrow;
+    mInteractionMask = mask | SceneUtil::Mask_EditorCellMarker | SceneUtil::Mask_EditorCellArrow;
 }
 
 unsigned int CSVRender::WorldspaceWidget::getInteractionMask() const
@@ -358,15 +362,15 @@ void CSVRender::WorldspaceWidget::setEditLock (bool locked)
 void CSVRender::WorldspaceWidget::addVisibilitySelectorButtons (
     CSVWidget::SceneToolToggle2 *tool)
 {
-    tool->addButton (Button_Reference, Mask_Reference, "Instances");
-    tool->addButton (Button_Water, Mask_Water, "Water");
-    tool->addButton (Button_Pathgrid, Mask_Pathgrid, "Pathgrid");
+    tool->addButton (Button_Reference, SceneUtil::Mask_EditorReference, "Instances");
+    tool->addButton (Button_Water, SceneUtil::Mask_Water, "Water");
+    tool->addButton (Button_Pathgrid, SceneUtil::Mask_Pathgrid, "Pathgrid");
 }
 
 void CSVRender::WorldspaceWidget::addEditModeSelectorButtons (CSVWidget::SceneToolMode *tool)
 {
     /// \todo replace EditMode with suitable subclasses
-    tool->addButton (new InstanceMode (this, tool), "object");
+    tool->addButton (new InstanceMode (this, mRootNode, tool), "object");
     tool->addButton (new PathgridMode (this, tool), "pathgrid");
 }
 
@@ -609,6 +613,8 @@ void CSVRender::WorldspaceWidget::updateOverlay()
 
 void CSVRender::WorldspaceWidget::mouseMoveEvent (QMouseEvent *event)
 {
+    dynamic_cast<CSVRender::EditMode&> (*mEditMode->getCurrent()).mouseMoveEvent (event);
+
     if (mDragging)
     {
         int diffX = event->x() - mDragX;
@@ -648,12 +654,6 @@ void CSVRender::WorldspaceWidget::mouseMoveEvent (QMouseEvent *event)
             mDragX = event->posF().x();
             mDragY = height() - event->posF().y();
 #endif
-
-            if (mDragMode == InteractionType_PrimaryEdit)
-            {
-                EditMode& editMode = dynamic_cast<CSVRender::EditMode&> (*mEditMode->getCurrent());
-                editMode.drag (event->pos(), mDragX, mDragY, mDragFactor); // note: terraintexturemode only uses pos
-            }
         }
     }
     else
@@ -702,6 +702,8 @@ void CSVRender::WorldspaceWidget::handleInteractionPress (const WorldspaceHitRes
         editMode.primarySelectPressed (hit);
     else if (type == InteractionType_SecondarySelect)
         editMode.secondarySelectPressed (hit);
+    else if (type == InteractionType_PrimaryOpen)
+        editMode.primaryOpenPressed (hit);
 }
 
 CSVRender::EditMode *CSVRender::WorldspaceWidget::getEditMode()
@@ -709,6 +711,11 @@ CSVRender::EditMode *CSVRender::WorldspaceWidget::getEditMode()
     return dynamic_cast<CSVRender::EditMode *> (mEditMode->getCurrent());
 }
 
+void CSVRender::WorldspaceWidget::primaryOpen(bool activate)
+{
+    handleInteraction(InteractionType_PrimaryOpen, activate);
+}
+
 void CSVRender::WorldspaceWidget::primaryEdit(bool activate)
 {
     handleInteraction(InteractionType_PrimaryEdit, activate);
