@@ -1,13 +1,15 @@
 #include "coloreditor.hpp"
 
 #include <QApplication>
-#include <QColor>
 #include <QColorDialog>
 #include <QDesktopWidget>
 #include <QPainter>
-#include <QRect>
 #include <QShowEvent>
 
+#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
+#include <QScreen>
+#endif
+
 #include "colorpickerpopup.hpp"
 
 CSVWidget::ColorEditor::ColorEditor(const QColor &color, QWidget *parent, const bool popupOnStart)
@@ -27,9 +29,7 @@ CSVWidget::ColorEditor::ColorEditor(QWidget *parent, const bool popupOnStart)
       mColorPicker(new ColorPickerPopup(this)),
       mPopupOnStart(popupOnStart)
 {
-    setCheckable(true);
     connect(this, SIGNAL(clicked()), this, SLOT(showPicker()));
-    connect(mColorPicker, SIGNAL(hid()), this, SLOT(pickerHid()));
     connect(mColorPicker, SIGNAL(colorChanged(const QColor &)), this, SLOT(pickerColorChanged(const QColor &)));
 }
 
@@ -85,19 +85,7 @@ void CSVWidget::ColorEditor::setColor(const int colorInt)
 
 void CSVWidget::ColorEditor::showPicker()
 {
-    if (isChecked())
-    {
-        mColorPicker->showPicker(calculatePopupPosition(), mColor);
-    }
-    else
-    {
-        mColorPicker->hide();
-    }
-}
-
-void CSVWidget::ColorEditor::pickerHid()
-{
-    setChecked(false);
+    mColorPicker->showPicker(calculatePopupPosition(), mColor);
     emit pickingFinished();
 }
 
@@ -111,7 +99,11 @@ QPoint CSVWidget::ColorEditor::calculatePopupPosition()
 {
     QRect editorGeometry = geometry();
     QRect popupGeometry = mColorPicker->geometry();
+#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
+    QRect screenGeometry = QGuiApplication::primaryScreen()->geometry();
+#else
     QRect screenGeometry = QApplication::desktop()->screenGeometry();
+#endif
 
     // Center the popup horizontally relative to the editor
     int localPopupX = (editorGeometry.width() - popupGeometry.width()) / 2;
