@@ -3,6 +3,8 @@
 
 #include "pushbutton.hpp"
 
+class QMenu;
+
 namespace CSVWidget
 {
     class SceneToolbar;
@@ -22,6 +24,14 @@ namespace CSVWidget
 
             /// Default-Implementation: do nothing
             virtual void deactivate (SceneToolbar *toolbar);
+
+            /// Add context menu items to \a menu. Default-implementation: return false
+            ///
+            /// \attention menu can be a 0-pointer
+            ///
+            /// \return Have there been any menu items to be added (if menu is 0 and there
+            /// items to be added, the function must return true anyway.
+            virtual bool createContextMenu (QMenu *menu);
     };
 }
 
