@@ -64,6 +64,8 @@ namespace CSVWorld
 
             virtual std::string getId() const;
 
+            std::string getClonedId() const;
+
             virtual std::string getIdValidatorResult() const;
 
             /// Allow subclasses to add additional data to \a command.
@@ -71,7 +73,7 @@ namespace CSVWorld
 
             /// Allow subclasses to wrap the create command together with additional commands
             /// into a macro.
-            virtual void pushCommand (std::auto_ptr<CSMWorld::CreateCommand> command,
+            virtual void pushCommand (std::unique_ptr<CSMWorld::CreateCommand> command,
                 const std::string& id);
 
             CSMWorld::Data& getData() const;
@@ -103,6 +105,8 @@ namespace CSVWorld
             virtual void cloneMode(const std::string& originId,
                                    const CSMWorld::UniversalId::Type type);
 
+            virtual void touch(const std::vector<CSMWorld::UniversalId>& ids);
+
             virtual std::string getErrors() const;
             ///< Return formatted error descriptions for the current state of the creator. if an empty
             /// string is returned, there is no error.
