@@ -32,27 +32,39 @@ std::string CSVWorld::InfoCreator::getId() const
 
 void CSVWorld::InfoCreator::configureCreateCommand (CSMWorld::CreateCommand& command) const
 {
-    int index =
-        dynamic_cast<CSMWorld::IdTable&> (*getData().getTableModel (getCollectionId())).
-        findColumnIndex (
-        getCollectionId().getType()==CSMWorld::UniversalId::Type_TopicInfos ?
-        CSMWorld::Columns::ColumnId_Topic : CSMWorld::Columns::ColumnId_Journal);
+    CSMWorld::IdTable& table = dynamic_cast<CSMWorld::IdTable&> (*getData().getTableModel (getCollectionId()));
 
-    command.addValue (index, mTopic->text());
+    if (getCollectionId() == CSMWorld::UniversalId::Type_TopicInfos)
+    {
+        command.addValue (table.findColumnIndex(CSMWorld::Columns::ColumnId_Topic), mTopic->text());
+        command.addValue (table.findColumnIndex(CSMWorld::Columns::ColumnId_Rank), -1);
+        command.addValue (table.findColumnIndex(CSMWorld::Columns::ColumnId_Gender), -1);
+        command.addValue (table.findColumnIndex(CSMWorld::Columns::ColumnId_PcRank), -1);
+    }
+    else
+    {
+        command.addValue (table.findColumnIndex(CSMWorld::Columns::ColumnId_Journal), mTopic->text());
+    }
 }
 
 CSVWorld::InfoCreator::InfoCreator (CSMWorld::Data& data, QUndoStack& undoStack,
     const CSMWorld::UniversalId& id, CSMWorld::IdCompletionManager& completionManager)
 : GenericCreator (data, undoStack, id)
 {
-    QLabel *label = new QLabel ("Topic", this);
-    insertBeforeButtons (label, false);
-
+    // Determine if we're dealing with topics or journals.
     CSMWorld::ColumnBase::Display displayType = CSMWorld::ColumnBase::Display_Topic;
+    QString labelText = "Topic";
     if (getCollectionId().getType() == CSMWorld::UniversalId::Type_JournalInfos)
     {
         displayType = CSMWorld::ColumnBase::Display_Journal;
+        labelText = "Journal";
     }
+
+    QLabel *label = new QLabel (labelText, this);
+    insertBeforeButtons (label, false);
+
+    // Add topic/journal ID input with auto-completion.
+    // Only existing topic/journal IDs are accepted so no ID validation is performed.
     mTopic = new CSVWidget::DropLineEdit(displayType, this);
     mTopic->setCompleter(completionManager.getCompleter(displayType).get());
     insertBeforeButtons (mTopic, true);
