@@ -38,7 +38,7 @@ namespace CSVWorld
         NestedTable(CSMDoc::Document& document,
                     CSMWorld::UniversalId id,
                     CSMWorld::NestedTableProxyModel* model,
-                    QWidget* parent = NULL,
+                    QWidget* parent = nullptr,
                     bool editable = true,
                     bool fixedRows = false);
 
