@@ -141,13 +141,16 @@ void CSVWorld::Table::contextMenuEvent (QContextMenuEvent *event)
 
         if (mModel->getFeatures() & CSMWorld::IdTableBase::Feature_Preview)
         {
+            const CSMWorld::UniversalId id = getUniversalId(currentRow);
+            const CSMWorld::UniversalId::Type type = id.getType();
+
             QModelIndex index = mModel->index (row,
                 mModel->findColumnIndex (CSMWorld::Columns::ColumnId_Modification));
 
             CSMWorld::RecordBase::State state = static_cast<CSMWorld::RecordBase::State> (
                 mModel->data (index).toInt());
 
-            if (state!=CSMWorld::RecordBase::State_Deleted)
+            if (state!=CSMWorld::RecordBase::State_Deleted && type != CSMWorld::UniversalId::Type_ItemLevelledList)
                 menu.addAction (mPreviewAction);
         }
     }
@@ -292,6 +295,7 @@ CSVWorld::Table::Table (const CSMWorld::UniversalId& id,
 
     mEditAction = new QAction (tr ("Edit Record"), this);
     connect (mEditAction, SIGNAL (triggered()), this, SLOT (editRecord()));
+    mEditAction->setIcon(QIcon(":edit-edit"));
     addAction (mEditAction);
     CSMPrefs::Shortcut* editShortcut = new CSMPrefs::Shortcut("table-edit", this);
     editShortcut->associateAction(mEditAction);
@@ -300,12 +304,14 @@ CSVWorld::Table::Table (const CSMWorld::UniversalId& id,
     {
         mCreateAction = new QAction (tr ("Add Record"), this);
         connect (mCreateAction, SIGNAL (triggered()), this, SIGNAL (createRequest()));
+        mCreateAction->setIcon(QIcon(":edit-add"));
         addAction (mCreateAction);
         CSMPrefs::Shortcut* createShortcut = new CSMPrefs::Shortcut("table-add", this);
         createShortcut->associateAction(mCreateAction);
 
         mCloneAction = new QAction (tr ("Clone Record"), this);
         connect(mCloneAction, SIGNAL (triggered()), this, SLOT (cloneRecord()));
+        mCloneAction->setIcon(QIcon(":edit-clone"));
         addAction(mCloneAction);
         CSMPrefs::Shortcut* cloneShortcut = new CSMPrefs::Shortcut("table-clone", this);
         cloneShortcut->associateAction(mCloneAction);
@@ -315,6 +321,7 @@ CSVWorld::Table::Table (const CSMWorld::UniversalId& id,
     {
         mTouchAction = new QAction(tr("Touch Record"), this);
         connect(mTouchAction, SIGNAL(triggered()), this, SLOT(touchRecord()));
+        mTouchAction->setIcon(QIcon(":edit-touch"));
         addAction(mTouchAction);
         CSMPrefs::Shortcut* touchShortcut = new CSMPrefs::Shortcut("table-touch", this);
         touchShortcut->associateAction(mTouchAction);
@@ -322,49 +329,56 @@ CSVWorld::Table::Table (const CSMWorld::UniversalId& id,
 
     mRevertAction = new QAction (tr ("Revert Record"), this);
     connect (mRevertAction, SIGNAL (triggered()), mDispatcher, SLOT (executeRevert()));
+    mRevertAction->setIcon(QIcon(":edit-undo"));
     addAction (mRevertAction);
     CSMPrefs::Shortcut* revertShortcut = new CSMPrefs::Shortcut("table-revert", this);
     revertShortcut->associateAction(mRevertAction);
 
     mDeleteAction = new QAction (tr ("Delete Record"), this);
     connect (mDeleteAction, SIGNAL (triggered()), mDispatcher, SLOT (executeDelete()));
+    mDeleteAction->setIcon(QIcon(":edit-delete"));
     addAction (mDeleteAction);
     CSMPrefs::Shortcut* deleteShortcut = new CSMPrefs::Shortcut("table-remove", this);
     deleteShortcut->associateAction(mDeleteAction);
 
-
     mMoveUpAction = new QAction (tr ("Move Up"), this);
     connect (mMoveUpAction, SIGNAL (triggered()), this, SLOT (moveUpRecord()));
+    mMoveUpAction->setIcon(QIcon(":record-up"));
     addAction (mMoveUpAction);
     CSMPrefs::Shortcut* moveUpShortcut = new CSMPrefs::Shortcut("table-moveup", this);
     moveUpShortcut->associateAction(mMoveUpAction);
 
     mMoveDownAction = new QAction (tr ("Move Down"), this);
     connect (mMoveDownAction, SIGNAL (triggered()), this, SLOT (moveDownRecord()));
+    mMoveDownAction->setIcon(QIcon(":record-down"));
     addAction (mMoveDownAction);
     CSMPrefs::Shortcut* moveDownShortcut = new CSMPrefs::Shortcut("table-movedown", this);
     moveDownShortcut->associateAction(mMoveDownAction);
 
     mViewAction = new QAction (tr ("View"), this);
     connect (mViewAction, SIGNAL (triggered()), this, SLOT (viewRecord()));
+    mViewAction->setIcon(QIcon(":/cell.png"));
     addAction (mViewAction);
     CSMPrefs::Shortcut* viewShortcut = new CSMPrefs::Shortcut("table-view", this);
     viewShortcut->associateAction(mViewAction);
 
     mPreviewAction = new QAction (tr ("Preview"), this);
     connect (mPreviewAction, SIGNAL (triggered()), this, SLOT (previewRecord()));
+    mPreviewAction->setIcon(QIcon(":edit-preview"));
     addAction (mPreviewAction);
     CSMPrefs::Shortcut* previewShortcut = new CSMPrefs::Shortcut("table-preview", this);
     previewShortcut->associateAction(mPreviewAction);
 
     mExtendedDeleteAction = new QAction (tr ("Extended Delete Record"), this);
     connect (mExtendedDeleteAction, SIGNAL (triggered()), this, SLOT (executeExtendedDelete()));
+    mExtendedDeleteAction->setIcon(QIcon(":edit-delete"));
     addAction (mExtendedDeleteAction);
     CSMPrefs::Shortcut* extendedDeleteShortcut = new CSMPrefs::Shortcut("table-extendeddelete", this);
     extendedDeleteShortcut->associateAction(mExtendedDeleteAction);
 
     mExtendedRevertAction = new QAction (tr ("Extended Revert Record"), this);
     connect (mExtendedRevertAction, SIGNAL (triggered()), this, SLOT (executeExtendedRevert()));
+    mExtendedRevertAction->setIcon(QIcon(":edit-undo"));
     addAction (mExtendedRevertAction);
     CSMPrefs::Shortcut* extendedRevertShortcut = new CSMPrefs::Shortcut("table-extendedrevert", this);
     extendedRevertShortcut->associateAction(mExtendedRevertAction);
@@ -723,7 +737,12 @@ void CSVWorld::Table::requestFocus (const std::string& id)
     QModelIndex index = mProxyModel->getModelIndex (id, 0);
 
     if (index.isValid())
-        scrollTo (index, QAbstractItemView::PositionAtTop);
+    {
+        // This will scroll to the row.
+        selectRow (index.row());
+        // This will actually select it.
+        selectionModel()->select (index, QItemSelectionModel::Select | QItemSelectionModel::Rows);
+    }
 }
 
 void CSVWorld::Table::recordFilterChanged (std::shared_ptr<CSMFilter::Node> filter)
