@@ -60,6 +60,10 @@ namespace CSVWorld
             void cloneRequest (const CSMWorld::UniversalId& toClone);
             void createFilterRequest(std::vector< CSMWorld::UniversalId >& types,
                                      Qt::DropAction action);
+
+        public slots:
+
+            void requestFocus (const std::string& id);
     };
 }
 
