@@ -1,17 +1,14 @@
-#include <iostream>
-
 #include <components/version/version.hpp>
 #include <components/files/configurationmanager.hpp>
 #include <components/files/escape.hpp>
+#include <components/fallback/fallback.hpp>
 #include <components/fallback/validate.hpp>
+#include <components/debug/debugging.hpp>
+#include <components/misc/rng.hpp>
 
-#include <SDL_messagebox.h>
 #include "engine.hpp"
 
-#include <boost/filesystem/fstream.hpp>
-
 #if defined(_WIN32)
-// For OutputDebugString
 #ifndef WIN32_LEAN_AND_MEAN
 #define WIN32_LEAN_AND_MEAN
 #endif
@@ -24,18 +21,6 @@
 #include <unistd.h>
 #endif
 
-#if (defined(__APPLE__) || (defined(__linux)  &&  !defined(ANDROID)) || (defined(__unix) &&  !defined(ANDROID)) || defined(__posix))
-    #define USE_CRASH_CATCHER 1
-#else
-    #define USE_CRASH_CATCHER 0
-#endif
-
-#if USE_CRASH_CATCHER
-#include <csignal>
-extern int cc_install_handlers(int argc, char **argv, int num_signals, int *sigs, const char *logfile, int (*user_info)(char*, char*));
-extern int is_debugger_attached(void);
-#endif
-
 /**
  * Workaround for problems with whitespaces in paths in older versions of Boost library
  */
@@ -148,7 +133,12 @@ bool parseOptions (int argc, char** argv, OMW::Engine& engine, Files::Configurat
         ("export-fonts", bpo::value<bool>()->implicit_value(true)
             ->default_value(false), "Export Morrowind .fnt fonts to PNG image and XML file in current directory")
 
-        ("activate-dist", bpo::value <int> ()->default_value (-1), "activation distance override");
+        ("activate-dist", bpo::value <int> ()->default_value (-1), "activation distance override")
+
+        ("random-seed", bpo::value <unsigned int> ()
+            ->default_value(Misc::Rng::generateDefaultSeed()),
+            "seed value for random number generator")
+    ;
 
     bpo::parsed_options valid_opts = bpo::command_line_parser(argc, argv)
         .options(desc).allow_unregistered().run();
@@ -202,6 +192,7 @@ bool parseOptions (int argc, char** argv, OMW::Engine& engine, Files::Configurat
 
     cfgMgr.processPaths(dataDirs);
 
+    engine.setResourceDir(variables["resources"].as<Files::EscapeHashString>().toStdString());
     engine.setDataDirs(dataDirs);
 
     // fallback archives
@@ -211,13 +202,11 @@ bool parseOptions (int argc, char** argv, OMW::Engine& engine, Files::Configurat
         engine.addArchive(*it);
     }
 
-    engine.setResourceDir(variables["resources"].as<Files::EscapeHashString>().toStdString());
-
     StringsVector content = variables["content"].as<Files::EscapeStringVector>().toStdStringVector();
     if (content.empty())
     {
-      std::cout << "No content file given (esm/esp, nor omwgame/omwaddon). Aborting..." << std::endl;
-      return false;
+        Log(Debug::Error) << "No content file given (esm/esp, nor omwgame/omwaddon). Aborting...";
+        return false;
     }
 
     StringsVector::const_iterator it(content.begin());
@@ -231,7 +220,7 @@ bool parseOptions (int argc, char** argv, OMW::Engine& engine, Files::Configurat
     engine.setCell(variables["start"].as<Files::EscapeHashString>().toStdString());
     engine.setSkipMenu (variables["skip-menu"].as<bool>(), variables["new-game"].as<bool>());
     if (!variables["skip-menu"].as<bool>() && variables["new-game"].as<bool>())
-        std::cerr << "Warning: new-game used without skip-menu -> ignoring it" << std::endl;
+        Log(Debug::Warning) << "Warning: new-game used without skip-menu -> ignoring it";
 
     // scripts
     engine.setCompileAll(variables["script-all"].as<bool>());
@@ -244,143 +233,42 @@ bool parseOptions (int argc, char** argv, OMW::Engine& engine, Files::Configurat
     engine.setSaveGameFile (variables["load-savegame"].as<Files::EscapeHashString>().toStdString());
 
     // other settings
+    Fallback::Map::init(variables["fallback"].as<FallbackMap>().mMap);
     engine.setSoundUsage(!variables["no-sound"].as<bool>());
-    engine.setFallbackValues(variables["fallback"].as<FallbackMap>().mMap);
     engine.setActivationDistanceOverride (variables["activate-dist"].as<int>());
     engine.enableFontExport(variables["export-fonts"].as<bool>());
+    engine.setRandomSeed(variables["random-seed"].as<unsigned int>());
 
     return true;
 }
 
-#if defined(_WIN32) && defined(_DEBUG)
-
-class DebugOutput : public boost::iostreams::sink
+int runApplication(int argc, char *argv[])
 {
-public:
-    std::streamsize write(const char *str, std::streamsize size)
-    {
-        // Make a copy for null termination
-        std::string tmp (str, static_cast<unsigned int>(size));
-        // Write string to Visual Studio Debug output
-        OutputDebugString (tmp.c_str ());
-        return size;
-    }
-};
-#else
-class Tee : public boost::iostreams::sink
-{
-public:
-    Tee(std::ostream &stream, std::ostream &stream2)
-        : out(stream), out2(stream2)
-    {
-    }
-
-    std::streamsize write(const char *str, std::streamsize size)
-    {
-        out.write (str, size);
-        out.flush();
-        out2.write (str, size);
-        out2.flush();
-        return size;
-    }
-
-private:
-    std::ostream &out;
-    std::ostream &out2;
-};
+#ifdef __APPLE__
+    boost::filesystem::path binary_path = boost::filesystem::system_complete(boost::filesystem::path(argv[0]));
+    boost::filesystem::current_path(binary_path.parent_path());
+    setenv("OSG_GL_TEXTURE_STORAGE", "OFF", 0);
 #endif
 
+    Files::ConfigurationManager cfgMgr;
+    std::unique_ptr<OMW::Engine> engine;
+    engine.reset(new OMW::Engine(cfgMgr));
+
+    if (parseOptions(argc, argv, *engine, cfgMgr))
+    {
+        engine->go();
+    }
+
+    return 0;
+}
+
 #ifdef ANDROID
 extern "C" int SDL_main(int argc, char**argv)
 #else
 int main(int argc, char**argv)
 #endif
 {
-#if defined(__APPLE__)
-    setenv("OSG_GL_TEXTURE_STORAGE", "OFF", 0);
-#endif
-
-    // Some objects used to redirect cout and cerr
-    // Scope must be here, so this still works inside the catch block for logging exceptions
-    std::streambuf* cout_rdbuf = std::cout.rdbuf ();
-    std::streambuf* cerr_rdbuf = std::cerr.rdbuf ();
-
-#if !(defined(_WIN32) && defined(_DEBUG))
-    boost::iostreams::stream_buffer<Tee> coutsb;
-    boost::iostreams::stream_buffer<Tee> cerrsb;
-#endif
-
-    std::ostream oldcout(cout_rdbuf);
-    std::ostream oldcerr(cerr_rdbuf);
-
-    boost::filesystem::ofstream logfile;
-
-    std::unique_ptr<OMW::Engine> engine;
-
-    int ret = 0;
-    try
-    {
-        Files::ConfigurationManager cfgMgr;
-
-#if defined(_WIN32) && defined(_DEBUG)
-        // Redirect cout and cerr to VS debug output when running in debug mode
-        boost::iostreams::stream_buffer<DebugOutput> sb;
-        sb.open(DebugOutput());
-        std::cout.rdbuf (&sb);
-        std::cerr.rdbuf (&sb);
-#else
-        // Redirect cout and cerr to openmw.log
-        logfile.open (boost::filesystem::path(cfgMgr.getLogPath() / "/openmw.log"));
-
-        coutsb.open (Tee(logfile, oldcout));
-        cerrsb.open (Tee(logfile, oldcerr));
-
-        std::cout.rdbuf (&coutsb);
-        std::cerr.rdbuf (&cerrsb);
-#endif
-
-
-#if USE_CRASH_CATCHER
-        // Unix crash catcher
-        if ((argc == 2 && strcmp(argv[1], "--cc-handle-crash") == 0) || !is_debugger_attached())
-        {
-            int s[5] = { SIGSEGV, SIGILL, SIGFPE, SIGBUS, SIGABRT };
-            cc_install_handlers(argc, argv, 5, s, (cfgMgr.getLogPath() / "crash.log").string().c_str(), NULL);
-            std::cout << "Installing crash catcher" << std::endl;
-        }
-        else
-            std::cout << "Running in a debugger, not installing crash catcher" << std::endl;
-#endif
-
-#ifdef __APPLE__
-        boost::filesystem::path binary_path = boost::filesystem::system_complete(boost::filesystem::path(argv[0]));
-        boost::filesystem::current_path(binary_path.parent_path());
-#endif
-
-        engine.reset(new OMW::Engine(cfgMgr));
-
-        if (parseOptions(argc, argv, *engine, cfgMgr))
-        {
-            engine->go();
-        }
-    }
-    catch (std::exception &e)
-    {
-#if (defined(__APPLE__) || defined(__linux) || defined(__unix) || defined(__posix))
-        if (!isatty(fileno(stdin)))
-#endif
-            SDL_ShowSimpleMessageBox(0, "OpenMW: Fatal error", e.what(), NULL);
-
-        std::cerr << "\nERROR: " << e.what() << std::endl;
-
-        ret = 1;
-    }
-
-    // Restore cout and cerr
-    std::cout.rdbuf(cout_rdbuf);
-    std::cerr.rdbuf(cerr_rdbuf);
-
-    return ret;
+    return wrapApplication(&runApplication, argc, argv, "OpenMW");
 }
 
 // Platform specific for Windows when there is no console built into the executable.
