@@ -39,6 +39,11 @@ namespace MWClass
         return "";
     }
 
+    bool CreatureLevList::hasToolTip(const MWWorld::ConstPtr& ptr) const
+    {
+        return false;
+    }
+
     void CreatureLevList::respawn(const MWWorld::Ptr &ptr) const
     {
         ensureCustomData(ptr);
@@ -56,8 +61,8 @@ namespace MWClass
             else if (creatureStats.isDead())
             {
                 const MWWorld::Store<ESM::GameSetting>& gmst = MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>();
-                static const float fCorpseRespawnDelay = gmst.find("fCorpseRespawnDelay")->getFloat();
-                static const float fCorpseClearDelay = gmst.find("fCorpseClearDelay")->getFloat();
+                static const float fCorpseRespawnDelay = gmst.find("fCorpseRespawnDelay")->mValue.getFloat();
+                static const float fCorpseClearDelay = gmst.find("fCorpseClearDelay")->mValue.getFloat();
 
                 float delay = std::min(fCorpseRespawnDelay, fCorpseClearDelay);
                 if (creatureStats.getTimeOfDeath() + delay <= MWBase::Environment::get().getWorld()->getTimeStamp())
@@ -146,19 +151,16 @@ namespace MWClass
         if (!state.mHasCustomState)
             return;
 
-        const ESM::CreatureLevListState& state2 = dynamic_cast<const ESM::CreatureLevListState&> (state);
-
         ensureCustomData(ptr);
         CreatureLevListCustomData& customData = ptr.getRefData().getCustomData()->asCreatureLevListCustomData();
-        customData.mSpawnActorId = state2.mSpawnActorId;
-        customData.mSpawn = state2.mSpawn;
+        const ESM::CreatureLevListState& levListState = state.asCreatureLevListState();
+        customData.mSpawnActorId = levListState.mSpawnActorId;
+        customData.mSpawn = levListState.mSpawn;
     }
 
     void CreatureLevList::writeAdditionalState (const MWWorld::ConstPtr& ptr, ESM::ObjectState& state)
         const
     {
-        ESM::CreatureLevListState& state2 = dynamic_cast<ESM::CreatureLevListState&> (state);
-
         if (!ptr.getRefData().getCustomData())
         {
             state.mHasCustomState = false;
@@ -166,7 +168,8 @@ namespace MWClass
         }
 
         const CreatureLevListCustomData& customData = ptr.getRefData().getCustomData()->asCreatureLevListCustomData();
-        state2.mSpawnActorId = customData.mSpawnActorId;
-        state2.mSpawn = customData.mSpawn;
+        ESM::CreatureLevListState& levListState = state.asCreatureLevListState();
+        levListState.mSpawnActorId = customData.mSpawnActorId;
+        levListState.mSpawn = customData.mSpawn;
     }
 }
