@@ -10,8 +10,10 @@ namespace MWClass
         public:
 
             virtual std::string getName (const MWWorld::ConstPtr& ptr) const;
-            ///< \return name (the one that is to be presented to the user; not the internal one);
-            /// can return an empty string.
+            ///< \return name or ID; can return an empty string.
+
+            virtual bool hasToolTip (const MWWorld::ConstPtr& ptr) const;
+            ///< @return true if this object has a tooltip when focused (default implementation: true)
 
             static void registerSelf();
     };
