@@ -1,11 +1,9 @@
 #include "dialoguemanagerimp.hpp"
 
-#include <cctype>
-#include <cstdlib>
 #include <algorithm>
-#include <iterator>
 #include <list>
-#include <iostream>
+
+#include <components/debug/debuglog.hpp>
 
 #include <components/esm/loaddial.hpp>
 #include <components/esm/loadinfo.hpp>
@@ -22,6 +20,8 @@
 #include <components/interpreter/interpreter.hpp>
 #include <components/interpreter/defines.hpp>
 
+#include <components/settings/settings.hpp>
+
 #include "../mwbase/environment.hpp"
 #include "../mwbase/world.hpp"
 #include "../mwbase/journal.hpp"
@@ -50,8 +50,7 @@ namespace MWDialogue
     DialogueManager::DialogueManager (const Compiler::Extensions& extensions, Translation::Storage& translationDataStorage) :
       mTranslationDataStorage(translationDataStorage)
       , mCompilerContext (MWScript::CompilerContext::Type_Dialogue)
-      , mErrorStream(std::cout.rdbuf())
-      , mErrorHandler(mErrorStream)
+      , mErrorHandler()
       , mTalkedTo(false)
       , mTemporaryDispositionChange(0.f)
       , mPermanentDispositionChange(0.f)
@@ -94,9 +93,6 @@ namespace MWDialogue
                 topicId = mTranslationDataStorage.topicStandardForm(topicId);
             }
 
-            if (tok->isImplicitKeyword() && mTranslationDataStorage.hasTranslation())
-                continue;
-
             if (mActorKnownTopics.count( topicId ))
                 mKnownTopics.insert( topicId );
         }
@@ -201,16 +197,13 @@ namespace MWDialogue
         }
         catch (const std::exception& error)
         {
-            std::cerr << std::string ("Dialogue error: An exception has been thrown: ") + error.what() << std::endl;
+            Log(Debug::Error) << std::string ("Dialogue error: An exception has been thrown: ") + error.what();
             success = false;
         }
 
         if (!success)
         {
-            std::cerr
-                << "Warning: compiling failed (dialogue script)" << std::endl
-                << cmd
-                << std::endl << std::endl;
+            Log(Debug::Error) << "Error: compiling failed (dialogue script): \n" << cmd << "\n";
         }
 
         return success;
@@ -230,7 +223,7 @@ namespace MWDialogue
             }
             catch (const std::exception& error)
             {
-                std::cerr << std::string ("Dialogue error: An exception has been thrown: ") + error.what() << std::endl;
+               Log(Debug::Error) << std::string ("Dialogue error: An exception has been thrown: ") + error.what();
             }
         }
     }
@@ -260,7 +253,7 @@ namespace MWDialogue
                 const MWWorld::Store<ESM::GameSetting>& gmsts =
                     MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>();
 
-                title = gmsts.find (modifiedTopic)->getString();
+                title = gmsts.find (modifiedTopic)->mValue.getString();
             }
             else
                 title = topic;
@@ -432,7 +425,6 @@ namespace MWDialogue
     void DialogueManager::addChoice (const std::string& text, int choice)
     {
         mIsInChoice = true;
-
         mChoices.push_back(std::make_pair(text, choice));
     }
 
@@ -509,9 +501,11 @@ namespace MWDialogue
         return static_cast<int>(mTemporaryDispositionChange);
     }
 
-    void DialogueManager::applyDispositionChange(int delta)
+    void DialogueManager::applyBarterDispositionChange(int delta)
     {
         mTemporaryDispositionChange += delta;
+        if (Settings::Manager::getBool("barter disposition change is permanent", "Game"))
+            mPermanentDispositionChange += delta;
     }
 
     bool DialogueManager::checkServiceRefused(ResponseCallback* callback)
@@ -535,7 +529,7 @@ namespace MWDialogue
 
             MWScript::InterpreterContext interpreterContext(&mActor.getRefData().getLocals(),mActor);
 
-            callback->addResponse(gmsts.find ("sServiceRefusal")->getString(), Interpreter::fixDefinesDialog(info->mResponse, interpreterContext));
+            callback->addResponse(gmsts.find ("sServiceRefusal")->mValue.getString(), Interpreter::fixDefinesDialog(info->mResponse, interpreterContext));
 
             executeScript (info->mResultScript, mActor);
             return true;
@@ -546,7 +540,7 @@ namespace MWDialogue
     void DialogueManager::say(const MWWorld::Ptr &actor, const std::string &topic)
     {
         MWBase::SoundManager *sndMgr = MWBase::Environment::get().getSoundManager();
-        if(!sndMgr->sayDone(actor))
+        if(sndMgr->sayActive(actor))
         {
             // Actor is already saying something.
             return;
@@ -570,7 +564,7 @@ namespace MWDialogue
         const MWMechanics::CreatureStats& creatureStats = actor.getClass().getCreatureStats(actor);
         Filter filter(actor, 0, creatureStats.hasTalkedToPlayer());
         const ESM::DialInfo *info = filter.search(*dial, false);
-        if(info != NULL)
+        if(info != nullptr)
         {
             MWBase::WindowManager *winMgr = MWBase::Environment::get().getWindowManager();
             if(winMgr->getSubtitlesEnabled())
