@@ -9,12 +9,12 @@ namespace MWGui
     /**
      * @brief A variant of MyGUI::ImageBox with aspect ratio correction using black bars
      */
-    class BackgroundImage : public MyGUI::ImageBox
+    class BackgroundImage final : public MyGUI::ImageBox
     {
     MYGUI_RTTI_DERIVED(BackgroundImage)
 
     public:
-        BackgroundImage() : mChild(NULL), mAspect(0) {}
+        BackgroundImage() : mChild(nullptr), mAspect(0) {}
 
         /**
          * @param fixedRatio Use a fixed ratio of 4:3, regardless of the image dimensions
@@ -22,8 +22,8 @@ namespace MWGui
          */
         void setBackgroundImage (const std::string& image, bool fixedRatio=true, bool stretch=true);
 
-        virtual void setSize (const MyGUI::IntSize &_value);
-        virtual void setCoord (const MyGUI::IntCoord &_value);
+        void setSize (const MyGUI::IntSize &_value) final;
+        void setCoord (const MyGUI::IntCoord &_value) final;
 
     private:
         MyGUI::ImageBox* mChild;
