@@ -6,19 +6,20 @@
 #include "../mwbase/environment.hpp"
 #include "../mwbase/world.hpp"
 #include "../mwbase/windowmanager.hpp"
-#include "../mwbase/dialoguemanager.hpp"
 #include "../mwbase/mechanicsmanager.hpp"
+#include "../mwbase/scriptmanager.hpp"
 
 #include "../mwworld/class.hpp"
 #include "../mwworld/inventorystore.hpp"
 
 #include "../mwmechanics/creaturestats.hpp"
 
+#include "../mwscript/interpretercontext.hpp"
+
 #include "countdialog.hpp"
 #include "inventorywindow.hpp"
 
 #include "itemview.hpp"
-#include "itemwidget.hpp"
 #include "inventoryitemmodel.hpp"
 #include "containeritemmodel.hpp"
 #include "sortfilteritemmodel.hpp"
@@ -31,8 +32,8 @@ namespace MWGui
     ContainerWindow::ContainerWindow(DragAndDrop* dragAndDrop)
         : WindowBase("openmw_container_window.layout")
         , mDragAndDrop(dragAndDrop)
-        , mSortModel(NULL)
-        , mModel(NULL)
+        , mSortModel(nullptr)
+        , mModel(nullptr)
         , mSelectedItem(-1)
     {
         getWidget(mDisposeCorpseButton, "DisposeCorpseButton");
@@ -52,7 +53,7 @@ namespace MWGui
 
     void ContainerWindow::onItemSelected(int index)
     {
-        if (mDragAndDrop->mIsOnDragAndDrop && mModel)
+        if (mDragAndDrop->mIsOnDragAndDrop)
         {
             dropItem();
             return;
@@ -83,11 +84,14 @@ namespace MWGui
             dialog->eventOkClicked += MyGUI::newDelegate(this, &ContainerWindow::dragItem);
         }
         else
-            dragItem (NULL, count);
+            dragItem (nullptr, count);
     }
 
     void ContainerWindow::dragItem(MyGUI::Widget* sender, int count)
     {
+        if (!mModel)
+            return;
+
         if (!onTakeItem(mModel->getItem(mSelectedItem), count))
             return;
 
@@ -96,6 +100,9 @@ namespace MWGui
 
     void ContainerWindow::dropItem()
     {
+        if (!mModel)
+            return;
+
         bool success = mModel->onDropItem(mDragAndDrop->mItem.mBase, mDragAndDrop->mDraggedCount);
 
         if (success)
@@ -104,7 +111,7 @@ namespace MWGui
 
     void ContainerWindow::onBackgroundSelected()
     {
-        if (mDragAndDrop->mIsOnDragAndDrop && mModel)
+        if (mDragAndDrop->mIsOnDragAndDrop)
             dropItem();
     }
 
@@ -145,9 +152,9 @@ namespace MWGui
     void ContainerWindow::resetReference()
     {
         ReferenceInterface::resetReference();
-        mItemView->setModel(NULL);
-        mModel = NULL;
-        mSortModel = NULL;
+        mItemView->setModel(nullptr);
+        mModel = nullptr;
+        mSortModel = nullptr;
     }
 
     void ContainerWindow::onClose()
@@ -156,6 +163,9 @@ namespace MWGui
 
         if (mModel)
             mModel->onClose();
+
+        if (!mPtr.isEmpty())
+            MWBase::Environment::get().getMechanicsManager()->onClose(mPtr);
     }
 
     void ContainerWindow::onCloseButtonClicked(MyGUI::Widget* _sender)
@@ -165,9 +175,11 @@ namespace MWGui
 
     void ContainerWindow::onTakeAllButtonClicked(MyGUI::Widget* _sender)
     {
-        if(mDragAndDrop != NULL && mDragAndDrop->mIsOnDragAndDrop)
+        if(mDragAndDrop != nullptr && mDragAndDrop->mIsOnDragAndDrop)
             return;
 
+        MWBase::Environment::get().getWindowManager()->setKeyFocusWidget(mCloseButton);
+
         // transfer everything into the player's inventory
         ItemModel* playerModel = MWBase::Environment::get().getWindowManager()->getInventoryWindow()->getModel();
         mModel->update();
@@ -211,14 +223,41 @@ namespace MWGui
 
     void ContainerWindow::onDisposeCorpseButtonClicked(MyGUI::Widget *sender)
     {
-        if(mDragAndDrop == NULL || !mDragAndDrop->mIsOnDragAndDrop)
+        if(mDragAndDrop == nullptr || !mDragAndDrop->mIsOnDragAndDrop)
         {
+            MWBase::Environment::get().getWindowManager()->setKeyFocusWidget(mCloseButton);
+
             onTakeAllButtonClicked(mTakeButton);
 
             if (mPtr.getClass().isPersistent(mPtr))
                 MWBase::Environment::get().getWindowManager()->messageBox("#{sDisposeCorpseFail}");
             else
+            {
+                MWMechanics::CreatureStats& creatureStats = mPtr.getClass().getCreatureStats(mPtr);
+
+                // If we dispose corpse before end of death animation, we should update death counter counter manually.
+                // Also we should run actor's script - it may react on actor's death.
+                if (creatureStats.isDead() && !creatureStats.isDeathAnimationFinished())
+                {
+                    creatureStats.setDeathAnimationFinished(true);
+                    MWBase::Environment::get().getMechanicsManager()->notifyDied(mPtr);
+
+                    const std::string script = mPtr.getClass().getScript(mPtr);
+                    if (!script.empty() && MWBase::Environment::get().getWorld()->getScriptsEnabled())
+                    {
+                        MWScript::InterpreterContext interpreterContext (&mPtr.getRefData().getLocals(), mPtr);
+                        MWBase::Environment::get().getScriptManager()->run (script, interpreterContext);
+                    }
+
+                    // Clean up summoned creatures as well
+                    std::map<MWMechanics::CreatureStats::SummonKey, int>& creatureMap = creatureStats.getSummonedCreatureMap();
+                    for (const auto& creature : creatureMap)
+                        MWBase::Environment::get().getMechanicsManager()->cleanupSummonedCreature(mPtr, creature.second);
+                    creatureMap.clear();
+                }
+
                 MWBase::Environment::get().getWorld()->deleteObject(mPtr);
+            }
 
             mPtr = MWWorld::Ptr();
         }
