@@ -9,6 +9,9 @@
 #include "../mwworld/class.hpp"
 #include "../mwworld/inventorystore.hpp"
 
+#include "../mwbase/environment.hpp"
+#include "../mwbase/mechanicsmanager.hpp"
+
 namespace MWGui
 {
 
@@ -34,20 +37,20 @@ size_t InventoryItemModel::getItemCount()
 ItemModel::ModelIndex InventoryItemModel::getIndex (ItemStack item)
 {
     size_t i = 0;
-    for (std::vector<ItemStack>::iterator it = mItems.begin(); it != mItems.end(); ++it)
+    for (ItemStack& itemStack : mItems)
     {
-        if (*it == item)
+        if (itemStack == item)
             return i;
         ++i;
     }
     return -1;
 }
 
-MWWorld::Ptr InventoryItemModel::copyItem (const ItemStack& item, size_t count, bool setNewOwner)
+MWWorld::Ptr InventoryItemModel::copyItem (const ItemStack& item, size_t count, bool allowAutoEquip)
 {
     if (item.mBase.getContainerStore() == &mActor.getClass().getContainerStore(mActor))
         throw std::runtime_error("Item to copy needs to be from a different container!");
-    return *mActor.getClass().getContainerStore(mActor).add(item.mBase, count, mActor, setNewOwner);
+    return *mActor.getClass().getContainerStore(mActor).add(item.mBase, count, mActor, allowAutoEquip);
 }
 
 void InventoryItemModel::removeItem (const ItemStack& item, size_t count)
@@ -85,7 +88,7 @@ MWWorld::Ptr InventoryItemModel::moveItem(const ItemStack &item, size_t count, I
     if (item.mFlags & ItemStack::Flag_Bound)
         return MWWorld::Ptr();
 
-    MWWorld::Ptr ret = otherModel->copyItem(item, count, false);
+    MWWorld::Ptr ret = otherModel->copyItem(item, count);
     removeItem(item, count);
     return ret;
 }
@@ -116,4 +119,16 @@ void InventoryItemModel::update()
     }
 }
 
+bool InventoryItemModel::onTakeItem(const MWWorld::Ptr &item, int count)
+{
+    // Looting a dead corpse is considered OK
+    if (mActor.getClass().isActor() && mActor.getClass().getCreatureStats(mActor).isDead())
+        return true;
+
+    MWWorld::Ptr player = MWMechanics::getPlayer();
+    MWBase::Environment::get().getMechanicsManager()->itemTaken(player, item, mActor, count);
+
+    return true;
+}
+
 }
