@@ -1,13 +1,9 @@
 #include "itemmodel.hpp"
 
-#include <set>
-
 #include "../mwworld/class.hpp"
 #include "../mwworld/containerstore.hpp"
-#include "../mwworld/store.hpp"
 #include "../mwworld/esmstore.hpp"
 
-#include "../mwbase/world.hpp"
 #include "../mwbase/environment.hpp"
 #include "../mwbase/mechanicsmanager.hpp"
 
@@ -31,7 +27,7 @@ namespace MWGui
     ItemStack::ItemStack()
         : mType(Type_Normal)
         , mFlags(0)
-        , mCreator(NULL)
+        , mCreator(nullptr)
         , mCount(0)
     {
     }
@@ -106,7 +102,7 @@ namespace MWGui
 
 
     ProxyItemModel::ProxyItemModel()
-        : mSourceModel(NULL)
+        : mSourceModel(nullptr)
     {
     }
 
@@ -120,9 +116,9 @@ namespace MWGui
         return mSourceModel->allowedToUseItems();
     }
 
-    MWWorld::Ptr ProxyItemModel::copyItem (const ItemStack& item, size_t count, bool setNewOwner)
+    MWWorld::Ptr ProxyItemModel::copyItem (const ItemStack& item, size_t count, bool allowAutoEquip)
     {
-        return mSourceModel->copyItem (item, count, setNewOwner);
+        return mSourceModel->copyItem (item, count, allowAutoEquip);
     }
 
     void ProxyItemModel::removeItem (const ItemStack& item, size_t count)
@@ -167,7 +163,7 @@ namespace MWGui
         if (mSourceModel)
         {
             delete mSourceModel;
-            mSourceModel = NULL;
+            mSourceModel = nullptr;
         }
 
         mSourceModel = sourceModel;
