@@ -1,7 +1,5 @@
 #include "journalbooks.hpp"
 
-#include <MyGUI_LanguageManager.h>
-
 #include "../mwbase/environment.hpp"
 #include "../mwbase/windowmanager.hpp"
 
@@ -157,7 +155,7 @@ MWGui::BookTypesetter::Utf8Span to_utf8_span (char const * text)
 typedef TypesetBook::Ptr book;
 
 JournalBooks::JournalBooks (JournalViewModel::Ptr model, ToUTF8::FromType encoding) :
-    mModel (model), mEncoding(encoding)
+    mModel (model), mEncoding(encoding), mIndexPagesCount(0)
 {
 }
 
@@ -229,10 +227,13 @@ book JournalBooks::createTopicIndexBook ()
 
 BookTypesetter::Ptr JournalBooks::createLatinJournalIndex ()
 {
-    BookTypesetter::Ptr typesetter = BookTypesetter::create (92, 250);
+    BookTypesetter::Ptr typesetter = BookTypesetter::create (92, 260);
 
     typesetter->setSectionAlignment (BookTypesetter::AlignCenter);
 
+    // Latin journal index always has two columns for now.
+    mIndexPagesCount = 2;
+
     char ch = 'A';
 
     BookTypesetter::Style* body = typesetter->createStyle ("", MyGUI::Colour::Black);
@@ -260,12 +261,23 @@ BookTypesetter::Ptr JournalBooks::createLatinJournalIndex ()
 
 BookTypesetter::Ptr JournalBooks::createCyrillicJournalIndex ()
 {
-    BookTypesetter::Ptr typesetter = BookTypesetter::create (92, 250);
+    BookTypesetter::Ptr typesetter = BookTypesetter::create (92, 260);
 
     typesetter->setSectionAlignment (BookTypesetter::AlignCenter);
 
     BookTypesetter::Style* body = typesetter->createStyle ("", MyGUI::Colour::Black);
 
+    int fontHeight = MWBase::Environment::get().getWindowManager()->getFontHeight();
+
+    // for small font size split alphabet to two columns (2x15 characers), for big font size split it to three colums (3x10 characters).
+    int sectionBreak = 10;
+    mIndexPagesCount = 3;
+    if (fontHeight < 18)
+    {
+        sectionBreak = 15;
+        mIndexPagesCount = 2;
+    }
+
     unsigned char ch[2] = {0xd0, 0x90}; // CYRILLIC CAPITAL A is a 0xd090 in UTF-8
 
     for (int i = 0; i < 32; ++i)
@@ -287,7 +299,7 @@ BookTypesetter::Ptr JournalBooks::createCyrillicJournalIndex ()
         if (i == 26 || i == 28)
             continue;
 
-        if (i == 15)
+        if (i % sectionBreak == 0)
             typesetter->sectionBreak ();
 
         typesetter->write (style, to_utf8_span (buffer));
