@@ -6,6 +6,8 @@
 #include <functional>
 #include <stdint.h>
 
+#include <components/misc/utf8stream.hpp>
+
 namespace MWGui
 {
     /// View-Model for the journal GUI
@@ -37,6 +39,8 @@ namespace MWGui
             /// and end of the span relative to the body, and a valid topic ID if
             /// the span represents a keyword, or zero if not.
             virtual void visitSpans (std::function <void (TopicId, size_t, size_t)> visitor) const = 0;
+
+            virtual ~Entry() = default;
         };
 
         /// An interface to topic data.
@@ -45,6 +49,8 @@ namespace MWGui
             /// Returns a pre-formatted span of UTF8 encoded text representing
             /// the name of the NPC this portion of dialog was heard from.
             virtual Utf8Span source () const = 0;
+
+            virtual ~TopicEntry() = default;
         };
 
         /// An interface to journal data.
@@ -53,8 +59,9 @@ namespace MWGui
             /// Returns a pre-formatted span of UTF8 encoded text representing
             /// the in-game date this entry was added to the journal.
             virtual Utf8Span timestamp () const = 0;
-        };
 
+            virtual ~JournalEntry() = default;
+        };
 
         /// called prior to journal opening
         virtual void load () = 0;
@@ -75,14 +82,16 @@ namespace MWGui
         /// provides the name of the topic specified by its id
         virtual void visitTopicName (TopicId topicId, std::function <void (Utf8Span)> visitor) const = 0;
 
-        /// walks over the topics whose names start with the specified character providing the topics name
-        virtual void visitTopicNamesStartingWith (char character, std::function < void (const std::string&) > visitor) const = 0;
+        /// walks over the topics whose names start with the character
+        virtual void visitTopicNamesStartingWith (Utf8Stream::UnicodeChar character, std::function < void (const std::string&) > visitor) const = 0;
 
         /// walks over the topic entries for the topic specified by its identifier
         virtual void visitTopicEntries (TopicId topicId, std::function <void (TopicEntry const &)> visitor) const = 0;
 
         // create an instance of the default journal view model implementation
         static Ptr create ();
+
+        virtual ~JournalViewModel() = default;
     };
 }
 
