@@ -6,6 +6,8 @@
 #include <MyGUI_Gui.h>
 #include <MyGUI_Window.h>
 
+#include <components/debug/debuglog.hpp>
+
 #include "../mwbase/windowmanager.hpp"
 #include "../mwbase/environment.hpp"
 
@@ -14,15 +16,14 @@ namespace MWGui
 
 bool shouldAcceptKeyFocus(MyGUI::Widget* w)
 {
-    if (w && w->getUserString("IgnoreTabKey") == "y")
-        return false;
-
     return w && !w->castType<MyGUI::Window>(false) && w->getInheritedEnabled() && w->getInheritedVisible() && w->getVisible() && w->getEnabled();
 }
 
 /// Recursively get all child widgets that accept keyboard input
 void getKeyFocusWidgets(MyGUI::Widget* parent, std::vector<MyGUI::Widget*>& results)
 {
+    assert(parent != nullptr);
+
     if (!parent->getVisible() || !parent->getEnabled())
         return;
 
@@ -49,7 +50,14 @@ KeyboardNavigation::KeyboardNavigation()
 
 KeyboardNavigation::~KeyboardNavigation()
 {
-    MyGUI::WidgetManager::getInstance().unregisterUnlinker(this);
+    try
+    {
+        MyGUI::WidgetManager::getInstance().unregisterUnlinker(this);
+    }
+    catch(const MyGUI::Exception& e)
+    {
+        Log(Debug::Error) << "Error in the destructor: " << e.what();
+    }
 }
 
 void KeyboardNavigation::saveFocus(int mode)
@@ -108,6 +116,12 @@ void KeyboardNavigation::onFrame()
     if (!mEnabled)
         return;
 
+    if (!MWBase::Environment::get().getWindowManager()->isGuiMode())
+    {
+        MWBase::Environment::get().getWindowManager()->setKeyFocusWidget(nullptr);
+        return;
+    }
+
     MyGUI::Widget* focus = MyGUI::InputManager::getInstance().getKeyFocusWidget();
 
     if (focus == mCurrentFocus)
@@ -172,7 +186,7 @@ enum Direction
     D_Prev
 };
 
-bool KeyboardNavigation::injectKeyPress(MyGUI::KeyCode key, unsigned int text)
+bool KeyboardNavigation::injectKeyPress(MyGUI::KeyCode key, unsigned int text, bool repeat)
 {
     if (!mEnabled)
         return false;
@@ -192,7 +206,14 @@ bool KeyboardNavigation::injectKeyPress(MyGUI::KeyCode key, unsigned int text)
     case MyGUI::KeyCode::Return:
     case MyGUI::KeyCode::NumpadEnter:
     case MyGUI::KeyCode::Space:
+    {
+        // We should disable repeating for activation keys
+        MyGUI::InputManager::getInstance().injectKeyRelease(MyGUI::KeyCode::None);
+        if (repeat)
+            return true;
+
         return accept();
+    }
     default:
         return false;
     }
@@ -200,6 +221,9 @@ bool KeyboardNavigation::injectKeyPress(MyGUI::KeyCode key, unsigned int text)
 
 bool KeyboardNavigation::switchFocus(int direction, bool wrap)
 {
+    if (!MWBase::Environment::get().getWindowManager()->isGuiMode())
+        return false;
+
     MyGUI::Widget* focus = MyGUI::InputManager::getInstance().getKeyFocusWidget();
 
     bool isCycle = (direction == D_Prev || direction == D_Next);
