@@ -1,6 +1,8 @@
 #ifndef MWGUI_LOADINGSCREEN_H
 #define MWGUI_LOADINGSCREEN_H
 
+#include <memory>
+
 #include <osg/Timer>
 #include <osg/ref_ptr>
 
@@ -34,8 +36,8 @@ namespace MWGui
         virtual ~LoadingScreen();
 
         /// Overridden from Loading::Listener, see the Loading::Listener documentation for usage details
-        virtual void setLabel (const std::string& label, bool important);
-        virtual void loadingOn();
+        virtual void setLabel (const std::string& label, bool important, bool center);
+        virtual void loadingOn(bool visible=true);
         virtual void loadingOff();
         virtual void setProgressRange (size_t range);
         virtual void setProgress (size_t value);
@@ -63,6 +65,8 @@ namespace MWGui
 
         bool mImportantLabel;
 
+        bool mVisible;
+
         size_t mProgress;
 
         bool mShowWallpaper;
