@@ -28,6 +28,8 @@ namespace MWGui
             bool createInteractiveMessageBox (const std::string& message, const std::vector<std::string>& buttons);
             bool isInteractiveMessageBox ();
 
+            int getMessagesCount();
+
             const InteractiveMessageBox* getInteractiveMessageBox() const { return mInterMessageBoxe; }
 
             /// Remove all message boxes
@@ -81,7 +83,7 @@ namespace MWGui
 
             MyGUI::Widget* getDefaultKeyFocus() override;
 
-            virtual bool exit() { return false; }
+            virtual bool exit() override { return false; }
 
             bool mMarkedToDelete;
 
