@@ -12,6 +12,8 @@
 #include <osgDB/ReadFile>
 #include <osg/Texture2D>
 
+#include <components/debug/debuglog.hpp>
+
 #include <components/myguiplatform/myguitexture.hpp>
 
 #include <components/misc/stringops.hpp>
@@ -35,8 +37,8 @@ namespace MWGui
     SaveGameDialog::SaveGameDialog()
         : WindowModal("openmw_savegame_dialog.layout")
         , mSaving(true)
-        , mCurrentCharacter(NULL)
-        , mCurrentSlot(NULL)
+        , mCurrentCharacter(nullptr)
+        , mCurrentSlot(nullptr)
     {
         getWidget(mScreenshot, "Screenshot");
         getWidget(mCharacterSelection, "SelectCharacter");
@@ -97,7 +99,7 @@ namespace MWGui
         if (mSaveList->getItemCount() == 0)
         {
             size_t previousIndex = mCharacterSelection->getIndexSelected();
-            mCurrentCharacter = NULL;
+            mCurrentCharacter = nullptr;
             mCharacterSelection->removeItemAt(previousIndex);
             if (mCharacterSelection->getItemCount())
             {
@@ -125,6 +127,9 @@ namespace MWGui
     void SaveGameDialog::onEditSelectAccept(MyGUI::EditBox *sender)
     {
         accept();
+
+        // To do not spam onEditSelectAccept() again and again
+        MWBase::Environment::get().getWindowManager()->injectKeyRelease(MyGUI::KeyCode::None);
     }
 
     void SaveGameDialog::onOpen()
@@ -141,8 +146,8 @@ namespace MWGui
 
         mCharacterSelection->setCaption("");
         mCharacterSelection->removeAllItems();
-        mCurrentCharacter = NULL;
-        mCurrentSlot = NULL;
+        mCurrentCharacter = nullptr;
+        mCurrentSlot = nullptr;
         mSaveList->removeAllItems();
         onSlotSelected(mSaveList, MyGUI::ITEM_NONE);
 
@@ -244,13 +249,10 @@ namespace MWGui
 
     void SaveGameDialog::accept(bool reallySure)
     {
-        // Remove for MyGUI 3.2.2
-        MWBase::Environment::get().getWindowManager()->setKeyFocusWidget(NULL);
-
         if (mSaving)
         {
             // If overwriting an existing slot, ask for confirmation first
-            if (mCurrentSlot != NULL && !reallySure)
+            if (mCurrentSlot != nullptr && !reallySure)
             {
                 ConfirmationDialog* dialog = MWBase::Environment::get().getWindowManager()->getConfirmationDialog();
                 dialog->askForConfirmation("#{sMessage4}");
@@ -313,7 +315,7 @@ namespace MWGui
         MWBase::StateManager* mgr = MWBase::Environment::get().getStateManager();
 
         unsigned int i=0;
-        const MWState::Character* character = NULL;
+        const MWState::Character* character = nullptr;
         for (MWBase::StateManager::CharacterIterator it = mgr->characterBegin(); it != mgr->characterEnd(); ++it, ++i)
         {
             if (i == pos)
@@ -322,7 +324,7 @@ namespace MWGui
         assert(character && "Can't find selected character");
 
         mCurrentCharacter = character;
-        mCurrentSlot = NULL;
+        mCurrentSlot = nullptr;
         fillSaveList();
     }
 
@@ -374,7 +376,7 @@ namespace MWGui
 
         if (pos == MyGUI::ITEM_NONE || !mCurrentCharacter)
         {
-            mCurrentSlot = NULL;
+            mCurrentSlot = nullptr;
             mInfoText->setCaption("");
             mScreenshot->setImageTexture("");
             return;
@@ -383,7 +385,7 @@ namespace MWGui
         if (mSaving)
             mSaveNameEdit->setCaption(sender->getItemNameAt(pos));
 
-        mCurrentSlot = NULL;
+        mCurrentSlot = nullptr;
         unsigned int i=0;
         for (MWState::Character::SlotIterator it = mCurrentCharacter->begin(); it != mCurrentCharacter->end(); ++it, ++i)
         {
@@ -398,17 +400,7 @@ namespace MWGui
         struct tm* timeinfo;
         timeinfo = localtime(&time);
 
-        // Use system/environment locale settings for datetime formatting
-        char* oldLctime = setlocale(LC_TIME, NULL);
-        setlocale(LC_TIME, "");
-
-        const int size=1024;
-        char buffer[size];
-        if (std::strftime(buffer, size, "%x %X", timeinfo) > 0)
-            text << buffer << "\n";
-
-        // reset
-        setlocale(LC_TIME, oldLctime);
+        text << std::put_time(timeinfo, "%Y.%m.%d %T") << "\n";
 
         text << "#{sLevel} " << mCurrentSlot->mProfile.mPlayerLevel << "\n";
         text << "#{sCell=" << mCurrentSlot->mProfile.mPlayerCell << "}\n";
@@ -438,14 +430,14 @@ namespace MWGui
         osgDB::ReaderWriter* readerwriter = osgDB::Registry::instance()->getReaderWriterForExtension("jpg");
         if (!readerwriter)
         {
-            std::cerr << "Error: Can't open savegame screenshot, no jpg readerwriter found" << std::endl;
+            Log(Debug::Error) << "Error: Can't open savegame screenshot, no jpg readerwriter found";
             return;
         }
 
         osgDB::ReaderWriter::ReadResult result = readerwriter->readImage(instream);
         if (!result.success())
         {
-            std::cerr << "Error: Failed to read savegame screenshot: " << result.message() << " code " << result.status() << std::endl;
+            Log(Debug::Error) << "Error: Failed to read savegame screenshot: " << result.message() << " code " << result.status();
             return;
         }
 
