@@ -6,6 +6,9 @@
 #include <MyGUI_Gui.h>
 
 #include <components/widgets/sharedstatebutton.hpp>
+#include <components/widgets/box.hpp>
+
+#include "tooltips.hpp"
 
 namespace MWGui
 {
@@ -21,7 +24,7 @@ namespace MWGui
     }
 
     SpellView::SpellView()
-        : mScrollView(NULL)
+        : mScrollView(nullptr)
         , mShowCostColumn(true)
         , mHighlightSelected(true)
     {
@@ -32,7 +35,7 @@ namespace MWGui
         Base::initialiseOverride();
 
         assignWidget(mScrollView, "ScrollView");
-        if (mScrollView == NULL)
+        if (mScrollView == nullptr)
             throw std::runtime_error("Item view needs a scroll view");
 
         mScrollView->setCanvasAlign(MyGUI::Align::Left | MyGUI::Align::Top);
@@ -103,11 +106,12 @@ namespace MWGui
             }
 
             const std::string skin = spell.mActive ? "SandTextButton" : "SpellTextUnequipped";
+            const std::string captionSuffix = MWGui::ToolTips::getCountString(spell.mCount);
 
             Gui::SharedStateButton* t = mScrollView->createWidget<Gui::SharedStateButton>(skin,
                 MyGUI::IntCoord(0, 0, 0, spellHeight), MyGUI::Align::Left | MyGUI::Align::Top);
             t->setNeedKeyFocus(true);
-            t->setCaption(spell.mName);
+            t->setCaption(spell.mName + captionSuffix);
             t->setTextAlign(MyGUI::Align::Left);
             adjustSpellWidget(spell, i, t);
 
@@ -127,7 +131,7 @@ namespace MWGui
                 mLines.push_back(LineInfo(t, costChance, i));
             }
             else
-                mLines.push_back(LineInfo(t, (MyGUI::Widget*)NULL, i));
+                mLines.push_back(LineInfo(t, (MyGUI::Widget*)nullptr, i));
 
             t->setStateSelected(spell.mSelected);
         }
@@ -145,13 +149,13 @@ namespace MWGui
         mModel->update();
         bool fullUpdateRequired = false;
         SpellModel::ModelIndex maxSpellIndexFound = -1;
-        for (std::vector< LineInfo >::iterator it = mLines.begin(); it != mLines.end(); ++it)
+        for (LineInfo& line : mLines)
         {
             // only update the lines that are "updateable"
-            SpellModel::ModelIndex spellIndex(it->mSpellIndex);
+            SpellModel::ModelIndex spellIndex(line.mSpellIndex);
             if (spellIndex != NoSpellIndex)
             {
-                Gui::SharedStateButton* nameButton = reinterpret_cast<Gui::SharedStateButton*>(it->mLeftWidget);
+                Gui::SharedStateButton* nameButton = reinterpret_cast<Gui::SharedStateButton*>(line.mLeftWidget);
 
                 // match model against line
                 // if don't match, then major change has happened, so do a full update
@@ -163,7 +167,8 @@ namespace MWGui
 
                 // more checking for major change.
                 const Spell& spell = mModel->getItem(spellIndex);
-                if (nameButton->getCaption() != spell.mName)
+                const std::string captionSuffix = MWGui::ToolTips::getCountString(spell.mCount);
+                if (nameButton->getCaption() != (spell.mName + captionSuffix))
                 {
                     fullUpdateRequired = true;
                     break;
@@ -171,8 +176,8 @@ namespace MWGui
                 else
                 {
                     maxSpellIndexFound = spellIndex;
-                    Gui::SharedStateButton* costButton = reinterpret_cast<Gui::SharedStateButton*>(it->mRightWidget);
-                    if ((costButton != NULL) && (costButton->getCaption() != spell.mCostColumn))
+                    Gui::SharedStateButton* costButton = reinterpret_cast<Gui::SharedStateButton*>(line.mRightWidget);
+                    if ((costButton != nullptr) && (costButton->getCaption() != spell.mCostColumn))
                     {
                         costButton->setCaption(spell.mCostColumn);
                     }
@@ -193,27 +198,25 @@ namespace MWGui
     void SpellView::layoutWidgets()
     {
         int height = 0;
-        for (std::vector< LineInfo >::iterator it = mLines.begin();
-             it != mLines.end(); ++it)
+        for (LineInfo& line : mLines)
         {
-            height += (it->mLeftWidget)->getHeight();
+            height += line.mLeftWidget->getHeight();
         }
 
         bool scrollVisible = height > mScrollView->getHeight();
         int width = mScrollView->getWidth() - (scrollVisible ? 18 : 0);
 
         height = 0;
-        for (std::vector< LineInfo >::iterator it = mLines.begin();
-             it != mLines.end(); ++it)
+        for (LineInfo& line : mLines)
         {
-            int lineHeight = (it->mLeftWidget)->getHeight();
-            (it->mLeftWidget)->setCoord(4, height, width - 8, lineHeight);
-            if (it->mRightWidget)
+            int lineHeight = line.mLeftWidget->getHeight();
+            line.mLeftWidget->setCoord(4, height, width - 8, lineHeight);
+            if (line.mRightWidget)
             {
-                (it->mRightWidget)->setCoord(4, height, width - 8, lineHeight);
-                MyGUI::TextBox* second = (it->mRightWidget)->castType<MyGUI::TextBox>(false);
+                line.mRightWidget->setCoord(4, height, width - 8, lineHeight);
+                MyGUI::TextBox* second = line.mRightWidget->castType<MyGUI::TextBox>(false);
                 if (second)
-                    (it->mLeftWidget)->setSize(width - 8 - second->getTextSize().width, lineHeight);
+                    line.mLeftWidget->setSize(width - 8 - second->getTextSize().width, lineHeight);
             }
 
             height += lineHeight;
@@ -233,10 +236,10 @@ namespace MWGui
                 MyGUI::IntCoord(0, 0, mScrollView->getWidth(), 18),
                 MyGUI::Align::Left | MyGUI::Align::Top);
             separator->setNeedMouseFocus(false);
-            mLines.push_back(LineInfo(separator, (MyGUI::Widget*)NULL, NoSpellIndex));
+            mLines.push_back(LineInfo(separator, (MyGUI::Widget*)nullptr, NoSpellIndex));
         }
 
-        MyGUI::TextBox* groupWidget = mScrollView->createWidget<MyGUI::TextBox>("SandBrightText",
+        MyGUI::TextBox* groupWidget = mScrollView->createWidget<Gui::TextBox>("SandBrightText",
             MyGUI::IntCoord(0, 0, mScrollView->getWidth(), 24),
             MyGUI::Align::Left | MyGUI::Align::Top);
         groupWidget->setCaptionWithReplacing(label);
@@ -245,7 +248,7 @@ namespace MWGui
 
         if (label2 != "")
         {
-            MyGUI::TextBox* groupWidget2 = mScrollView->createWidget<MyGUI::TextBox>("SandBrightText",
+            MyGUI::TextBox* groupWidget2 = mScrollView->createWidget<Gui::TextBox>("SandBrightText",
                 MyGUI::IntCoord(0, 0, mScrollView->getWidth(), 24),
                 MyGUI::Align::Left | MyGUI::Align::Top);
             groupWidget2->setCaptionWithReplacing(label2);
@@ -255,7 +258,7 @@ namespace MWGui
             mLines.push_back(LineInfo(groupWidget, groupWidget2, NoSpellIndex));
         }
         else
-            mLines.push_back(LineInfo(groupWidget, (MyGUI::Widget*)NULL, NoSpellIndex));
+            mLines.push_back(LineInfo(groupWidget, (MyGUI::Widget*)nullptr, NoSpellIndex));
     }
 
 
