@@ -32,6 +32,8 @@ namespace MWGui
         void onEnchantedItemSelected(MWWorld::Ptr item, bool alreadyEquipped);
         void onSpellSelected(const std::string& spellId);
         void onModelIndexSelected(SpellModel::ModelIndex index);
+        void onFilterChanged(MyGUI::EditBox *sender);
+        void onDeleteClicked(MyGUI::Widget *widget);
         void onDeleteSpellAccept();
         void askDeleteSpell(const std::string& spellId);
 
@@ -41,6 +43,7 @@ namespace MWGui
 
         SpellView* mSpellView;
         SpellIcons* mSpellIcons;
+        MyGUI::EditBox* mFilterEdit;
 
     private:
         float mUpdateTimer;
