@@ -53,8 +53,6 @@ namespace MWGui
             void onWindowResize(MyGUI::Window* window);
             void onMouseWheel(MyGUI::Widget* _sender, int _rel);
 
-            static const int sLineHeight;
-
             MyGUI::Widget* mLeftPane;
             MyGUI::Widget* mRightPane;
 
@@ -71,6 +69,7 @@ namespace MWGui
             std::set<std::string> mExpelled;
 
             bool mChanged;
+            const int mMinFullWidth;
 
         protected:
             virtual void onPinToggled();
