@@ -20,7 +20,9 @@ namespace MWGui
 
         void setNextButtonShow(bool shown);
         void setTextLabel(const std::string &label);
-        virtual void open();
+        virtual void onOpen();
+
+        bool exit() { return false; }
 
         /** Event : Dialog finished, OK button clicked.\n
             signature : void method()\n
