@@ -18,13 +18,14 @@
 #include "combat.hpp"
 #include "weaponpriority.hpp"
 #include "spellpriority.hpp"
+#include "weapontype.hpp"
 
 namespace MWMechanics
 {
     float suggestCombatRange(int rangeTypes)
     {
-        static const float fCombatDistance = MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>().find("fCombatDistance")->getFloat();
-        static float fHandToHandReach = MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>().find("fHandToHandReach")->getFloat();
+        static const float fCombatDistance = MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>().find("fCombatDistance")->mValue.getFloat();
+        static float fHandToHandReach = MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>().find("fHandToHandReach")->mValue.getFloat();
 
         // This distance is a possible distance of melee attack
         static float distance = fCombatDistance * std::max(2.f, fHandToHandReach);
@@ -114,19 +115,18 @@ namespace MWMechanics
     {
         isRanged = false;
 
-        static const float fCombatDistance = MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>().find("fCombatDistance")->getFloat();
-        static const float fProjectileMaxSpeed = MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>().find("fProjectileMaxSpeed")->getFloat();
+        static const float fCombatDistance = MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>().find("fCombatDistance")->mValue.getFloat();
+        static const float fProjectileMaxSpeed = MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>().find("fProjectileMaxSpeed")->mValue.getFloat();
 
         if (mWeapon.isEmpty())
         {
             static float fHandToHandReach =
-                MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>().find("fHandToHandReach")->getFloat();
+                MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>().find("fHandToHandReach")->mValue.getFloat();
             return fHandToHandReach * fCombatDistance;
         }
 
         const ESM::Weapon* weapon = mWeapon.get<ESM::Weapon>()->mBase;
-
-        if (weapon->mData.mType >= ESM::Weapon::MarksmanBow)
+        if (MWMechanics::getWeaponType(weapon->mData.mType)->mWeaponClass != ESM::WeaponType::Melee)
         {
             isRanged = true;
             return fProjectileMaxSpeed;
@@ -138,7 +138,7 @@ namespace MWMechanics
     const ESM::Weapon* ActionWeapon::getWeapon() const
     {
         if (mWeapon.isEmpty())
-            return NULL;
+            return nullptr;
         return mWeapon.get<ESM::Weapon>()->mBase;
     }
 
@@ -190,20 +190,16 @@ namespace MWMechanics
 
             for (MWWorld::ContainerStoreIterator it = store.begin(); it != store.end(); ++it)
             {
-                std::vector<int> equipmentSlots = it->getClass().getEquipmentSlots(*it).first;
-                if (std::find(equipmentSlots.begin(), equipmentSlots.end(), (int)MWWorld::InventoryStore::Slot_CarriedRight)
-                        == equipmentSlots.end())
-                    continue;
-
                 float rating = rateWeapon(*it, actor, enemy, -1, bestArrowRating, bestBoltRating);
                 if (rating > bestActionRating)
                 {
                     const ESM::Weapon* weapon = it->get<ESM::Weapon>()->mBase;
+                    int ammotype = getWeaponType(weapon->mData.mType)->mAmmoType;
 
                     MWWorld::Ptr ammo;
-                    if (weapon->mData.mType == ESM::Weapon::MarksmanBow)
+                    if (ammotype == ESM::Weapon::Arrow)
                         ammo = bestArrow;
-                    else if (weapon->mData.mType == ESM::Weapon::MarksmanCrossbow)
+                    else if (ammotype == ESM::Weapon::Bolt)
                         ammo = bestBolt;
 
                     bestActionRating = rating;
@@ -215,14 +211,12 @@ namespace MWMechanics
 
         for (Spells::TIterator it = spells.begin(); it != spells.end(); ++it)
         {
-            const ESM::Spell* spell = it->first;
-
-            float rating = rateSpell(spell, actor, enemy);
+            float rating = rateSpell(it->first, actor, enemy);
             if (rating > bestActionRating)
             {
                 bestActionRating = rating;
-                bestAction.reset(new ActionSpell(spell->mId));
-                antiFleeRating = vanillaRateSpell(spell, actor, enemy);
+                bestAction.reset(new ActionSpell(it->first->mId));
+                antiFleeRating = vanillaRateSpell(it->first, actor, enemy);
             }
         }
 
@@ -265,11 +259,6 @@ namespace MWMechanics
 
             for (MWWorld::ContainerStoreIterator it = store.begin(); it != store.end(); ++it)
             {
-                std::vector<int> equipmentSlots = it->getClass().getEquipmentSlots(*it).first;
-                if (std::find(equipmentSlots.begin(), equipmentSlots.end(), (int)MWWorld::InventoryStore::Slot_CarriedRight)
-                        == equipmentSlots.end())
-                    continue;
-
                 float rating = rateWeapon(*it, actor, enemy, -1, bestArrowRating, bestBoltRating);
                 if (rating > bestActionRating)
                 {
@@ -280,9 +269,7 @@ namespace MWMechanics
 
         for (Spells::TIterator it = spells.begin(); it != spells.end(); ++it)
         {
-            const ESM::Spell* spell = it->first;
-
-            float rating = rateSpell(spell, actor, enemy);
+            float rating = rateSpell(it->first, actor, enemy);
             if (rating > bestActionRating)
             {
                 bestActionRating = rating;
@@ -336,7 +323,7 @@ namespace MWMechanics
         float dist = 1.0f;
         if (activeWeapon.isEmpty() && !selectedSpellId.empty() && !selectedEnchItem.isEmpty())
         {
-            static const float fHandToHandReach = gmst.find("fHandToHandReach")->getFloat();
+            static const float fHandToHandReach = gmst.find("fHandToHandReach")->mValue.getFloat();
             dist = fHandToHandReach;
         }
         else if (stats.getDrawState() == MWMechanics::DrawState_Spell)
@@ -375,15 +362,15 @@ namespace MWMechanics
                 }
             }
 
-            static const float fTargetSpellMaxSpeed = gmst.find("fTargetSpellMaxSpeed")->getFloat();
+            static const float fTargetSpellMaxSpeed = gmst.find("fTargetSpellMaxSpeed")->mValue.getFloat();
             dist *= std::max(1000.0f, fTargetSpellMaxSpeed);
         }
         else if (!activeWeapon.isEmpty())
         {
             const ESM::Weapon* esmWeap = activeWeapon.get<ESM::Weapon>()->mBase;
-            if (esmWeap->mData.mType >= ESM::Weapon::MarksmanBow)
+            if (MWMechanics::getWeaponType(esmWeap->mData.mType)->mWeaponClass != ESM::WeaponType::Melee)
             {
-                static const float fTargetSpellMaxSpeed = gmst.find("fProjectileMaxSpeed")->getFloat();
+                static const float fTargetSpellMaxSpeed = gmst.find("fProjectileMaxSpeed")->mValue.getFloat();
                 dist = fTargetSpellMaxSpeed;
                 if (!activeAmmo.isEmpty())
                 {
@@ -399,8 +386,8 @@ namespace MWMechanics
 
         dist = (dist > 0.f) ? dist : 1.0f;
 
-        static const float fCombatDistance = gmst.find("fCombatDistance")->getFloat();
-        static const float fCombatDistanceWerewolfMod = gmst.find("fCombatDistanceWerewolfMod")->getFloat();
+        static const float fCombatDistance = gmst.find("fCombatDistance")->mValue.getFloat();
+        static const float fCombatDistanceWerewolfMod = gmst.find("fCombatDistanceWerewolfMod")->mValue.getFloat();
 
         float combatDistance = fCombatDistance;
         if (actor.getClass().isNpc() && actor.getClass().getNpcStats(actor).isWerewolf())
@@ -417,12 +404,9 @@ namespace MWMechanics
         ESM::Position actorPos = actor.getRefData().getPosition();
         ESM::Position enemyPos = enemy.getRefData().getPosition();
 
-        const CreatureStats& enemyStats = enemy.getClass().getCreatureStats(enemy);
-        if (enemyStats.getMagicEffects().get(ESM::MagicEffect::Invisibility).getMagnitude() > 0
-                || enemyStats.getMagicEffects().get(ESM::MagicEffect::Chameleon).getMagnitude() > 0)
+        if (isTargetMagicallyHidden(enemy) && !MWBase::Environment::get().getMechanicsManager()->awarenessCheck(enemy, actor))
         {
-            if (!MWBase::Environment::get().getMechanicsManager()->awarenessCheck(enemy, actor))
-                return false;
+            return false;
         }
 
         if (actor.getClass().isPureWaterCreature(actor))
@@ -439,6 +423,11 @@ namespace MWMechanics
                 return true;
         }
 
+        if (actor.getClass().isPureLandCreature(actor) && MWBase::Environment::get().getWorld()->isWalkingOnWater(enemy))
+        {
+            return false;
+        }
+
         if (actor.getClass().isPureFlyingCreature(actor) || actor.getClass().isPureLandCreature(actor))
         {
             if (MWBase::Environment::get().getWorld()->isSwimming(enemy))
@@ -485,19 +474,22 @@ namespace MWMechanics
         if (flee >= 100)
             return flee;
 
-        static const float fAIFleeHealthMult = gmst.find("fAIFleeHealthMult")->getFloat();
-        static const float fAIFleeFleeMult = gmst.find("fAIFleeFleeMult")->getFloat();
+        static const float fAIFleeHealthMult = gmst.find("fAIFleeHealthMult")->mValue.getFloat();
+        static const float fAIFleeFleeMult = gmst.find("fAIFleeFleeMult")->mValue.getFloat();
 
         float healthPercentage = (stats.getHealth().getModified() == 0.0f)
                                     ? 1.0f : stats.getHealth().getCurrent() / stats.getHealth().getModified();
         float rating = (1.0f - healthPercentage) * fAIFleeHealthMult + flee * fAIFleeFleeMult;
 
-        static const int iWereWolfLevelToAttack = gmst.find("iWereWolfLevelToAttack")->getInt();
+        static const int iWereWolfLevelToAttack = gmst.find("iWereWolfLevelToAttack")->mValue.getInteger();
 
-        if (enemy.getClass().isNpc() && enemy.getClass().getNpcStats(enemy).isWerewolf() && stats.getLevel() < iWereWolfLevelToAttack)
+        if (actor.getClass().isNpc() && enemy.getClass().isNpc())
         {
-            static const int iWereWolfFleeMod = gmst.find("iWereWolfFleeMod")->getInt();
-            rating = iWereWolfFleeMod;
+            if (enemy.getClass().getNpcStats(enemy).isWerewolf() && stats.getLevel() < iWereWolfLevelToAttack)
+            {
+                static const int iWereWolfFleeMod = gmst.find("iWereWolfFleeMod")->mValue.getInteger();
+                rating = iWereWolfFleeMod;
+            }
         }
 
         if (rating != 0.0f)
