@@ -11,7 +11,6 @@
 #include "../mwworld/cellstore.hpp"
 
 #include "creaturestats.hpp"
-#include "steering.hpp"
 #include "movement.hpp"
 
 /*
@@ -22,28 +21,32 @@
 namespace MWMechanics
 {
     AiEscort::AiEscort(const std::string &actorId, int duration, float x, float y, float z)
-    : mActorId(actorId), mX(x), mY(y), mZ(z), mDuration(duration), mRemainingDuration(static_cast<float>(duration))
+    : mX(x), mY(y), mZ(z), mDuration(duration), mRemainingDuration(static_cast<float>(duration))
     , mCellX(std::numeric_limits<int>::max())
     , mCellY(std::numeric_limits<int>::max())
     {
+        mTargetActorRefId = actorId;
         mMaxDist = 450;
     }
 
-    AiEscort::AiEscort(const std::string &actorId, const std::string &cellId,int duration, float x, float y, float z)
-    : mActorId(actorId), mCellId(cellId), mX(x), mY(y), mZ(z), mDuration(duration), mRemainingDuration(static_cast<float>(duration))
+    AiEscort::AiEscort(const std::string &actorId, const std::string &cellId, int duration, float x, float y, float z)
+    : mCellId(cellId), mX(x), mY(y), mZ(z), mDuration(duration), mRemainingDuration(static_cast<float>(duration))
     , mCellX(std::numeric_limits<int>::max())
     , mCellY(std::numeric_limits<int>::max())
     {
+        mTargetActorRefId = actorId;
         mMaxDist = 450;
     }
 
     AiEscort::AiEscort(const ESM::AiSequence::AiEscort *escort)
-        : mActorId(escort->mTargetId), mCellId(escort->mCellId), mX(escort->mData.mX), mY(escort->mData.mY), mZ(escort->mData.mZ)
+        : mCellId(escort->mCellId), mX(escort->mData.mX), mY(escort->mData.mY), mZ(escort->mData.mZ)
         , mMaxDist(450)
         , mRemainingDuration(escort->mRemainingDuration)
         , mCellX(std::numeric_limits<int>::max())
         , mCellY(std::numeric_limits<int>::max())
     {
+        mTargetActorRefId = escort->mTargetId;
+        mTargetActorId = escort->mTargetActorId;
         // mDuration isn't saved in the save file, so just giving it "1" for now if the package has a duration.
         // The exact value of mDuration only matters for repeating packages.
         if (mRemainingDuration > 0) // Previously mRemainingDuration could be negative even when mDuration was 0. Checking for > 0 should fix old saves.
@@ -78,25 +81,14 @@ namespace MWMechanics
         actor.getClass().getCreatureStats(actor).setDrawState(DrawState_Nothing);
         actor.getClass().getCreatureStats(actor).setMovementFlag(CreatureStats::Flag_Run, false);
 
-        const MWWorld::Ptr follower = MWBase::Environment::get().getWorld()->getPtr(mActorId, false);
-        const float* const leaderPos = actor.getRefData().getPosition().pos;
-        const float* const followerPos = follower.getRefData().getPosition().pos;
-        double differenceBetween[3];
+        const MWWorld::Ptr follower = MWBase::Environment::get().getWorld()->getPtr(mTargetActorRefId, false);
+        const osg::Vec3f leaderPos = actor.getRefData().getPosition().asVec3();
+        const osg::Vec3f followerPos = follower.getRefData().getPosition().asVec3();
 
-        for (short counter = 0; counter < 3; counter++)
-            differenceBetween[counter] = (leaderPos[counter] - followerPos[counter]);
-
-        double distanceBetweenResult =
-            (differenceBetween[0] * differenceBetween[0]) + (differenceBetween[1] * differenceBetween[1]) + (differenceBetween[2] *
-                differenceBetween[2]);
-
-        if (distanceBetweenResult <= mMaxDist * mMaxDist)
+        if ((leaderPos - followerPos).length2() <= mMaxDist * mMaxDist)
         {
-            ESM::Pathgrid::Point point(static_cast<int>(mX), static_cast<int>(mY), static_cast<int>(mZ));
-            point.mAutogenerated = 0;
-            point.mConnectionNum = 0;
-            point.mUnknown = 0;
-            if (pathTo(actor,point,duration)) //Returns true on path complete
+            const osg::Vec3f dest(mX, mY, mZ);
+            if (pathTo(actor, dest, duration)) //Returns true on path complete
             {
                 mRemainingDuration = mDuration;
                 return true;
@@ -119,18 +111,14 @@ namespace MWMechanics
         return TypeIdEscort;
     }
 
-    MWWorld::Ptr AiEscort::getTarget() const
-    {
-        return MWBase::Environment::get().getWorld()->getPtr(mActorId, false);
-    }
-
     void AiEscort::writeState(ESM::AiSequence::AiSequence &sequence) const
     {
         std::unique_ptr<ESM::AiSequence::AiEscort> escort(new ESM::AiSequence::AiEscort());
         escort->mData.mX = mX;
         escort->mData.mY = mY;
         escort->mData.mZ = mZ;
-        escort->mTargetId = mActorId;
+        escort->mTargetId = mTargetActorRefId;
+        escort->mTargetActorId = mTargetActorId;
         escort->mRemainingDuration = mRemainingDuration;
         escort->mCellId = mCellId;
 
