@@ -3,10 +3,6 @@
 
 #include "aipackage.hpp"
 
-#include "../mwbase/world.hpp"
-
-#include "pathfinding.hpp"
-
 namespace ESM
 {
 namespace AiSequence
@@ -40,10 +36,6 @@ namespace MWMechanics
 
             virtual bool canCancel() const { return false; }
             virtual bool shouldCancelPreviousAi() const { return false; }
-
-        private:
-
-            int mTargetActorId; // The actor to pursue
     };
 }
 #endif
