@@ -3,8 +3,6 @@
 
 #include "aipackage.hpp"
 
-#include "pathfinding.hpp"
-
 namespace ESM
 {
 namespace AiSequence
@@ -20,7 +18,7 @@ namespace MWMechanics
     {
         public:
             /// Default constructor
-            AiTravel(float x, float y, float z);
+            AiTravel(float x, float y, float z, bool hidden = false);
             AiTravel(const ESM::AiSequence::AiTravel* travel);
 
             /// Simulates the passing of time
@@ -34,10 +32,18 @@ namespace MWMechanics
 
             virtual int getTypeId() const;
 
+            virtual bool useVariableSpeed() const { return true;}
+
+            virtual bool alwaysActive() const { return true; }
+
+            virtual osg::Vec3f getDestination() const { return osg::Vec3f(mX, mY, mZ); }
+
         private:
             float mX;
             float mY;
             float mZ;
+
+            bool mHidden;
     };
 }
 
