@@ -10,6 +10,8 @@
 
 #include "../mwrender/animation.hpp"
 
+#include "weapontype.hpp"
+
 namespace MWWorld
 {
     class InventoryStore;
@@ -39,8 +41,8 @@ enum Priority {
     Priority_Knockdown,
     Priority_Torch,
     Priority_Storm,
-
     Priority_Death,
+    Priority_Persistent,
 
     Num_Priorities
 };
@@ -113,21 +115,6 @@ enum CharacterState {
     CharState_Block
 };
 
-enum WeaponType {
-    WeapType_None,
-
-    WeapType_HandToHand,
-    WeapType_OneHand,
-    WeapType_TwoHand,
-    WeapType_TwoWide,
-    WeapType_BowAndArrow,
-    WeapType_Crossbow,
-    WeapType_Thrown,
-    WeapType_PickProbe,
-
-    WeapType_Spell
-};
-
 enum UpperBodyCharacterState {
     UpperCharState_Nothing,
     UpperCharState_EquipingWeap,
@@ -152,6 +139,7 @@ struct WeaponInfo;
 class CharacterController : public MWRender::Animation::TextKeyListener
 {
     MWWorld::Ptr mPtr;
+    MWWorld::Ptr mWeapon;
     MWRender::Animation *mAnimation;
     
     struct AnimationQueueEntry
@@ -185,7 +173,7 @@ class CharacterController : public MWRender::Animation::TextKeyListener
     JumpingState mJumpState;
     std::string mCurrentJump;
 
-    WeaponType mWeaponType;
+    int mWeaponType;
     std::string mCurrentWeapon;
 
     float mAttackStrength;
@@ -203,23 +191,29 @@ class CharacterController : public MWRender::Animation::TextKeyListener
     std::string mAttackType; // slash, chop or thrust
 
     bool mAttackingOrSpell;
+    bool mCastingManualSpell;
 
     float mTimeUntilWake;
 
     void setAttackTypeBasedOnMovement();
 
     void refreshCurrentAnims(CharacterState idle, CharacterState movement, JumpingState jump, bool force=false);
-    void refreshHitRecoilAnims();
-    void refreshJumpAnims(const WeaponInfo* weap, JumpingState jump, bool force=false);
-    void refreshMovementAnims(const WeaponInfo* weap, CharacterState movement, bool force=false);
-    void refreshIdleAnims(const WeaponInfo* weap, CharacterState idle, bool force=false);
+    void refreshHitRecoilAnims(CharacterState& idle);
+    void refreshJumpAnims(const std::string& weapShortGroup, JumpingState jump, CharacterState& idle, bool force=false);
+    void refreshMovementAnims(const std::string& weapShortGroup, CharacterState movement, CharacterState& idle, bool force=false);
+    void refreshIdleAnims(const std::string& weapShortGroup, CharacterState idle, bool force=false);
 
-    void clearAnimQueue();
+    void clearAnimQueue(bool clearPersistAnims = false);
 
-    bool updateWeaponState();
+    bool updateWeaponState(CharacterState& idle);
     bool updateCreatureState();
     void updateIdleStormState(bool inwater);
 
+    std::string chooseRandomAttackAnimation() const;
+    bool isRandomAttackAnimation(const std::string& group) const;
+
+    bool isPersistentAnimPlaying();
+
     void updateAnimQueue();
 
     void updateHeadTracking(float duration);
@@ -231,10 +225,14 @@ class CharacterController : public MWRender::Animation::TextKeyListener
     void playRandomDeath(float startpoint = 0.0f);
 
     /// choose a random animation group with \a prefix and numeric suffix
-    /// @param num if non-NULL, the chosen animation number will be written here
-    std::string chooseRandomGroup (const std::string& prefix, int* num = NULL) const;
+    /// @param num if non-nullptr, the chosen animation number will be written here
+    std::string chooseRandomGroup (const std::string& prefix, int* num = nullptr) const;
 
-    bool updateCarriedLeftVisible(WeaponType weaptype) const;
+    bool updateCarriedLeftVisible(int weaptype) const;
+
+    std::string fallbackShortWeaponGroup(const std::string& baseGroupName, MWRender::Animation::BlendMask* blendMask = nullptr);
+
+    std::string getWeaponAnimation(int weaponType) const;
 
 public:
     CharacterController(const MWWorld::Ptr &ptr, MWRender::Animation *anim);
@@ -248,7 +246,10 @@ public:
 
     void updatePtr(const MWWorld::Ptr &ptr);
 
-    void update(float duration);
+    void update(float duration, bool animationOnly=false);
+
+    bool onOpen();
+    void onClose();
 
     void persistAnimationState();
     void unpersistAnimationState();
@@ -272,7 +273,8 @@ public:
 
     void forceStateUpdate();
     
-    bool isAttackPrepairing() const;
+    bool isAttackPreparing() const;
+    bool isCastingSpell() const;
     bool isReadyToBlock() const;
     bool isKnockedDown() const;
     bool isKnockedOut() const;
@@ -282,7 +284,9 @@ public:
     bool isTurning() const;
     bool isAttackingOrSpell() const;
 
+    void setVisibility(float visibility);
     void setAttackingOrSpell(bool attackingOrSpell);
+    void castSpell(const std::string spellId, bool manualSpell=false);
     void setAIAttackType(const std::string& attackType);
     static void setAttackTypeRandomly(std::string& attackType);
 
@@ -299,8 +303,6 @@ public:
 
     void playSwishSound(float attackStrength);
 };
-
-    MWWorld::ContainerStoreIterator getActiveWeapon(CreatureStats &stats, MWWorld::InventoryStore &inv, WeaponType *weaptype);
 }
 
 #endif /* GAME_MWMECHANICS_CHARACTER_HPP */
