@@ -7,6 +7,7 @@
 #include <components/esm/esmwriter.hpp>
 
 #include "../mwworld/esmstore.hpp"
+#include "../mwworld/player.hpp"
 
 #include "../mwbase/environment.hpp"
 #include "../mwbase/world.hpp"
@@ -22,12 +23,53 @@ namespace MWMechanics
           mKnockdown(false), mKnockdownOneFrame(false), mKnockdownOverOneFrame(false),
           mHitRecovery(false), mBlock(false), mMovementFlags(0),
           mFallHeight(0), mRecalcMagicka(false), mLastRestock(0,0), mGoldPool(0), mActorId(-1), mHitAttemptActorId(-1),
-          mDeathAnimation(-1), mTimeOfDeath(), mLevel (0)
+          mDeathAnimation(-1), mTimeOfDeath(), mGreetingState(Greet_None),
+          mGreetingTimer(0), mTargetAngleRadians(0), mIsTurningToPlayer(false), mLevel (0)
     {
         for (int i=0; i<4; ++i)
             mAiSettings[i] = 0;
     }
 
+    int MWMechanics::CreatureStats::getGreetingTimer() const
+    {
+        return mGreetingTimer;
+    }
+
+    void MWMechanics::CreatureStats::setGreetingTimer(int timer)
+    {
+        mGreetingTimer = timer;
+    }
+
+    float MWMechanics::CreatureStats::getAngleToPlayer() const
+    {
+        return mTargetAngleRadians;
+    }
+
+    void MWMechanics::CreatureStats::setAngleToPlayer(float angle)
+    {
+        mTargetAngleRadians = angle;
+    }
+
+    GreetingState MWMechanics::CreatureStats::getGreetingState() const
+    {
+        return mGreetingState;
+    }
+
+    void MWMechanics::CreatureStats::setGreetingState(GreetingState state)
+    {
+        mGreetingState = state;
+    }
+
+    bool MWMechanics::CreatureStats::isTurningToPlayer() const
+    {
+        return mIsTurningToPlayer;
+    }
+
+    void MWMechanics::CreatureStats::setTurningToPlayer(bool turning)
+    {
+        mIsTurningToPlayer = turning;
+    }
+
     const AiSequence& CreatureStats::getAiSequence() const
     {
         return mAiSequence;
@@ -48,8 +90,8 @@ namespace MWMechanics
         const MWWorld::Store<ESM::GameSetting> &gmst =
             MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>();
 
-        static const float fFatigueBase = gmst.find("fFatigueBase")->getFloat();
-        static const float fFatigueMult = gmst.find("fFatigueMult")->getFloat();
+        static const float fFatigueBase = gmst.find("fFatigueBase")->mValue.getFloat();
+        static const float fFatigueMult = gmst.find("fFatigueMult")->mValue.getFloat();
 
         return fFatigueBase - fFatigueMult * (1-normalised);
     }
@@ -157,7 +199,7 @@ namespace MWMechanics
                 int endurance    = getAttribute(ESM::Attribute::Endurance).getModified();
                 DynamicStat<float> fatigue = getFatigue();
                 float diff = (strength+willpower+agility+endurance) - fatigue.getBase();
-                float currentToBaseRatio = (fatigue.getCurrent() / fatigue.getBase());
+                float currentToBaseRatio = fatigue.getBase() > 0 ? (fatigue.getCurrent() / fatigue.getBase()) : 0;
                 fatigue.setModified(fatigue.getModified() + diff, 0);
                 fatigue.setCurrent(fatigue.getBase() * currentToBaseRatio);
                 setFatigue(fatigue);
@@ -195,10 +237,8 @@ namespace MWMechanics
             mDead = true;
 
             mDynamic[index].setModifier(0);
+            mDynamic[index].setCurrentModifier(0);
             mDynamic[index].setCurrent(0);
-
-            if (MWBase::Environment::get().getWorld()->getGodModeState())
-                MWBase::Environment::get().getMechanicsManager()->keepPlayerAlive();
         }
     }
 
@@ -386,8 +426,16 @@ namespace MWMechanics
         mFallHeight += height;
     }
 
-    float CreatureStats::land()
+    float CreatureStats::getFallHeight() const
     {
+        return mFallHeight;
+    }
+
+    float CreatureStats::land(bool isPlayer)
+    {
+        if (isPlayer)
+            MWBase::Environment::get().getWorld()->getPlayer().setJumping(false);
+
         float height = mFallHeight;
         mFallHeight = 0;
         return height;
