@@ -1,12 +1,9 @@
 #include "repair.hpp"
 
-#include <boost/format.hpp>
-
 #include <components/misc/rng.hpp>
 
 #include "../mwbase/world.hpp"
 #include "../mwbase/environment.hpp"
-#include "../mwbase/mechanicsmanager.hpp"
 #include "../mwbase/windowmanager.hpp"
 
 #include "../mwworld/containerstore.hpp"
@@ -14,7 +11,6 @@
 #include "../mwworld/esmstore.hpp"
 
 #include "creaturestats.hpp"
-#include "npcstats.hpp"
 #include "actorutil.hpp"
 
 namespace MWMechanics
@@ -34,15 +30,14 @@ void Repair::repair(const MWWorld::Ptr &itemToRepair)
     mTool.getCellRef().setCharge(uses-1);
 
     MWMechanics::CreatureStats& stats = player.getClass().getCreatureStats(player);
-    MWMechanics::NpcStats& npcStats = player.getClass().getNpcStats(player);
 
     float fatigueTerm = stats.getFatigueTerm();
     int pcStrength = stats.getAttribute(ESM::Attribute::Strength).getModified();
     int pcLuck = stats.getAttribute(ESM::Attribute::Luck).getModified();
-    int armorerSkill = npcStats.getSkill(ESM::Skill::Armorer).getModified();
+    int armorerSkill = player.getClass().getSkill(player, ESM::Skill::Armorer);
 
     float fRepairAmountMult = MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>()
-            .find("fRepairAmountMult")->getFloat();
+            .find("fRepairAmountMult")->mValue.getFloat();
 
     float toolQuality = ref->mBase->mData.mQuality;
 
@@ -87,9 +82,10 @@ void Repair::repair(const MWWorld::Ptr &itemToRepair)
         store.remove(mTool, 1, player);
 
         std::string message = MWBase::Environment::get().getWorld()->getStore().get<ESM::GameSetting>()
-                .find("sNotifyMessage51")->getString();
+                .find("sNotifyMessage51")->mValue.getString();
+        message = Misc::StringUtils::format(message, mTool.getClass().getName(mTool));
 
-        MWBase::Environment::get().getWindowManager()->messageBox((boost::format(message) % mTool.getClass().getName(mTool)).str());
+        MWBase::Environment::get().getWindowManager()->messageBox(message);
 
         // try to find a new tool of the same ID
         for (MWWorld::ContainerStoreIterator iter (store.begin());
