@@ -1,6 +1,6 @@
 #include "summoning.hpp"
 
-#include <iostream>
+#include <components/debug/debuglog.hpp>
 
 #include "../mwbase/environment.hpp"
 #include "../mwbase/world.hpp"
@@ -59,7 +59,7 @@ namespace MWMechanics
                         MWMechanics::CreatureStats& summonedCreatureStats = ref.getPtr().getClass().getCreatureStats(ref.getPtr());
 
                         // Make the summoned creature follow its master and help in fights
-                        AiFollow package(mActor.getCellRef().getRefId());
+                        AiFollow package(mActor);
                         summonedCreatureStats.getAiSequence().stack(package, ref.getPtr());
                         creatureActorId = summonedCreatureStats.getActorId();
 
@@ -76,7 +76,7 @@ namespace MWMechanics
                     }
                     catch (std::exception& e)
                     {
-                        std::cerr << "Failed to spawn summoned creature: " << e.what() << std::endl;
+                        Log(Debug::Error) << "Failed to spawn summoned creature: " << e.what();
                         // still insert into creatureMap so we don't try to spawn again every frame, that would spam the warning log
                     }
 
@@ -99,25 +99,11 @@ namespace MWMechanics
             ++it;
         }
 
-        std::vector<int>& graveyard = creatureStats.getSummonedCreatureGraveyard();
-        for (std::vector<int>::iterator it = graveyard.begin(); it != graveyard.end(); )
-        {
-            MWWorld::Ptr ptr = MWBase::Environment::get().getWorld()->searchPtrViaActorId(*it);
-            if (!ptr.isEmpty())
-            {
-                it = graveyard.erase(it);
+        std::vector<int> graveyard = creatureStats.getSummonedCreatureGraveyard();
+        creatureStats.getSummonedCreatureGraveyard().clear();
 
-                const ESM::Static* fx = MWBase::Environment::get().getWorld()->getStore().get<ESM::Static>()
-                        .search("VFX_Summon_End");
-                if (fx)
-                    MWBase::Environment::get().getWorld()->spawnEffect("meshes\\" + fx->mModel,
-                        "", ptr.getRefData().getPosition().asVec3());
-
-                MWBase::Environment::get().getWorld()->deleteObject(ptr);
-            }
-            else
-                ++it;
-        }
+        for (const int creature : graveyard)
+            MWBase::Environment::get().getMechanicsManager()->cleanupSummonedCreature(mActor, creature);
 
         if (!cleanup)
             return;
