@@ -1,13 +1,13 @@
 #include "bulletdebugdraw.hpp"
 
-#include <iostream>
-
 #include <BulletCollision/CollisionDispatch/btCollisionWorld.h>
 
 #include <osg/Geometry>
 #include <osg/Group>
 
-#include "vismask.hpp"
+#include <components/debug/debuglog.hpp>
+
+#include <components/sceneutil/vismask.hpp>
 
 namespace
 {
@@ -34,7 +34,7 @@ void DebugDrawer::createGeometry()
     if (!mGeometry)
     {
         mGeometry = new osg::Geometry;
-        mGeometry->setNodeMask(Mask_Debug);
+        mGeometry->setNodeMask(SceneUtil::Mask_Debug);
 
         mVertices = new osg::Vec3Array;
 
@@ -54,9 +54,9 @@ void DebugDrawer::destroyGeometry()
     if (mGeometry)
     {
         mParentNode->removeChild(mGeometry);
-        mGeometry = NULL;
-        mVertices = NULL;
-        mDrawArrays = NULL;
+        mGeometry = nullptr;
+        mVertices = nullptr;
+        mDrawArrays = nullptr;
     }
 }
 
@@ -91,7 +91,7 @@ void DebugDrawer::drawContactPoint(const btVector3 &PointOnB, const btVector3 &n
 
 void DebugDrawer::reportErrorWarning(const char *warningString)
 {
-    std::cerr << warningString << std::endl;
+    Log(Debug::Warning) << warningString;
 }
 
 void DebugDrawer::setDebugMode(int isOn)
