@@ -37,7 +37,6 @@ namespace MWRender
 
         bool mFirstPersonView;
         bool mPreviewMode;
-        bool mFreeLook;
         float mNearest;
         float mFurthest;
         bool mIsNearest;
@@ -119,8 +118,6 @@ namespace MWRender
         /// Stores focal and camera world positions in passed arguments
         void getPosition(osg::Vec3f &focal, osg::Vec3f &camera);
 
-        void togglePlayerLooking(bool enable);
-
         bool isVanityOrPreviewModeEnabled();
 
         bool isNearest();
