@@ -1,7 +1,5 @@
 #include "globalmap.hpp"
 
-#include <climits>
-
 #include <osg/Image>
 #include <osg/Texture2D>
 #include <osg/Group>
@@ -15,7 +13,10 @@
 #include <components/settings/settings.hpp>
 #include <components/files/memorystream.hpp>
 
+#include <components/debug/debuglog.hpp>
+
 #include <components/sceneutil/workqueue.hpp>
+#include <components/sceneutil/vismask.hpp>
 
 #include <components/esm/globalmap.hpp>
 
@@ -24,8 +25,6 @@
 
 #include "../mwworld/esmstore.hpp"
 
-#include "vismask.hpp"
-
 namespace
 {
 
@@ -74,17 +73,17 @@ namespace
         {
             if (mRendered)
             {
-                node->setNodeMask(0);
+                if (mParent->copyResult(static_cast<osg::Camera*>(node), nv->getTraversalNumber()))
+                {
+                    node->setNodeMask(SceneUtil::Mask_Disabled);
+                    mParent->markForRemoval(static_cast<osg::Camera*>(node));
+                }
                 return;
             }
 
             traverse(node, nv);
 
-            if (!mRendered)
-            {
-                mRendered = true;
-                mParent->markForRemoval(static_cast<osg::Camera*>(node));
-            }
+            mRendered = true;
         }
 
     private:
@@ -234,10 +233,10 @@ namespace MWRender
 
     GlobalMap::~GlobalMap()
     {
-        for (CameraVector::iterator it = mCamerasPendingRemoval.begin(); it != mCamerasPendingRemoval.end(); ++it)
-            removeCamera(*it);
-        for (CameraVector::iterator it = mActiveCameras.begin(); it != mActiveCameras.end(); ++it)
-            removeCamera(*it);
+        for (auto& camera : mCamerasPendingRemoval)
+            removeCamera(camera);
+        for (auto& camera : mActiveCameras)
+            removeCamera(camera);
 
         if (mWorkItem)
             mWorkItem->waitTillDone();
@@ -271,9 +270,9 @@ namespace MWRender
 
     void GlobalMap::worldPosToImageSpace(float x, float z, float& imageX, float& imageY)
     {
-        imageX = float(x / 8192.f - mMinX) / (mMaxX - mMinX + 1);
+        imageX = float(x / float(Constants::CellSizeInUnits) - mMinX) / (mMaxX - mMinX + 1);
 
-        imageY = 1.f-float(z / 8192.f - mMinY) / (mMaxY - mMinY + 1);
+        imageY = 1.f-float(z / float(Constants::CellSizeInUnits) - mMinY) / (mMaxY - mMinY + 1);
     }
 
     void GlobalMap::cellTopLeftCornerToImageSpace(int x, int y, float& imageX, float& imageY)
@@ -288,12 +287,12 @@ namespace MWRender
                                                 float srcLeft, float srcTop, float srcRight, float srcBottom)
     {
         osg::ref_ptr<osg::Camera> camera (new osg::Camera);
-        camera->setNodeMask(Mask_RenderToTexture);
+        camera->setNodeMask(SceneUtil::Mask_RenderToTexture);
         camera->setReferenceFrame(osg::Camera::ABSOLUTE_RF);
         camera->setViewMatrix(osg::Matrix::identity());
         camera->setProjectionMatrix(osg::Matrix::identity());
         camera->setProjectionResizePolicy(osg::Camera::FIXED);
-        camera->setRenderOrder(osg::Camera::PRE_RENDER);
+        camera->setRenderOrder(osg::Camera::PRE_RENDER, 1); // Make sure the global map is rendered after the local map
         y = mHeight - y - height; // convert top-left origin to bottom-left
         camera->setViewport(x, y, width, height);
 
@@ -325,7 +324,7 @@ namespace MWRender
             imageDest.mImage = image;
             imageDest.mX = x;
             imageDest.mY = y;
-            mPendingImageDest.push_back(imageDest);
+            mPendingImageDest[camera] = imageDest;
         }
 
         // Create a quad rendering the updated texture
@@ -411,14 +410,14 @@ namespace MWRender
         osgDB::ReaderWriter* readerwriter = osgDB::Registry::instance()->getReaderWriterForExtension("png");
         if (!readerwriter)
         {
-            std::cerr << "Error: Can't write map overlay: no png readerwriter found" << std::endl;
+            Log(Debug::Error) << "Error: Can't write map overlay: no png readerwriter found";
             return;
         }
 
         osgDB::ReaderWriter::WriteResult result = readerwriter->writeImage(*mOverlayImage, ostream);
         if (!result.success())
         {
-            std::cerr << "Error: Can't write map overlay: " << result.message() << " code " << result.status() << std::endl;
+            Log(Debug::Warning) << "Error: Can't write map overlay: " << result.message() << " code " << result.status();
             return;
         }
 
@@ -458,19 +457,19 @@ namespace MWRender
         if (map.mImageData.empty())
             return;
 
-        Files::IMemStream istream(&map.mImageData[0], map.mImageData.size());
+        Files::IMemStream istream(map.mImageData.data(), map.mImageData.size());
 
         osgDB::ReaderWriter* readerwriter = osgDB::Registry::instance()->getReaderWriterForExtension("png");
         if (!readerwriter)
         {
-            std::cerr << "Error: Can't read map overlay: no png readerwriter found" << std::endl;
+            Log(Debug::Error) << "Error: Can't read map overlay: no png readerwriter found";
             return;
         }
 
         osgDB::ReaderWriter::ReadResult result = readerwriter->readImage(istream);
         if (!result.success())
         {
-            std::cerr << "Error: Can't read map overlay: " << result.message() << " code " << result.status() << std::endl;
+            Log(Debug::Error) << "Error: Can't read map overlay: " << result.message() << " code " << result.status();
             return;
         }
 
@@ -523,7 +522,7 @@ namespace MWRender
 
         if (srcBox == destBox && imageWidth == mWidth && imageHeight == mHeight)
         {
-            mOverlayImage->copySubImage(0, 0, 0, image);
+            mOverlayImage = image;
 
             requestOverlayTextureUpdate(0, 0, mWidth, mHeight, texture, true, false);
         }
@@ -563,7 +562,28 @@ namespace MWRender
 
             requestOverlayTextureUpdate(0, 0, mWidth, mHeight, osg::ref_ptr<osg::Texture2D>(), true, false);
 
-            mWorkItem = NULL;
+            mWorkItem = nullptr;
+        }
+    }
+
+    bool GlobalMap::copyResult(osg::Camera *camera, unsigned int frame)
+    {
+        ImageDestMap::iterator it = mPendingImageDest.find(camera);
+        if (it == mPendingImageDest.end())
+            return true;
+        else
+        {
+            ImageDest& imageDest = it->second;
+            if (imageDest.mFrameDone == 0) imageDest.mFrameDone = frame+2; // wait an extra frame to ensure the draw thread has completed its frame.
+            if (imageDest.mFrameDone > frame)
+            {
+                ++it;
+                return false;
+            }
+
+            mOverlayImage->copySubImage(imageDest.mX, imageDest.mY, 0, imageDest.mImage);
+            it = mPendingImageDest.erase(it);
+            return true;
         }
     }
 
@@ -572,7 +592,7 @@ namespace MWRender
         CameraVector::iterator found = std::find(mActiveCameras.begin(), mActiveCameras.end(), camera);
         if (found == mActiveCameras.end())
         {
-            std::cerr << "Error: GlobalMap trying to remove an inactive camera" << std::endl;
+            Log(Debug::Error) << "Error: GlobalMap trying to remove an inactive camera";
             return;
         }
         mActiveCameras.erase(found);
@@ -581,25 +601,10 @@ namespace MWRender
 
     void GlobalMap::cleanupCameras()
     {
-        for (CameraVector::iterator it = mCamerasPendingRemoval.begin(); it != mCamerasPendingRemoval.end(); ++it)
-            removeCamera(*it);
+        for (auto& camera : mCamerasPendingRemoval)
+            removeCamera(camera);
 
         mCamerasPendingRemoval.clear();
-
-        for (ImageDestVector::iterator it = mPendingImageDest.begin(); it != mPendingImageDest.end();)
-        {
-            ImageDest& imageDest = *it;
-            if (--imageDest.mFramesUntilDone > 0)
-            {
-                ++it;
-                continue;
-            }
-
-            ensureLoaded();
-            mOverlayImage->copySubImage(imageDest.mX, imageDest.mY, 0, imageDest.mImage);
-
-            it = mPendingImageDest.erase(it);
-        }
     }
 
     void GlobalMap::removeCamera(osg::Camera *cam)
