@@ -3,6 +3,7 @@
 
 #include <osg/Object>
 
+#include <components/resource/objectcache.hpp>
 #include <components/resource/resourcemanager.hpp>
 #include <components/esmterrain/storage.hpp>
 
@@ -14,12 +15,12 @@ namespace ESM
 namespace MWRender
 {
 
-    class LandManager : public Resource::ResourceManager
+    class LandManager : public Resource::GenericResourceManager<std::pair<int, int> >
     {
     public:
         LandManager(int loadFlags);
 
-        /// @note Will return NULL if not found.
+        /// @note Will return nullptr if not found.
         osg::ref_ptr<ESMTerrain::LandObject> getLand(int x, int y);
 
         virtual void reportStats(unsigned int frameNumber, osg::Stats* stats) const;
