@@ -1,12 +1,11 @@
 #include "objects.hpp"
 
-#include <cmath>
-
 #include <osg/Group>
 #include <osg/UserDataContainer>
 
 #include <components/sceneutil/positionattitudetransform.hpp>
 #include <components/sceneutil/unrefqueue.hpp>
+#include <components/sceneutil/vismask.hpp>
 
 #include "../mwworld/ptr.hpp"
 #include "../mwworld/class.hpp"
@@ -14,8 +13,6 @@
 #include "animation.hpp"
 #include "npcanimation.hpp"
 #include "creatureanimation.hpp"
-#include "vismask.hpp"
-
 
 namespace MWRender
 {
@@ -73,6 +70,7 @@ void Objects::insertBegin(const MWWorld::Ptr& ptr)
 void Objects::insertModel(const MWWorld::Ptr &ptr, const std::string &mesh, bool animated, bool allowLight)
 {
     insertBegin(ptr);
+    ptr.getRefData().getBaseNode()->setNodeMask(SceneUtil::Mask_Object);
 
     osg::ref_ptr<ObjectAnimation> anim (new ObjectAnimation(ptr, mesh, mResourceSystem, animated, allowLight));
 
@@ -82,7 +80,7 @@ void Objects::insertModel(const MWWorld::Ptr &ptr, const std::string &mesh, bool
 void Objects::insertCreature(const MWWorld::Ptr &ptr, const std::string &mesh, bool weaponsShields)
 {
     insertBegin(ptr);
-    ptr.getRefData().getBaseNode()->setNodeMask(Mask_Actor);
+    ptr.getRefData().getBaseNode()->setNodeMask(SceneUtil::Mask_Actor);
 
     // CreatureAnimation
     osg::ref_ptr<Animation> anim;
@@ -99,7 +97,7 @@ void Objects::insertCreature(const MWWorld::Ptr &ptr, const std::string &mesh, b
 void Objects::insertNPC(const MWWorld::Ptr &ptr)
 {
     insertBegin(ptr);
-    ptr.getRefData().getBaseNode()->setNodeMask(Mask_Actor);
+    ptr.getRefData().getBaseNode()->setNodeMask(SceneUtil::Mask_Actor);
 
     osg::ref_ptr<NpcAnimation> anim (new NpcAnimation(ptr, osg::ref_ptr<osg::Group>(ptr.getRefData().getBaseNode()), mResourceSystem));
 
@@ -126,14 +124,14 @@ bool Objects::removeObject (const MWWorld::Ptr& ptr)
         if (ptr.getClass().isActor())
         {
             if (ptr.getClass().hasInventoryStore(ptr))
-                ptr.getClass().getInventoryStore(ptr).setInvListener(NULL, ptr);
+                ptr.getClass().getInventoryStore(ptr).setInvListener(nullptr, ptr);
 
-            ptr.getClass().getContainerStore(ptr).setContListener(NULL);
+            ptr.getClass().getContainerStore(ptr).setContListener(nullptr);
         }
 
         ptr.getRefData().getBaseNode()->getParent(0)->removeChild(ptr.getRefData().getBaseNode());
 
-        ptr.getRefData().setBaseNode(NULL);
+        ptr.getRefData().setBaseNode(nullptr);
         return true;
     }
     return false;
@@ -153,8 +151,8 @@ void Objects::removeCell(const MWWorld::CellStore* store)
             if (ptr.getClass().isNpc() && ptr.getRefData().getCustomData())
             {
                 MWWorld::InventoryStore& invStore = ptr.getClass().getInventoryStore(ptr);
-                invStore.setInvListener(NULL, ptr);
-                invStore.setContListener(NULL);
+                invStore.setInvListener(nullptr, ptr);
+                invStore.setContListener(nullptr);
             }
 
             mObjects.erase(iter++);
@@ -218,7 +216,7 @@ Animation* Objects::getAnimation(const MWWorld::Ptr &ptr)
     if(iter != mObjects.end())
         return iter->second;
 
-    return NULL;
+    return nullptr;
 }
 
 const Animation* Objects::getAnimation(const MWWorld::ConstPtr &ptr) const
@@ -227,7 +225,7 @@ const Animation* Objects::getAnimation(const MWWorld::ConstPtr &ptr) const
     if(iter != mObjects.end())
         return iter->second;
 
-    return NULL;
+    return nullptr;
 }
 
 }
