@@ -6,29 +6,26 @@
 #include <osg/PositionAttitudeTransform>
 #include <osg/Group>
 
-#include <components/esm/loadstat.hpp>
 #include <components/esm/loadpgrd.hpp>
 #include <components/sceneutil/pathgridutil.hpp>
+#include <components/sceneutil/vismask.hpp>
 
 #include "../mwbase/world.hpp" // these includes can be removed once the static-hack is gone
 #include "../mwbase/environment.hpp"
 
-#include "../mwworld/ptr.hpp"
 #include "../mwworld/cellstore.hpp"
 #include "../mwworld/esmstore.hpp"
 #include "../mwmechanics/pathfinding.hpp"
 #include "../mwmechanics/coordinateconverter.hpp"
 
-#include "vismask.hpp"
-
 namespace MWRender
 {
 
 Pathgrid::Pathgrid(osg::ref_ptr<osg::Group> root)
     : mPathgridEnabled(false)
     , mRootNode(root)
-    , mPathGridRoot(NULL)
-    , mInteriorPathgridNode(NULL)
+    , mPathGridRoot(nullptr)
+    , mInteriorPathgridNode(nullptr)
 {
 }
 
@@ -75,26 +72,26 @@ void Pathgrid::togglePathgrid()
     {
         // add path grid meshes to already loaded cells
         mPathGridRoot = new osg::Group;
-        mPathGridRoot->setNodeMask(Mask_Debug);
+        mPathGridRoot->setNodeMask(SceneUtil::Mask_Pathgrid);
         mRootNode->addChild(mPathGridRoot);
 
-        for(CellList::iterator it = mActiveCells.begin(); it != mActiveCells.end(); ++it)
+        for(const MWWorld::CellStore* cell : mActiveCells)
         {
-            enableCellPathgrid(*it);
+            enableCellPathgrid(cell);
         }
     }
     else
     {
         // remove path grid meshes from already loaded cells
-        for(CellList::iterator it = mActiveCells.begin(); it != mActiveCells.end(); ++it)
+        for(const MWWorld::CellStore* cell : mActiveCells)
         {
-            disableCellPathgrid(*it);
+            disableCellPathgrid(cell);
         }
 
         if (mPathGridRoot)
         {
             mRootNode->removeChild(mPathGridRoot);
-            mPathGridRoot = NULL;
+            mPathGridRoot = nullptr;
         }
     }
 }
@@ -124,7 +121,7 @@ void Pathgrid::enableCellPathgrid(const MWWorld::CellStore *store)
     }
     else
     {
-        assert(mInteriorPathgridNode == NULL);
+        assert(mInteriorPathgridNode == nullptr);
         mInteriorPathgridNode = cellPathGrid;
     }
 }
@@ -146,7 +143,7 @@ void Pathgrid::disableCellPathgrid(const MWWorld::CellStore *store)
         if (mInteriorPathgridNode)
         {
             mPathGridRoot->removeChild(mInteriorPathgridNode);
-            mInteriorPathgridNode = NULL;
+            mInteriorPathgridNode = nullptr;
         }
     }
 }
