@@ -16,8 +16,7 @@
 #include <components/resource/resourcesystem.hpp>
 #include <components/resource/scenemanager.hpp>
 #include <components/fallback/fallback.hpp>
-
-#include "vismask.hpp"
+#include <components/sceneutil/vismask.hpp>
 
 #include "../mwbase/world.hpp"
 #include "../mwbase/environment.hpp"
@@ -26,13 +25,13 @@
 
 namespace
 {
-    void createWaterRippleStateSet(Resource::ResourceSystem* resourceSystem, const Fallback::Map* fallback, osg::Node* node)
+    void createWaterRippleStateSet(Resource::ResourceSystem* resourceSystem,osg::Node* node)
     {
-        int rippleFrameCount = fallback->getFallbackInt("Water_RippleFrameCount");
+        int rippleFrameCount = Fallback::Map::getInt("Water_RippleFrameCount");
         if (rippleFrameCount <= 0)
             return;
 
-        std::string tex = fallback->getFallbackString("Water_RippleTexture");
+        const std::string& tex = Fallback::Map::getString("Water_RippleTexture");
 
         std::vector<osg::ref_ptr<osg::Texture2D> > textures;
         for (int i=0; i<rippleFrameCount; ++i)
@@ -68,8 +67,8 @@ namespace
 
         osg::ref_ptr<osg::Material> mat (new osg::Material);
         mat->setDiffuse(osg::Material::FRONT_AND_BACK, osg::Vec4f(0.f, 0.f, 0.f, 1.f));
-        mat->setAmbient(osg::Material::FRONT_AND_BACK, osg::Vec4f(0.f, 0.f, 0.f, 1.f));
-        mat->setEmission(osg::Material::FRONT_AND_BACK, osg::Vec4f(1.f, 1.f, 1.f, 1.f));
+        mat->setAmbient(osg::Material::FRONT_AND_BACK, osg::Vec4f(1.f, 1.f, 1.f, 1.f));
+        mat->setEmission(osg::Material::FRONT_AND_BACK, osg::Vec4f(0.f, 0.f, 0.f, 1.f));
         mat->setSpecular(osg::Material::FRONT_AND_BACK, osg::Vec4f(0.f, 0.f, 0.f, 0.f));
         mat->setColorMode(osg::Material::DIFFUSE);
         stateset->setAttributeAndModes(mat, osg::StateAttribute::ON);
@@ -81,7 +80,7 @@ namespace
 namespace MWRender
 {
 
-RippleSimulation::RippleSimulation(osg::Group *parent, Resource::ResourceSystem* resourceSystem, const Fallback::Map* fallback)
+RippleSimulation::RippleSimulation(osg::Group *parent, Resource::ResourceSystem* resourceSystem)
     : mParent(parent)
 {
     mParticleSystem = new osgParticle::ParticleSystem;
@@ -94,18 +93,21 @@ RippleSimulation::RippleSimulation(osg::Group *parent, Resource::ResourceSystem*
     particleTemplate.setSizeRange(osgParticle::rangef(15, 180));
     particleTemplate.setColorRange(osgParticle::rangev4(osg::Vec4f(1,1,1,0.7), osg::Vec4f(1,1,1,0.7)));
     particleTemplate.setAlphaRange(osgParticle::rangef(1.f, 0.f));
-    particleTemplate.setAngularVelocity(osg::Vec3f(0,0,fallback->getFallbackFloat("Water_RippleRotSpeed")));
-    particleTemplate.setLifeTime(fallback->getFallbackFloat("Water_RippleLifetime"));
+    particleTemplate.setAngularVelocity(osg::Vec3f(0,0,Fallback::Map::getFloat("Water_RippleRotSpeed")));
+    particleTemplate.setLifeTime(Fallback::Map::getFloat("Water_RippleLifetime"));
 
     osg::ref_ptr<osgParticle::ParticleSystemUpdater> updater (new osgParticle::ParticleSystemUpdater);
     updater->addParticleSystem(mParticleSystem);
 
     mParticleNode = new osg::PositionAttitudeTransform;
+    mParticleNode->setName("Ripple Root");
     mParticleNode->addChild(updater);
     mParticleNode->addChild(mParticleSystem);
-    mParticleNode->setNodeMask(Mask_Effect);
+    mParticleNode->setNodeMask(SceneUtil::Mask_Water);
 
-    createWaterRippleStateSet(resourceSystem, fallback, mParticleNode);
+    createWaterRippleStateSet(resourceSystem, mParticleNode);
+
+    resourceSystem->getSceneManager()->recreateShaders(mParticleNode);
 
     mParent->addChild(mParticleNode);
 }
@@ -118,21 +120,22 @@ RippleSimulation::~RippleSimulation()
 void RippleSimulation::update(float dt)
 {
     const MWBase::World* world = MWBase::Environment::get().getWorld();
-    for (std::vector<Emitter>::iterator it=mEmitters.begin(); it !=mEmitters.end(); ++it)
+    for (Emitter& emitter : mEmitters)
     {
-        if (it->mPtr == MWBase::Environment::get().getWorld ()->getPlayerPtr())
+        MWWorld::ConstPtr& ptr = emitter.mPtr;
+        if (ptr == MWBase::Environment::get().getWorld ()->getPlayerPtr())
         {
             // fetch a new ptr (to handle cell change etc)
             // for non-player actors this is done in updateObjectCell
-            it->mPtr = MWBase::Environment::get().getWorld ()->getPlayerPtr();
+            ptr = MWBase::Environment::get().getWorld ()->getPlayerPtr();
         }
 
-        osg::Vec3f currentPos (it->mPtr.getRefData().getPosition().asVec3());
+        osg::Vec3f currentPos (ptr.getRefData().getPosition().asVec3());
 
-        bool shouldEmit = ( world->isUnderwater (it->mPtr.getCell(), it->mPtr.getRefData().getPosition().asVec3()) && !world->isSubmerged(it->mPtr) ) || world->isWalkingOnWater(it->mPtr);
-        if ( shouldEmit && (currentPos - it->mLastEmitPosition).length() > 10 )
+        bool shouldEmit = (world->isUnderwater(ptr.getCell(), currentPos) && !world->isSubmerged(ptr)) || world->isWalkingOnWater(ptr);
+        if (shouldEmit && (currentPos - emitter.mLastEmitPosition).length() > 10)
         {
-            it->mLastEmitPosition = currentPos;
+            emitter.mLastEmitPosition = currentPos;
 
             currentPos.z() = mParticleNode->getPosition().z();
 
@@ -196,7 +199,7 @@ void RippleSimulation::emitRipple(const osg::Vec3f &pos)
 {
     if (std::abs(pos.z() - mParticleNode->getPosition().z()) < 20)
     {
-        osgParticle::Particle* p = mParticleSystem->createParticle(NULL);
+        osgParticle::Particle* p = mParticleSystem->createParticle(nullptr);
         p->setPosition(osg::Vec3f(pos.x(), pos.y(), 0.f));
         p->setAngle(osg::Vec3f(0,0, Misc::Rng::rollProbability() * osg::PI * 2 - osg::PI));
     }
