@@ -2,11 +2,10 @@
 
 #include <stdexcept>
 
-#include <boost/format.hpp>
-
 #include <MyGUI_LanguageManager.h>
 
-#include <components/compiler/extensions.hpp>
+#include <components/debug/debuglog.hpp>
+
 #include <components/compiler/opcodes.hpp>
 
 #include <components/interpreter/interpreter.hpp>
@@ -20,13 +19,13 @@
 #include "../mwbase/windowmanager.hpp"
 #include "../mwbase/world.hpp"
 
+#include "../mwworld/action.hpp"
 #include "../mwworld/class.hpp"
 #include "../mwworld/containerstore.hpp"
 #include "../mwworld/inventorystore.hpp"
 
 #include "../mwmechanics/actorutil.hpp"
 
-#include "interpretercontext.hpp"
 #include "ref.hpp"
 
 namespace MWScript
@@ -61,7 +60,19 @@ namespace MWScript
                             || ::Misc::StringUtils::ciEqual(item, "gold_100"))
                         item = "gold_001";
 
-                    MWWorld::Ptr itemPtr = *ptr.getClass().getContainerStore (ptr).add (item, count, ptr);
+                    MWWorld::ContainerStore& store = ptr.getClass().getContainerStore (ptr);
+                    // Create a Ptr for the first added item to recover the item name later
+                    MWWorld::Ptr itemPtr = *store.add (item, 1, ptr);
+                    if (itemPtr.getClass().getScript(itemPtr).empty())
+                    {
+                        store.add (item, count-1, ptr);
+                    }
+                    else
+                    {
+                        // Adding just one item per time to make sure there isn't a stack of scripted items
+                        for (int i = 1; i < count; i++)
+                            store.add (item, 1, ptr);
+                    }
 
                     // Spawn a messagebox (only for items added to player's inventory and if player is talking to someone)
                     if (ptr == MWBase::Environment::get().getWorld ()->getPlayerPtr() )
@@ -72,12 +83,12 @@ namespace MWScript
                         if (count == 1)
                         {
                             msgBox = MyGUI::LanguageManager::getInstance().replaceTags("#{sNotifyMessage60}");
-                            msgBox = boost::str(boost::format(msgBox) % itemName);
+                            msgBox = ::Misc::StringUtils::format(msgBox, itemName);
                         }
                         else
                         {
                             msgBox = MyGUI::LanguageManager::getInstance().replaceTags("#{sNotifyMessage61}");
-                            msgBox = boost::str(boost::format(msgBox) % count % itemName);
+                            msgBox = ::Misc::StringUtils::format(msgBox, count, itemName);
                         }
                         MWBase::Environment::get().getWindowManager()->messageBox(msgBox, MWGui::ShowInDialogueMode_Only);
                     }
@@ -140,8 +151,13 @@ namespace MWScript
 
                     std::string itemName;
                     for (MWWorld::ConstContainerStoreIterator iter(store.cbegin()); iter != store.cend(); ++iter)
+                    {
                         if (::Misc::StringUtils::ciEqual(iter->getCellRef().getRefId(), item))
+                        {
                             itemName = iter->getClass().getName(*iter);
+                            break;
+                        }
+                    }
 
                     int numRemoved = store.remove(item, count, ptr);
 
@@ -152,15 +168,15 @@ namespace MWScript
                         // The two GMST entries below expand to strings informing the player of what, and how many of it has been removed from their inventory
                         std::string msgBox;
 
-                        if(numRemoved > 1)
+                        if (numRemoved > 1)
                         {
                             msgBox = MyGUI::LanguageManager::getInstance().replaceTags("#{sNotifyMessage63}");
-                            msgBox = boost::str (boost::format(msgBox) % numRemoved % itemName);
+                            msgBox = ::Misc::StringUtils::format(msgBox, numRemoved, itemName);
                         }
                         else
                         {
                             msgBox = MyGUI::LanguageManager::getInstance().replaceTags("#{sNotifyMessage62}");
-                            msgBox = boost::str (boost::format(msgBox) % itemName);
+                            msgBox = ::Misc::StringUtils::format(msgBox, itemName);
                         }
                         MWBase::Environment::get().getWindowManager()->messageBox(msgBox, MWGui::ShowInDialogueMode_Only);
                     }
@@ -189,16 +205,16 @@ namespace MWScript
                     if (it == invStore.end())
                     {
                         it = ptr.getClass().getContainerStore (ptr).add (item, 1, ptr);
-                        std::cerr << "Implicitly adding one " << item << " to container "
-                            "to fulfil requirements of Equip instruction" << std::endl;
+                        Log(Debug::Warning) << "Implicitly adding one " << item << 
+                            " to the inventory store of " << ptr.getCellRef().getRefId() <<
+                            " to fulfill the requirements of Equip instruction";
                     }
 
-                    if (ptr == MWBase::Environment::get().getWorld()->getPlayerPtr())
-                        MWBase::Environment::get().getWindowManager()->useItem(*it);
+                    if (ptr == MWMechanics::getPlayer())
+                        MWBase::Environment::get().getWindowManager()->useItem(*it, true);
                     else
                     {
-                        std::shared_ptr<MWWorld::Action> action = it->getClass().use(*it);
-                        // No equip sound for actors other than the player
+                        std::shared_ptr<MWWorld::Action> action = it->getClass().use(*it, true);
                         action->execute(ptr, true);
                     }
                 }
@@ -338,11 +354,27 @@ namespace MWScript
 
                     const MWWorld::InventoryStore& invStore = ptr.getClass().getInventoryStore (ptr);
                     MWWorld::ConstContainerStoreIterator it = invStore.getSlot (MWWorld::InventoryStore::Slot_CarriedRight);
-                    if (it == invStore.end() || it->getTypeName () != typeid(ESM::Weapon).name())
+                    if (it == invStore.end())
                     {
                         runtime.push(-1);
                         return;
                     }
+                    else if (it->getTypeName() != typeid(ESM::Weapon).name())
+                    {
+                        if (it->getTypeName() == typeid(ESM::Lockpick).name())
+                        {
+                            runtime.push(-2);
+                        }
+                        else if (it->getTypeName() == typeid(ESM::Probe).name())
+                        {
+                            runtime.push(-3);
+                        }
+                        else
+                        {
+                            runtime.push(-1);
+                        }
+                        return;
+                    }
 
                     runtime.push(it->get<ESM::Weapon>()->mBase->mData.mType);
                 }
