@@ -4,8 +4,6 @@
 #include <stdexcept>
 #include <sstream>
 
-#include <components/interpreter/types.hpp>
-
 #include <components/compiler/locals.hpp>
 
 #include <components/esm/cellid.hpp>
@@ -18,6 +16,7 @@
 #include "../mwbase/windowmanager.hpp"
 #include "../mwbase/inputmanager.hpp"
 
+#include "../mwworld/action.hpp"
 #include "../mwworld/class.hpp"
 #include "../mwworld/cellstore.hpp"
 #include "../mwworld/containerstore.hpp"
@@ -288,10 +287,17 @@ namespace MWScript
         return "None";
     }
 
-    std::string InterpreterContext::getNPCName() const
+    std::string InterpreterContext::getActorName() const
     {
-        ESM::NPC npc = *getReferenceImp().get<ESM::NPC>()->mBase;
-        return npc.mName;
+        const MWWorld::Ptr& ptr = getReferenceImp();
+        if (ptr.getClass().isNpc())
+        {
+            const ESM::NPC* npc = ptr.get<ESM::NPC>()->mBase;
+            return npc->mName;
+        }
+
+        const ESM::Creature* creature = ptr.get<ESM::Creature>()->mBase;
+        return creature->mName;
     }
 
     std::string InterpreterContext::getNPCRace() const
