@@ -1,10 +1,9 @@
-#include <iostream>
+#include <components/debug/debuglog.hpp>
 
 #include <components/sceneutil/positionattitudetransform.hpp>
 
 #include <components/esm/loadcell.hpp>
 
-#include <components/compiler/extensions.hpp>
 #include <components/compiler/opcodes.hpp>
 
 #include <components/interpreter/interpreter.hpp>
@@ -18,7 +17,6 @@
 #include "../mwworld/class.hpp"
 #include "../mwworld/manualref.hpp"
 #include "../mwworld/player.hpp"
-#include "../mwworld/esmstore.hpp"
 
 #include "../mwmechanics/actorutil.hpp"
 
@@ -29,6 +27,18 @@ namespace MWScript
 {
     namespace Transformation
     {
+        void moveStandingActors(const MWWorld::Ptr &ptr, const osg::Vec3f& diff)
+        {
+            std::vector<MWWorld::Ptr> actors;
+            MWBase::Environment::get().getWorld()->getActorsStandingOn (ptr, actors);
+            for (auto& actor : actors)
+            {
+                osg::Vec3f actorPos(actor.getRefData().getPosition().asVec3());
+                actorPos += diff;
+                MWBase::Environment::get().getWorld()->moveObject(actor, actorPos.x(), actorPos.y(), actorPos.z());
+            }
+        }
+
         template<class R>
         class OpSetScale : public Interpreter::Opcode0
         {
@@ -98,8 +108,6 @@ namespace MWScript
                         MWBase::Environment::get().getWorld()->rotateObject(ptr,ax,angle,az);
                     else if (axis == "z")
                         MWBase::Environment::get().getWorld()->rotateObject(ptr,ax,ay,angle);
-                    else
-                        throw std::runtime_error ("invalid rotation axis: " + axis);
                 }
         };
 
@@ -127,8 +135,6 @@ namespace MWScript
                     {
                         runtime.push(osg::RadiansToDegrees(ptr.getCellRef().getPosition().rot[2]));
                     }
-                    else
-                        throw std::runtime_error ("invalid rotation axis: " + axis);
                 }
         };
 
@@ -156,8 +162,6 @@ namespace MWScript
                     {
                         runtime.push(osg::RadiansToDegrees(ptr.getRefData().getPosition().rot[2]));
                     }
-                    else
-                        throw std::runtime_error ("invalid rotation axis: " + axis);
                 }
         };
 
@@ -185,8 +189,6 @@ namespace MWScript
                     {
                         runtime.push(ptr.getRefData().getPosition().pos[2]);
                     }
-                    else
-                        throw std::runtime_error ("invalid axis: " + axis);
                 }
         };
 
@@ -216,11 +218,11 @@ namespace MWScript
                     MWWorld::Ptr updated = ptr;
                     if(axis == "x")
                     {
-                        updated = MWBase::Environment::get().getWorld()->moveObject(ptr,pos,ay,az);
+                        updated = MWBase::Environment::get().getWorld()->moveObject(ptr,pos,ay,az,true);
                     }
                     else if(axis == "y")
                     {
-                        updated = MWBase::Environment::get().getWorld()->moveObject(ptr,ax,pos,az);
+                        updated = MWBase::Environment::get().getWorld()->moveObject(ptr,ax,pos,az,true);
                     }
                     else if(axis == "z")
                     {
@@ -235,10 +237,12 @@ namespace MWScript
                                 pos = terrainHeight;
                         }
 
-                        updated = MWBase::Environment::get().getWorld()->moveObject(ptr,ax,ay,pos);
+                        updated = MWBase::Environment::get().getWorld()->moveObject(ptr,ax,ay,pos,true);
                     }
                     else
-                        throw std::runtime_error ("invalid axis: " + axis);
+                    {
+                        return;
+                    }
 
                     dynamic_cast<MWScript::InterpreterContext&>(runtime.getContext()).updatePtr(ptr,updated);
                 }
@@ -268,8 +272,6 @@ namespace MWScript
                     {
                         runtime.push(ptr.getCellRef().getPosition().pos[2]);
                     }
-                    else
-                        throw std::runtime_error ("invalid axis: " + axis);
                 }
         };
 
@@ -317,7 +319,7 @@ namespace MWScript
                         {
                             std::string error = "Warning: PositionCell: unknown interior cell (" + cellID + "), moving to exterior instead";
                             runtime.getContext().report (error);
-                            std::cerr << error << std::endl;
+                            Log(Debug::Warning) << error;
                         }
                     }
                     if(store)
@@ -378,7 +380,7 @@ namespace MWScript
                     }
                     else
                     {
-                        ptr = MWBase::Environment::get().getWorld()->moveObject(ptr, x, y, z);
+                        ptr = MWBase::Environment::get().getWorld()->moveObject(ptr, x, y, z, true);
                     }
                     dynamic_cast<MWScript::InterpreterContext&>(runtime.getContext()).updatePtr(base,ptr);
 
@@ -429,7 +431,7 @@ namespace MWScript
                         if(!cell)
                         {
                             runtime.getContext().report ("unknown cell (" + cellID + ")");
-                            std::cerr << "unknown cell (" << cellID << ")\n";
+                            Log(Debug::Error) << "Error: unknown cell (" << cellID << ")";
                         }
                     }
                     if(store)
@@ -472,7 +474,7 @@ namespace MWScript
                     if (!player.isInCell())
                         throw std::runtime_error("player not in a cell");
 
-                    MWWorld::CellStore* store = NULL;
+                    MWWorld::CellStore* store = nullptr;
                     if (player.getCell()->isExterior())
                     {
                         int cx,cy;
@@ -560,8 +562,6 @@ namespace MWScript
                         MWBase::Environment::get().getWorld()->rotateObject(ptr,ax,ay+rotation,az);
                     else if (axis == "z")
                         MWBase::Environment::get().getWorld()->rotateObject(ptr,ax,ay,az+rotation);
-                    else
-                        throw std::runtime_error ("invalid rotation axis: " + axis);
                 }
         };
 
@@ -579,26 +579,25 @@ namespace MWScript
                     Interpreter::Type_Float rotation = osg::DegreesToRadians(runtime[0].mFloat*MWBase::Environment::get().getFrameDuration());
                     runtime.pop();
 
-                    const float *objRot = ptr.getRefData().getPosition().rot;
+                    if (!ptr.getRefData().getBaseNode())
+                        return;
 
-                    float ax = objRot[0];
-                    float ay = objRot[1];
-                    float az = objRot[2];
+                    // We can rotate actors only around Z axis
+                    if (ptr.getClass().isActor() && (axis == "x" || axis == "y"))
+                        return;
 
+                    osg::Quat rot;
                     if (axis == "x")
-                    {
-                        MWBase::Environment::get().getWorld()->rotateObject(ptr,ax+rotation,ay,az);
-                    }
+                        rot = osg::Quat(rotation, -osg::X_AXIS);
                     else if (axis == "y")
-                    {
-                        MWBase::Environment::get().getWorld()->rotateObject(ptr,ax,ay+rotation,az);
-                    }
+                        rot = osg::Quat(rotation, -osg::Y_AXIS);
                     else if (axis == "z")
-                    {
-                        MWBase::Environment::get().getWorld()->rotateObject(ptr,ax,ay,az+rotation);
-                    }
+                        rot = osg::Quat(rotation, -osg::Z_AXIS);
                     else
-                        throw std::runtime_error ("invalid rotation axis: " + axis);
+                        return;
+
+                    osg::Quat attitude = ptr.getRefData().getBaseNode()->getAttitude();
+                    MWBase::Environment::get().getWorld()->rotateWorldObject(ptr, attitude * rot);
                 }
         };
 
@@ -658,7 +657,7 @@ namespace MWScript
                         posChange=osg::Vec3f(0, 0, movement);
                     }
                     else
-                        throw std::runtime_error ("invalid movement axis: " + axis);
+                        return;
 
                     // is it correct that disabled objects can't be Move-d?
                     if (!ptr.getRefData().getBaseNode())
@@ -667,6 +666,10 @@ namespace MWScript
                     osg::Vec3f diff = ptr.getRefData().getBaseNode()->getAttitude() * posChange;
                     osg::Vec3f worldPos(ptr.getRefData().getPosition().asVec3());
                     worldPos += diff;
+
+                    // We should move actors, standing on moving object, too.
+                    // This approach can be used to create elevators.
+                    moveStandingActors(ptr, diff);
                     MWBase::Environment::get().getWorld()->moveObject(ptr, worldPos.x(), worldPos.y(), worldPos.z());
                 }
         };
@@ -689,22 +692,21 @@ namespace MWScript
                     runtime.pop();
 
                     const float *objPos = ptr.getRefData().getPosition().pos;
+                    osg::Vec3f diff;
 
-                    MWWorld::Ptr updated;
                     if (axis == "x")
-                    {
-                        updated = MWBase::Environment::get().getWorld()->moveObject(ptr, objPos[0]+movement, objPos[1], objPos[2]);
-                    }
+                        diff.x() += movement;
                     else if (axis == "y")
-                    {
-                        updated = MWBase::Environment::get().getWorld()->moveObject(ptr, objPos[0], objPos[1]+movement, objPos[2]);
-                    }
+                        diff.y() += movement;
                     else if (axis == "z")
-                    {
-                        updated = MWBase::Environment::get().getWorld()->moveObject(ptr, objPos[0], objPos[1], objPos[2]+movement);
-                    }
+                        diff.z() += movement;
                     else
-                        throw std::runtime_error ("invalid movement axis: " + axis);
+                        return;
+
+                    // We should move actors, standing on moving object, too.
+                    // This approach can be used to create elevators.
+                    moveStandingActors(ptr, diff);
+                    MWBase::Environment::get().getWorld()->moveObject(ptr, objPos[0]+diff.x(), objPos[1]+diff.y(), objPos[2]+diff.z());
                 }
         };
 
@@ -718,15 +720,13 @@ namespace MWScript
             }
         };
 
-        template <class R>
         class OpFixme : public Interpreter::Opcode0
         {
         public:
 
             virtual void execute (Interpreter::Runtime& runtime)
             {
-                MWWorld::Ptr ptr = R()(runtime);
-                MWBase::Environment::get().getWorld()->fixPosition(ptr);
+                MWBase::Environment::get().getWorld()->fixPosition();
             }
         };
 
@@ -770,8 +770,7 @@ namespace MWScript
             interpreter.installSegment5(Compiler::Transformation::opcodeGetStartingAngle, new OpGetStartingAngle<ImplicitRef>);
             interpreter.installSegment5(Compiler::Transformation::opcodeGetStartingAngleExplicit, new OpGetStartingAngle<ExplicitRef>);
             interpreter.installSegment5(Compiler::Transformation::opcodeResetActors, new OpResetActors);
-            interpreter.installSegment5(Compiler::Transformation::opcodeFixme, new OpFixme<ImplicitRef>);
-            interpreter.installSegment5(Compiler::Transformation::opcodeFixmeExplicit, new OpFixme<ExplicitRef>);
+            interpreter.installSegment5(Compiler::Transformation::opcodeFixme, new OpFixme);
         }
     }
 }
