@@ -1,7 +1,5 @@
 #include "movieaudiofactory.hpp"
 
-#include <iostream>
-
 #include <extern/osg-ffmpeg-videoplayer/audiodecoder.hpp>
 #include <extern/osg-ffmpeg-videoplayer/videostate.hpp>
 
@@ -19,7 +17,7 @@ namespace MWSound
     {
     public:
         MWSoundDecoderBridge(MWSound::MovieAudioDecoder* decoder)
-            : Sound_Decoder(NULL)
+            : Sound_Decoder(nullptr)
             , mDecoder(decoder)
         {
         }
@@ -48,7 +46,7 @@ namespace MWSound
         {
             ssize_t clock_delay = (mFrameSize-mFramePos) / av_get_channel_layout_nb_channels(mOutputChannelLayout) /
                                   av_get_bytes_per_sample(mOutputSampleFormat);
-            return (size_t)(mAudioClock*mAVStream->codec->sample_rate) - clock_delay;
+            return (size_t)(mAudioClock*mAudioContext->sample_rate) - clock_delay;
         }
 
         std::string getStreamName()
@@ -61,7 +59,7 @@ namespace MWSound
 
         virtual double getAudioClock()
         {
-            return (double)getSampleOffset()/(double)mAVStream->codec->sample_rate -
+            return (double)getSampleOffset()/(double)mAudioContext->sample_rate -
                    MWBase::Environment::get().getSoundManager()->getTrackTimeDelay(mAudioTrack);
         }
 
