@@ -1,18 +1,13 @@
 #include "character.hpp"
 
-#include <ctime>
-
+#include <cctype>
 #include <sstream>
-#include <algorithm>
-#include <stdexcept>
 
 #include <boost/filesystem.hpp>
 
 #include <components/esm/esmreader.hpp>
 #include <components/esm/defs.hpp>
 
-#include <components/misc/stringops.hpp>
-
 bool MWState::operator< (const Slot& left, const Slot& right)
 {
     return left.mTimeStamp<right.mTimeStamp;
@@ -64,10 +59,8 @@ void MWState::Character::addSlot (const ESM::SavedGame& profile)
     int i=0;
     while (boost::filesystem::exists(slot.mPath))
     {
-           std::ostringstream test;
-           test << stream.str();
-           test << " - " << ++i;
-           slot.mPath = mPath / (test.str() + ext);
+        const std::string test = stream.str() + " - " + std::to_string(++i);
+        slot.mPath = mPath / (test + ext);
     }
 
     slot.mProfile = profile;
