@@ -1,5 +1,7 @@
 #include "statemanagerimp.hpp"
 
+#include <components/debug/debuglog.hpp>
+
 #include <components/esm/esmwriter.hpp>
 #include <components/esm/esmreader.hpp>
 #include <components/esm/cellid.hpp>
@@ -30,7 +32,6 @@
 #include "../mwworld/class.hpp"
 #include "../mwworld/cellstore.hpp"
 #include "../mwworld/esmstore.hpp"
-#include "../mwworld/inventorystore.hpp"
 
 #include "../mwmechanics/npcstats.hpp"
 #include "../mwmechanics/actorutil.hpp"
@@ -53,7 +54,7 @@ void MWState::StateManager::cleanup (bool force)
         MWBase::Environment::get().getMechanicsManager()->clear();
 
         mState = State_NoGame;
-        mCharacterManager.setCurrentCharacter(NULL);
+        mCharacterManager.setCurrentCharacter(nullptr);
         mTimePlayed = 0;
 
         MWMechanics::CreatureStats::cleanup();
@@ -157,7 +158,7 @@ void MWState::StateManager::newGame (bool bypass)
         std::stringstream error;
         error << "Failed to start new game: " << e.what();
 
-        std::cerr << error.str() << std::endl;
+        Log(Debug::Error) << error.str();
         cleanup (true);
 
         MWBase::Environment::get().getWindowManager()->pushGuiMode (MWGui::GM_MainMenu);
@@ -173,6 +174,11 @@ void MWState::StateManager::endGame()
     mState = State_Ended;
 }
 
+void MWState::StateManager::resumeGame()
+{
+    mState = State_Running;
+}
+
 void MWState::StateManager::saveGame (const std::string& description, const Slot *slot)
 {
     MWState::Character* character = getCurrentCharacter();
@@ -258,9 +264,10 @@ void MWState::StateManager::saveGame (const std::string& description, const Slot
         writer.save (stream);
 
         Loading::Listener& listener = *MWBase::Environment::get().getWindowManager()->getLoadingScreen();
+        int messagesCount = MWBase::Environment::get().getWindowManager()->getMessagesCount();
         // Using only Cells for progress information, since they typically have the largest records by far
         listener.setProgressRange(MWBase::Environment::get().getWorld()->countSavedGameCells());
-        listener.setLabel("#{sNotifyMessage4}", true);
+        listener.setLabel("#{sNotifyMessage4}", true, messagesCount > 0);
 
         Loading::ScopedLoad load(&listener);
 
@@ -278,7 +285,7 @@ void MWState::StateManager::saveGame (const std::string& description, const Slot
 
         // Ensure we have written the number of records that was estimated
         if (writer.getRecordCount() != recordCount+1) // 1 extra for TES3 record
-            std::cerr << "Warning: number of written savegame records does not match. Estimated: " << recordCount+1 << ", written: " << writer.getRecordCount() << std::endl;
+            Log(Debug::Warning) << "Warning: number of written savegame records does not match. Estimated: " << recordCount+1 << ", written: " << writer.getRecordCount();
 
         writer.close();
 
@@ -300,7 +307,7 @@ void MWState::StateManager::saveGame (const std::string& description, const Slot
         std::stringstream error;
         error << "Failed to save game: " << e.what();
 
-        std::cerr << error.str() << std::endl;
+        Log(Debug::Error) << error.str();
 
         std::vector<std::string> buttons;
         buttons.push_back("#{sOk}");
@@ -382,9 +389,10 @@ void MWState::StateManager::loadGame (const Character *character, const std::str
         std::map<int, int> contentFileMap = buildContentFileIndexMap (reader);
 
         Loading::Listener& listener = *MWBase::Environment::get().getWindowManager()->getLoadingScreen();
+        int messagesCount = MWBase::Environment::get().getWindowManager()->getMessagesCount();
 
         listener.setProgressRange(100);
-        listener.setLabel("#{sLoadingMessage14}");
+        listener.setLabel("#{sLoadingMessage14}", false, messagesCount > 0);
 
         Loading::ScopedLoad load(&listener);
 
@@ -478,7 +486,7 @@ void MWState::StateManager::loadGame (const Character *character, const std::str
                 default:
 
                     // ignore invalid records
-                    std::cerr << "Warning: Ignoring unknown record: " << n.toString() << std::endl;
+                    Log(Debug::Warning) << "Warning: Ignoring unknown record: " << n.toString();
                     reader.skipRecord();
             }
             int progressPercent = static_cast<int>(float(reader.getFileOffset())/total*100);
@@ -544,7 +552,7 @@ void MWState::StateManager::loadGame (const Character *character, const std::str
         std::stringstream error;
         error << "Failed to load saved game: " << e.what();
 
-        std::cerr << error.str() << std::endl;
+        Log(Debug::Error) << error.str();
         cleanup (true);
 
         MWBase::Environment::get().getWindowManager()->pushGuiMode (MWGui::GM_MainMenu);
@@ -620,7 +628,7 @@ bool MWState::StateManager::verifyProfile(const ESM::SavedGame& profile) const
         if (std::find(selectedContentFiles.begin(), selectedContentFiles.end(), *it)
                 == selectedContentFiles.end())
         {
-            std::cerr << "Warning: Savegame dependency " << *it << " is missing." << std::endl;
+            Log(Debug::Warning) << "Warning: Savegame dependency " << *it << " is missing.";
             notFound = true;
         }
     }
@@ -648,7 +656,7 @@ void MWState::StateManager::writeScreenshot(std::vector<char> &imageData) const
     osgDB::ReaderWriter* readerwriter = osgDB::Registry::instance()->getReaderWriterForExtension("jpg");
     if (!readerwriter)
     {
-        std::cerr << "Error: Unable to write screenshot, can't find a jpg ReaderWriter" << std::endl;
+        Log(Debug::Error) << "Error: Unable to write screenshot, can't find a jpg ReaderWriter";
         return;
     }
 
@@ -656,7 +664,7 @@ void MWState::StateManager::writeScreenshot(std::vector<char> &imageData) const
     osgDB::ReaderWriter::WriteResult result = readerwriter->writeImage(*screenshot, ostream);
     if (!result.success())
     {
-        std::cerr << "Error: Unable to write screenshot: " << result.message() << " code " << result.status() << std::endl;
+        Log(Debug::Error) << "Error: Unable to write screenshot: " << result.message() << " code " << result.status();
         return;
     }
 
