@@ -1,18 +1,15 @@
 #include "actionopen.hpp"
 
 #include "../mwbase/environment.hpp"
+#include "../mwbase/mechanicsmanager.hpp"
 #include "../mwbase/windowmanager.hpp"
 
 #include "../mwmechanics/disease.hpp"
 
-#include "class.hpp"
-#include "containerstore.hpp"
-
 namespace MWWorld
 {
-    ActionOpen::ActionOpen (const MWWorld::Ptr& container, bool loot)
+    ActionOpen::ActionOpen (const MWWorld::Ptr& container)
         : Action (false, container)
-        , mLoot(loot)
     {
     }
 
@@ -21,8 +18,14 @@ namespace MWWorld
         if (!MWBase::Environment::get().getWindowManager()->isAllowed(MWGui::GW_Inventory))
             return;
 
+        if (actor != MWMechanics::getPlayer())
+            return;
+
+        if (!MWBase::Environment::get().getMechanicsManager()->onOpen(getTarget()))
+            return;
+
         MWMechanics::diseaseContact(actor, getTarget());
 
-        MWBase::Environment::get().getWindowManager()->openContainer(getTarget(), mLoot);
+        MWBase::Environment::get().getWindowManager()->pushGuiMode(MWGui::GM_Container, getTarget());
     }
 }
