@@ -4,7 +4,6 @@
 #include <string>
 
 #include "action.hpp"
-#include "ptr.hpp"
 
 namespace MWWorld
 {
@@ -18,10 +17,9 @@ namespace MWWorld
         public:
 
             /// @param spellId
-            /// @param actor Actor that activated the trap
             /// @param trapSource
-            ActionTrap (const Ptr& actor, const std::string& spellId, const Ptr& trapSource)
-                : Action(false, actor), mSpellId(spellId), mTrapSource(trapSource) {}
+            ActionTrap (const std::string& spellId, const Ptr& trapSource)
+                : Action(false, trapSource), mSpellId(spellId), mTrapSource(trapSource) {}
     };
 }
 
