@@ -25,6 +25,11 @@ namespace MWWorld
         return mCellRef.mRefID;
     }
 
+    const std::string* CellRef::getRefIdPtr() const
+    {
+        return &mCellRef.mRefID;
+    }
+
     bool CellRef::getTeleport() const
     {
         return mCellRef.mTeleport;
@@ -70,6 +75,22 @@ namespace MWWorld
         return mCellRef.mEnchantmentCharge;
     }
 
+    float CellRef::getNormalizedEnchantmentCharge(int maxCharge) const
+    {
+        if (maxCharge == 0)
+        {
+            return 0;
+        }
+        else if (mCellRef.mEnchantmentCharge == -1)
+        {
+            return 1;
+        }
+        else
+        {
+            return mCellRef.mEnchantmentCharge / static_cast<float>(maxCharge);
+        }
+    }
+
     void CellRef::setEnchantmentCharge(float charge)
     {
         if (charge != mCellRef.mEnchantmentCharge)
@@ -209,6 +230,19 @@ namespace MWWorld
         }
     }
 
+    void CellRef::lock(int lockLevel)
+    {
+        if(lockLevel != 0)
+            setLockLevel(abs(lockLevel)); //Changes lock to locklevel, if positive
+        else
+            setLockLevel(ESM::UnbreakableLock); // If zero, set to max lock level
+    }
+
+    void CellRef::unlock()
+    {
+        setLockLevel(-abs(mCellRef.mLockLevel)); //Makes lockLevel negative
+    }
+
     std::string CellRef::getKey() const
     {
         return mCellRef.mKey;
