@@ -43,6 +43,7 @@ namespace MWWorld
         public:
             virtual void itemAdded(const ConstPtr& item, int count) {}
             virtual void itemRemoved(const ConstPtr& item, int count) {}
+            virtual ~ContainerStoreListener() = default;
     };
 
     class ContainerStore
@@ -68,6 +69,15 @@ namespace MWWorld
 
             static const std::string sGoldId;
 
+        protected:
+            ContainerStoreListener* mListener;
+
+            // (item, max charge)
+            typedef std::vector<std::pair<ContainerStoreIterator, float> > TRechargingItems;
+            TRechargingItems mRechargingItems;
+
+            bool mRechargingItemsUpToDate;
+
         private:
 
             MWWorld::CellRefList<ESM::Potion>            potions;
@@ -87,12 +97,11 @@ namespace MWWorld
             ///< Stores result of levelled item spawns. <(refId, spawningGroup), count>
             /// This is used to restock levelled items(s) if the old item was sold.
 
-            ContainerStoreListener* mListener;
-
             mutable float mCachedWeight;
             mutable bool mWeightUpToDate;
             ContainerStoreIterator addImp (const Ptr& ptr, int count);
             void addInitialItem (const std::string& id, const std::string& owner, int count, bool topLevel=true, const std::string& levItem = "");
+            void addInitialItemImp (const MWWorld::Ptr& ptr, const std::string& owner, int count, bool topLevel=true, const std::string& levItem = "");
 
             template<typename T>
             ContainerStoreIterator getState (CellRefList<T>& collection,
@@ -106,6 +115,7 @@ namespace MWWorld
                 ESM::InventoryState& inventory, int& index,
                 bool equipable = false) const;
 
+            void updateRechargingItems();
 
             virtual void storeEquipmentState (const MWWorld::LiveCellRefBase& ref, int index, ESM::InventoryState& inventory) const;
 
@@ -127,7 +137,9 @@ namespace MWWorld
             ContainerStoreIterator begin (int mask = Type_All);
             ContainerStoreIterator end();
 
-            virtual ContainerStoreIterator add (const Ptr& itemPtr, int count, const Ptr& actorPtr, bool setOwner=false);
+            bool hasVisibleItems() const;
+
+            virtual ContainerStoreIterator add (const Ptr& itemPtr, int count, const Ptr& actorPtr, bool allowAutoEquip = true);
             ///< Add the item pointed to by \a ptr to this container. (Stacks automatically if needed)
             ///
             /// \note The item pointed to is not required to exist beyond this function call.
@@ -135,8 +147,6 @@ namespace MWWorld
             /// \attention Do not add items to an existing stack by increasing the count instead of
             /// calling this function!
             ///
-            /// @param setOwner Set the owner of the added item to \a actorPtr? If false, the owner is reset to "".
-            ///
             /// @return if stacking happened, return iterator to the item that was stacked against, otherwise iterator to the newly inserted item.
 
             ContainerStoreIterator add(const std::string& id, int count, const Ptr& actorPtr);
@@ -152,6 +162,9 @@ namespace MWWorld
             ///
             /// @return the number of items actually removed
 
+            void rechargeItems (float duration);
+            ///< Restore charge on enchanted items. Note this should only be done for the player.
+
             ContainerStoreIterator unstack (const Ptr& ptr, const Ptr& container, int count = 1);
             ///< Unstack an item in this container. The item's count will be set to count, then a new stack will be added with (origCount-count).
             ///
@@ -198,6 +211,9 @@ namespace MWWorld
             ///< This function throws an exception, if ptr does not point to an object, that can be
             /// put into a container.
 
+            Ptr findReplacement(const std::string& id);
+            ///< Returns replacement for object with given id. Prefer used items (with low durability left).
+
             Ptr search (const std::string& id);
 
             virtual void writeState (ESM::InventoryState& state) const;
@@ -329,6 +345,7 @@ namespace MWWorld
             ContainerStoreIteratorBase& operator++ ();
             ContainerStoreIteratorBase operator++ (int);
             ContainerStoreIteratorBase& operator= (const ContainerStoreIteratorBase& rhs);
+            ContainerStoreIteratorBase (const ContainerStoreIteratorBase& rhs) = default;
 
             int getType() const;
             const ContainerStore *getContainerStore() const;
