@@ -1,7 +1,8 @@
 #include "player.hpp"
 
 #include <stdexcept>
-#include <iostream>
+
+#include <components/debug/debuglog.hpp>
 
 #include <components/esm/esmreader.hpp>
 #include <components/esm/esmwriter.hpp>
@@ -10,15 +11,16 @@
 #include <components/esm/loadbsgn.hpp>
 
 #include "../mwworld/esmstore.hpp"
+#include "../mwworld/inventorystore.hpp"
 
 #include "../mwbase/environment.hpp"
 #include "../mwbase/world.hpp"
 #include "../mwbase/windowmanager.hpp"
-#include "../mwbase/soundmanager.hpp"
 #include "../mwbase/mechanicsmanager.hpp"
 
 #include "../mwmechanics/movement.hpp"
 #include "../mwmechanics/npcstats.hpp"
+#include "../mwmechanics/spellcasting.hpp"
 
 #include "class.hpp"
 #include "ptr.hpp"
@@ -29,13 +31,15 @@ namespace MWWorld
     Player::Player (const ESM::NPC *player)
       : mCellStore(0),
         mLastKnownExteriorPosition(0,0,0),
-        mMarkedCell(NULL),
+        mMarkedPosition(ESM::Position()),
+        mMarkedCell(nullptr),
         mAutoMove(false),
         mForwardBackward(0),
         mTeleported(false),
         mCurrentCrimeId(-1),
         mPaidCrimeId(-1),
-        mAttackingOrSpell(false)
+        mAttackingOrSpell(false),
+        mJumping(false)
     {
         ESM::CellRef cellRef;
         cellRef.blank();
@@ -63,7 +67,7 @@ namespace MWWorld
         MWMechanics::CreatureStats& creatureStats = getPlayer().getClass().getCreatureStats(getPlayer());
         MWMechanics::NpcStats& npcStats = getPlayer().getClass().getNpcStats(getPlayer());
         MWMechanics::DynamicStat<float> health = creatureStats.getDynamic(0);
-        creatureStats.setHealth(int(health.getBase() / gmst.find("fWereWolfHealth")->getFloat()));
+        creatureStats.setHealth(int(health.getBase() / gmst.find("fWereWolfHealth")->mValue.getFloat()));
         for (int i=0; i<ESM::Skill::Length; ++i)
             npcStats.setSkill(i, mSaveSkills[i]);
         for (int i=0; i<ESM::Attribute::Length; ++i)
@@ -76,7 +80,7 @@ namespace MWWorld
         MWMechanics::CreatureStats& creatureStats = getPlayer().getClass().getCreatureStats(getPlayer());
         MWMechanics::NpcStats& npcStats = getPlayer().getClass().getNpcStats(getPlayer());
         MWMechanics::DynamicStat<float> health = creatureStats.getDynamic(0);
-        creatureStats.setHealth(int(health.getBase() * gmst.find("fWereWolfHealth")->getFloat()));
+        creatureStats.setHealth(int(health.getBase() * gmst.find("fWereWolfHealth")->mValue.getFloat()));
         for(size_t i = 0;i < ESM::Attribute::Length;++i)
         {
             // Oh, Bethesda. It's "Intelligence".
@@ -84,7 +88,7 @@ namespace MWWorld
                                             ESM::Attribute::sAttributeNames[i]);
 
             MWMechanics::AttributeValue value = npcStats.getAttribute(i);
-            value.setBase(int(gmst.find(name)->getFloat()));
+            value.setBase(int(gmst.find(name)->mValue.getFloat()));
             npcStats.setAttribute(i, value);
         }
 
@@ -99,7 +103,7 @@ namespace MWWorld
                                             ESM::Skill::sSkillNames[i]);
 
             MWMechanics::SkillValue value = npcStats.getSkill(i);
-            value.setBase(int(gmst.find(name)->getFloat()));
+            value.setBase(int(gmst.find(name)->mValue.getFloat()));
             npcStats.setSkill(i, value);
         }
     }
@@ -120,6 +124,12 @@ namespace MWWorld
         return ptr;
     }
 
+    MWWorld::ConstPtr Player::getConstPlayer() const
+    {
+        MWWorld::ConstPtr ptr (&mPlayer, mCellStore);
+        return ptr;
+    }
+
     void Player::setBirthSign (const std::string &sign)
     {
         mSign = sign;
@@ -152,16 +162,16 @@ namespace MWWorld
         if (mAutoMove)
             value = 1;
 
-        ptr.getClass().getMovementSettings(ptr).mPosition[1] = static_cast<float>(value);
+        ptr.getClass().getMovementSettings(ptr).mPosition[1] = value;
     }
 
-    void Player::setLeftRight (int value)
+    void Player::setLeftRight (float value)
     {
         MWWorld::Ptr ptr = getPlayer();
-        ptr.getClass().getMovementSettings(ptr).mPosition[0] = static_cast<float>(value);
+        ptr.getClass().getMovementSettings(ptr).mPosition[0] = value;
     }
 
-    void Player::setForwardBackward (int value)
+    void Player::setForwardBackward (float value)
     {
         MWWorld::Ptr ptr = getPlayer();
 
@@ -170,7 +180,7 @@ namespace MWWorld
         if (mAutoMove)
             value = 1;
 
-        ptr.getClass().getMovementSettings(ptr).mPosition[1] = static_cast<float>(value);
+        ptr.getClass().getMovementSettings(ptr).mPosition[1] = value;
     }
 
     void Player::setUpDown(int value)
@@ -228,7 +238,7 @@ namespace MWWorld
         if (toActivate.isEmpty())
             return;
 
-        if (!toActivate.getClass().canBeActivated(toActivate))
+        if (!toActivate.getClass().hasToolTip(toActivate))
             return;
 
         MWBase::Environment::get().getWorld()->activate(toActivate, player);
@@ -254,6 +264,16 @@ namespace MWWorld
         return mAttackingOrSpell;
     }
 
+    void Player::setJumping(bool jumping)
+    {
+        mJumping = jumping;
+    }
+
+    bool Player::getJumping() const
+    {
+        return mJumping;
+    }
+
     bool Player::isInCombat() {
         return MWBase::Environment::get().getMechanicsManager()->getActorsFighting(getPlayer()).size() != 0;
     }
@@ -285,8 +305,10 @@ namespace MWWorld
         mForwardBackward = 0;
         mTeleported = false;
         mAttackingOrSpell = false;
+        mJumping = false;
         mCurrentCrimeId = -1;
         mPaidCrimeId = -1;
+        mPreviousItems.clear();
         mLastKnownExteriorPosition = osg::Vec3f(0,0,0);
 
         for (int i=0; i<ESM::Skill::Length; ++i)
@@ -341,6 +363,8 @@ namespace MWWorld
         for (int i=0; i<ESM::Skill::Length; ++i)
             mSaveSkills[i].writeState(player.mSaveSkills[i]);
 
+        player.mPreviousItems = mPreviousItems;
+
         writer.startRecord (ESM::REC_PLAY);
         player.save (writer);
         writer.endRecord (ESM::REC_PLAY);
@@ -361,7 +385,7 @@ namespace MWWorld
 
             if (!player.mObject.mEnabled)
             {
-                std::cerr << "Warning: Savegame attempted to disable the player." << std::endl;
+                Log(Debug::Warning) << "Warning: Savegame attempted to disable the player.";
                 player.mObject.mEnabled = true;
             }
 
@@ -388,9 +412,9 @@ namespace MWWorld
             }
             catch (...)
             {
-                std::cerr << "Warning: Player cell '" << player.mCellId.mWorldspace << "' no longer exists" << std::endl;
+                Log(Debug::Warning) << "Warning: Player cell '" << player.mCellId.mWorldspace << "' no longer exists";
                 // Cell no longer exists. The loader will have to choose a default cell.
-                mCellStore = NULL;
+                mCellStore = nullptr;
             }
 
             if (!player.mBirthsign.empty())
@@ -398,14 +422,6 @@ namespace MWWorld
                 const ESM::BirthSign* sign = world.getStore().get<ESM::BirthSign>().search (player.mBirthsign);
                 if (!sign)
                     throw std::runtime_error ("invalid player state record (birthsign does not exist)");
-
-                // To handle the case where a birth sign was edited in between play sessions (does not yet handle removing the old spells)
-                // Also needed for ess-imported savegames which do not specify the birtsign spells in the player's spell list.
-                for (std::vector<std::string>::const_iterator iter (sign->mPowers.mList.begin());
-                    iter!=sign->mPowers.mList.end(); ++iter)
-                {
-                    getPlayer().getClass().getCreatureStats(getPlayer()).getSpells().add (*iter);
-                }
             }
 
             mCurrentCrimeId = player.mCurrentCrimeId;
@@ -441,6 +457,8 @@ namespace MWWorld
             mForwardBackward = 0;
             mTeleported = false;
 
+            mPreviousItems = player.mPreviousItems;
+
             return true;
         }
 
@@ -461,4 +479,29 @@ namespace MWWorld
     {
         return mPaidCrimeId;
     }
+
+    void Player::setPreviousItem(const std::string& boundItemId, const std::string& previousItemId)
+    {
+        mPreviousItems[boundItemId] = previousItemId;
+    }
+
+    std::string Player::getPreviousItem(const std::string& boundItemId)
+    {
+        return mPreviousItems[boundItemId];
+    }
+
+    void Player::erasePreviousItem(const std::string& boundItemId)
+    {
+        mPreviousItems.erase(boundItemId);
+    }
+
+    void Player::setSelectedSpell(const std::string& spellId)
+    {
+        Ptr player = getPlayer();
+        InventoryStore& store = player.getClass().getInventoryStore(player);
+        store.setSelectedEnchantItem(store.end());
+        int castChance = int(MWMechanics::getSpellSuccessChance(spellId, player));
+        MWBase::Environment::get().getWindowManager()->setSelectedSpell(spellId, castChance);
+        MWBase::Environment::get().getWindowManager()->updateSpellWindow();
+    }
 }
