@@ -6,6 +6,7 @@
 
 #include <set>
 #include <memory>
+#include <unordered_map>
 
 namespace osg
 {
@@ -27,6 +28,12 @@ namespace Loading
     class Listener;
 }
 
+namespace DetourNavigator
+{
+    struct Navigator;
+    class Water;
+}
+
 namespace MWRender
 {
     class SkyManager;
@@ -44,6 +51,12 @@ namespace MWWorld
     class CellStore;
     class CellPreloader;
 
+    enum class RotationOrder
+    {
+        direct,
+        inverse
+    };
+
     class Scene
     {
         public:
@@ -57,7 +70,8 @@ namespace MWWorld
             bool mCellChanged;
             MWPhysics::PhysicsSystem *mPhysics;
             MWRender::RenderingManager& mRendering;
-            std::auto_ptr<CellPreloader> mPreloader;
+            DetourNavigator::Navigator& mNavigator;
+            std::unique_ptr<CellPreloader> mPreloader;
             float mPreloadTimer;
             int mHalfGridSize;
             float mCellLoadingThreshold;
@@ -67,30 +81,35 @@ namespace MWWorld
             bool mPreloadExteriorGrid;
             bool mPreloadDoors;
             bool mPreloadFastTravel;
+            float mPredictionTime;
 
-            void insertCell (CellStore &cell, bool rescale, Loading::Listener* loadingListener);
+            osg::Vec3f mLastPlayerPos;
+
+            void insertCell (CellStore &cell, Loading::Listener* loadingListener, bool test = false);
 
             // Load and unload cells as necessary to create a cell grid with "X" and "Y" in the center
-            void changeCellGrid (int X, int Y, bool changeEvent = true);
+            void changeCellGrid (int playerCellX, int playerCellY, bool changeEvent = true);
 
             void getGridCenter(int& cellX, int& cellY);
 
-            void preloadCells();
-            void preloadTeleportDoorDestinations();
-            void preloadExteriorGrid();
-            void preloadFastTravelDestinations();
-
-            void preloadCell(MWWorld::CellStore* cell, bool preloadSurrounding=false);
+            void preloadCells(float dt);
+            void preloadTeleportDoorDestinations(const osg::Vec3f& playerPos, const osg::Vec3f& predictedPos, std::vector<osg::Vec3f>& exteriorPositions);
+            void preloadExteriorGrid(const osg::Vec3f& playerPos, const osg::Vec3f& predictedPos);
+            void preloadFastTravelDestinations(const osg::Vec3f& playerPos, const osg::Vec3f& predictedPos, std::vector<osg::Vec3f>& exteriorPositions);
 
         public:
 
-            Scene (MWRender::RenderingManager& rendering, MWPhysics::PhysicsSystem *physics);
+            Scene (MWRender::RenderingManager& rendering, MWPhysics::PhysicsSystem *physics,
+                   DetourNavigator::Navigator& navigator);
 
             ~Scene();
 
-            void unloadCell (CellStoreCollection::iterator iter);
+            void preloadCell(MWWorld::CellStore* cell, bool preloadSurrounding=false);
+            void preloadTerrain(const osg::Vec3f& pos);
 
-            void loadCell (CellStore *cell, Loading::Listener* loadingListener, bool respawn);
+            void unloadCell (CellStoreCollection::iterator iter, bool test = false);
+
+            void loadCell (CellStore *cell, Loading::Listener* loadingListener, bool respawn, bool test = false);
 
             void playerMoved (const osg::Vec3f& pos);
 
@@ -111,7 +130,7 @@ namespace MWWorld
             ///< Move to exterior cell.
             /// @param changeEvent Set cellChanged flag?
 
-            void changeToVoid();
+            void clear();
             ///< Change into a void
 
             void markCellAsUnchanged();
@@ -124,12 +143,17 @@ namespace MWWorld
             void removeObjectFromScene (const Ptr& ptr);
             ///< Remove an object from the scene, but not from the world model.
 
-            void updateObjectRotation (const Ptr& ptr, bool inverseRotationOrder);
+            void updateObjectRotation(const Ptr& ptr, RotationOrder order);
             void updateObjectScale(const Ptr& ptr);
 
             bool isCellActive(const CellStore &cell);
 
             Ptr searchPtrViaActorId (int actorId);
+
+            void preload(const std::string& mesh, bool useAnim=false);
+
+            void testExteriorCells();
+            void testInteriorCells();
     };
 }
 
