@@ -1,5 +1,7 @@
 #include "store.hpp"
 
+#include <components/debug/debuglog.hpp>
+
 #include <components/esm/esmreader.hpp>
 #include <components/esm/esmwriter.hpp>
 
@@ -7,8 +9,6 @@
 #include <components/misc/rng.hpp>
 
 #include <stdexcept>
-#include <sstream>
-#include <iostream>
 
 namespace
 {
@@ -95,16 +95,16 @@ namespace MWWorld
         typename Static::const_iterator it = mStatic.find(index);
         if (it != mStatic.end())
             return &(it->second);
-        return NULL;
+        return nullptr;
     }
     template<typename T>
     const T *IndexedStore<T>::find(int index) const
     {
         const T *ptr = search(index);
-        if (ptr == 0) {
-            std::ostringstream msg;
-            msg << T::getRecordType() << " with index " << index << " not found";
-            throw std::runtime_error(msg.str());
+        if (ptr == 0)
+        {
+            const std::string msg = T::getRecordType() + " with index " + std::to_string(index) + " not found";
+            throw std::runtime_error(msg);
         }
         return ptr;
     }
@@ -164,16 +164,16 @@ namespace MWWorld
         std::for_each(mShared.begin(), mShared.end(), GetRecords<T>(id, &results));
         if(!results.empty())
             return results[Misc::Rng::rollDice(results.size())];
-        return NULL;
+        return nullptr;
     }
     template<typename T>
     const T *Store<T>::find(const std::string &id) const
     {
         const T *ptr = search(id);
-        if (ptr == 0) {
-            std::ostringstream msg;
-            msg << T::getRecordType() << " '" << id << "' not found";
-            throw std::runtime_error(msg.str());
+        if (ptr == 0)
+        {
+            const std::string msg = T::getRecordType() + " '" + id + "' not found";
+            throw std::runtime_error(msg);
         }
         return ptr;
     }
@@ -183,14 +183,13 @@ namespace MWWorld
         const T *ptr = searchRandom(id);
         if(ptr == 0)
         {
-            std::ostringstream msg;
-            msg << T::getRecordType() << " starting with '"<<id<<"' not found";
-            throw std::runtime_error(msg.str());
+            const std::string msg = T::getRecordType() + " starting with '" + id + "' not found";
+            throw std::runtime_error(msg);
         }
         return ptr;
     }
     template<typename T>
-    RecordId Store<T>::load(ESM::ESMReader &esm) 
+    RecordId Store<T>::load(ESM::ESMReader &esm)
     {
         T record;
         bool isDeleted = false;
@@ -357,16 +356,16 @@ namespace MWWorld
         const LandTextureList &ltexl = mStatic[plugin];
 
         if (index >= ltexl.size())
-            return NULL;
+            return nullptr;
         return &ltexl[index];
     }
     const ESM::LandTexture *Store<ESM::LandTexture>::find(size_t index, size_t plugin) const
     {
         const ESM::LandTexture *ptr = search(index, plugin);
-        if (ptr == 0) {
-            std::ostringstream msg;
-            msg << "Land texture with index " << index << " not found";
-            throw std::runtime_error(msg.str());
+        if (ptr == 0)
+        {
+            const std::string msg = "Land texture with index " + std::to_string(index) + " not found";
+            throw std::runtime_error(msg);
         }
         return ptr;
     }
@@ -388,6 +387,21 @@ namespace MWWorld
 
         assert(plugin < mStatic.size());
 
+        // Replace texture for records with given ID and index from all plugins.
+        for (unsigned int i=0; i<mStatic.size(); i++)
+        {
+            ESM::LandTexture* tex = const_cast<ESM::LandTexture*>(search(lt.mIndex, i));
+            if (tex)
+            {
+                const std::string texId = Misc::StringUtils::lowerCase(tex->mId);
+                const std::string ltId = Misc::StringUtils::lowerCase(lt.mId);
+                if (texId == ltId)
+                {
+                    tex->mTexture = lt.mTexture;
+                }
+            }
+        }
+
         LandTextureList &ltexl = mStatic[plugin];
         if(lt.mIndex + 1 > (int)ltexl.size())
             ltexl.resize(lt.mIndex+1);
@@ -421,10 +435,9 @@ namespace MWWorld
     //=========================================================================
     Store<ESM::Land>::~Store()
     {
-        for (std::vector<ESM::Land *>::const_iterator it =
-                         mStatic.begin(); it != mStatic.end(); ++it)
+        for (const ESM::Land* staticLand : mStatic)
         {
-            delete *it;
+            delete staticLand;
         }
 
     }
@@ -455,10 +468,10 @@ namespace MWWorld
     const ESM::Land *Store<ESM::Land>::find(int x, int y) const
     {
         const ESM::Land *ptr = search(x, y);
-        if (ptr == 0) {
-            std::ostringstream msg;
-            msg << "Land at (" << x << ", " << y << ") not found";
-            throw std::runtime_error(msg.str());
+        if (ptr == 0)
+        {
+            const std::string msg = "Land at (" + std::to_string(x) + ", " + std::to_string(y) + ") not found";
+            throw std::runtime_error(msg);
         }
         return ptr;
     }
@@ -487,7 +500,12 @@ namespace MWWorld
     }
     void Store<ESM::Land>::setUp()
     {
+        // The land is static for given game session, there is no need to refresh it every load.
+        if (mBuilt)
+            return;
+
         std::sort(mStatic.begin(), mStatic.end(), Compare());
+        mBuilt = true;
     }
 
 
@@ -590,20 +608,20 @@ namespace MWWorld
     const ESM::Cell *Store<ESM::Cell>::find(const std::string &id) const
     {
         const ESM::Cell *ptr = search(id);
-        if (ptr == 0) {
-            std::ostringstream msg;
-            msg << "Cell '" << id << "' not found";
-            throw std::runtime_error(msg.str());
+        if (ptr == 0)
+        {
+            const std::string msg = "Cell '" + id + "' not found";
+            throw std::runtime_error(msg);
         }
         return ptr;
     }
     const ESM::Cell *Store<ESM::Cell>::find(int x, int y) const
     {
         const ESM::Cell *ptr = search(x, y);
-        if (ptr == 0) {
-            std::ostringstream msg;
-            msg << "Exterior at (" << x << ", " << y << ") not found";
-            throw std::runtime_error(msg.str());
+        if (ptr == 0)
+        {
+            const std::string msg = "Exterior at (" + std::to_string(x) + ", " + std::to_string(y) + ") not found";
+            throw std::runtime_error(msg);
         }
         return ptr;
     }
@@ -692,7 +710,7 @@ namespace MWWorld
                             if (it_lease != wipecell->mLeasedRefs.end())
                                 wipecell->mLeasedRefs.erase(it_lease);
                             else
-                                std::cerr << "Error: can't find " << it->mRefNum.mIndex << " " << it->mRefNum.mContentFile  << " in leasedRefs " << std::endl;
+                                Log(Debug::Error) << "Error: can't find " << it->mRefNum.mIndex << " " << it->mRefNum.mContentFile  << " in leasedRefs";
                         }
                         *itold = *it;
                     }
@@ -738,15 +756,16 @@ namespace MWWorld
     }
     const ESM::Cell *Store<ESM::Cell>::searchExtByName(const std::string &id) const
     {
-        ESM::Cell *cell = 0;
-        std::vector<ESM::Cell *>::const_iterator it = mSharedExt.begin();
-        for (; it != mSharedExt.end(); ++it) {
-            if (Misc::StringUtils::ciEqual((*it)->mName, id)) {
-                if ( cell == 0 ||
-                    ( (*it)->mData.mX > cell->mData.mX ) ||
-                    ( (*it)->mData.mX == cell->mData.mX && (*it)->mData.mY > cell->mData.mY ) )
+        const ESM::Cell *cell = nullptr;
+        for (const ESM::Cell *sharedCell : mSharedExt)
+        {
+            if (Misc::StringUtils::ciEqual(sharedCell->mName, id))
+            {
+                if (cell == 0 ||
+                    (sharedCell->mData.mX > cell->mData.mX) ||
+                    (sharedCell->mData.mX == cell->mData.mX && sharedCell->mData.mY > cell->mData.mY))
                 {
-                    cell = *it;
+                    cell = sharedCell;
                 }
             }
         }
@@ -754,15 +773,16 @@ namespace MWWorld
     }
     const ESM::Cell *Store<ESM::Cell>::searchExtByRegion(const std::string &id) const
     {
-        ESM::Cell *cell = 0;
-        std::vector<ESM::Cell *>::const_iterator it = mSharedExt.begin();
-        for (; it != mSharedExt.end(); ++it) {
-            if (Misc::StringUtils::ciEqual((*it)->mRegion, id)) {
-                if ( cell == 0 ||
-                    ( (*it)->mData.mX > cell->mData.mX ) ||
-                    ( (*it)->mData.mX == cell->mData.mX && (*it)->mData.mY > cell->mData.mY ) )
+        const ESM::Cell *cell = nullptr;
+        for (const ESM::Cell *sharedCell : mSharedExt)
+        {
+            if (Misc::StringUtils::ciEqual(sharedCell->mRegion, id))
+            {
+                if (cell == nullptr ||
+                    (sharedCell->mData.mX > cell->mData.mX) ||
+                    (sharedCell->mData.mX == cell->mData.mX && sharedCell->mData.mY > cell->mData.mY))
                 {
-                    cell = *it;
+                    cell = sharedCell;
                 }
             }
         }
@@ -772,24 +792,29 @@ namespace MWWorld
     {
         return mSharedInt.size() + mSharedExt.size();
     }
+    size_t Store<ESM::Cell>::getExtSize() const
+    {
+        return mSharedExt.size();
+    }
+    size_t Store<ESM::Cell>::getIntSize() const
+    {
+        return mSharedInt.size();
+    }
     void Store<ESM::Cell>::listIdentifier(std::vector<std::string> &list) const
     {
         list.reserve(list.size() + mSharedInt.size());
 
-        std::vector<ESM::Cell *>::const_iterator it = mSharedInt.begin();
-        for (; it != mSharedInt.end(); ++it) {
-            list.push_back((*it)->mName);
+        for (const ESM::Cell *sharedCell : mSharedInt)
+        {
+            list.push_back(sharedCell->mName);
         }
     }
     ESM::Cell *Store<ESM::Cell>::insert(const ESM::Cell &cell)
     {
-        if (search(cell) != 0) {
-            std::ostringstream msg;
-            msg << "Failed to create ";
-            msg << ((cell.isExterior()) ? "exterior" : "interior");
-            msg << " cell";
-
-            throw std::runtime_error(msg.str());
+        if (search(cell) != 0)
+        {
+            const std::string cellType = (cell.isExterior()) ? "exterior" : "interior";
+            throw std::runtime_error("Failed to create " + cellType + " cell");
         }
         ESM::Cell *ptr;
         if (cell.isExterior()) {
@@ -866,7 +891,7 @@ namespace MWWorld
     //=========================================================================
 
     Store<ESM::Pathgrid>::Store()
-        : mCells(NULL)
+        : mCells(nullptr)
     {
     }
 
@@ -886,7 +911,7 @@ namespace MWWorld
         // mX and mY will be (0,0) for interior cells, but there is also an exterior cell with the coordinates of (0,0), so that doesn't help.
         // Check whether mCell is an interior cell. This isn't perfect, will break if a Region with the same name as an interior cell is created.
         // A proper fix should be made for future versions of the file format.
-        bool interior = mCells->search(pathgrid.mCell) != NULL;
+        bool interior = mCells->search(pathgrid.mCell) != nullptr;
 
         // Try to overwrite existing record
         if (interior)
@@ -916,23 +941,22 @@ namespace MWWorld
         Exterior::const_iterator it = mExt.find(std::make_pair(x,y));
         if (it != mExt.end())
             return &(it->second);
-        return NULL;
+        return nullptr;
     }
     const ESM::Pathgrid *Store<ESM::Pathgrid>::search(const std::string& name) const
     {
         Interior::const_iterator it = mInt.find(name);
         if (it != mInt.end())
             return &(it->second);
-        return NULL;
+        return nullptr;
     }
     const ESM::Pathgrid *Store<ESM::Pathgrid>::find(int x, int y) const
     {
         const ESM::Pathgrid* pathgrid = search(x,y);
         if (!pathgrid)
         {
-            std::ostringstream msg;
-            msg << "Pathgrid in cell '" << x << " " << y << "' not found";
-            throw std::runtime_error(msg.str());
+            const std::string msg = "Pathgrid in cell '" + std::to_string(x) + " " + std::to_string(y) + "' not found";
+            throw std::runtime_error(msg);
         }
         return pathgrid;
     }
@@ -941,9 +965,8 @@ namespace MWWorld
         const ESM::Pathgrid* pathgrid = search(name);
         if (!pathgrid)
         {
-            std::ostringstream msg;
-            msg << "Pathgrid in cell '" << name << "' not found";
-            throw std::runtime_error(msg.str());
+            const std::string msg = "Pathgrid in cell '" + name + "' not found";
+            throw std::runtime_error(msg);
         }
         return pathgrid;
     }
@@ -997,23 +1020,22 @@ namespace MWWorld
     const ESM::Attribute *Store<ESM::Attribute>::find(size_t index) const
     {
         const ESM::Attribute *ptr = search(index);
-        if (ptr == 0) {
-            std::ostringstream msg;
-            msg << "Attribute with index " << index << " not found";
-            throw std::runtime_error(msg.str());
+        if (ptr == 0)
+        {
+            const std::string msg = "Attribute with index " + std::to_string(index) + " not found";
+            throw std::runtime_error(msg);
         }
         return ptr;
     }
     void Store<ESM::Attribute>::setUp()
     {
-        for (int i = 0; i < ESM::Attribute::Length; ++i) {
-            mStatic.push_back(
-                ESM::Attribute(
-                    ESM::Attribute::sAttributeIds[i],
-                    ESM::Attribute::sGmstAttributeIds[i],
-                    ESM::Attribute::sGmstAttributeDescIds[i]
-                )
-            );
+        for (int i = 0; i < ESM::Attribute::Length; ++i) 
+        {
+            ESM::Attribute newAttribute;
+            newAttribute.mId = ESM::Attribute::sAttributeIds[i];
+            newAttribute.mName = ESM::Attribute::sGmstAttributeIds[i];
+            newAttribute.mDescription = ESM::Attribute::sGmstAttributeDescIds[i];
+            mStatic.push_back(newAttribute);
         }
     }
     size_t Store<ESM::Attribute>::getSize() const
@@ -1053,32 +1075,6 @@ namespace MWWorld
         }
     }
 
-    template<>
-    void Store<ESM::Static>::setUp()
-    {
-        // Load default marker definitions, if game files do not have them for some reason
-        std::pair<std::string, std::string> markers[] = {
-            std::make_pair("divinemarker", "marker_divine.nif"),
-            std::make_pair("doormarker", "marker_arrow.nif"),
-            std::make_pair("northmarker", "marker_north.nif"),
-            std::make_pair("templemarker", "marker_temple.nif"),
-            std::make_pair("travelmarker", "marker_travel.nif")
-        };
-
-        for (const std::pair<std::string, std::string> marker : markers)
-        {
-            if (search(marker.first) == 0)
-            {
-                ESM::Static newMarker = ESM::Static(marker.first, marker.second);
-                std::pair<typename Static::iterator, bool> ret = mStatic.insert(std::make_pair(marker.first, newMarker));
-                if (ret.first != mStatic.end())
-                {
-                    mShared.push_back(&ret.first->second);
-                }
-            }
-        }
-    }
-
     template <>
     inline RecordId Store<ESM::Dialogue>::load(ESM::ESMReader &esm) {
         // The original letter case of a dialogue ID is saved, because it's printed
@@ -1103,6 +1099,18 @@ namespace MWWorld
         return RecordId(dialogue.mId, isDeleted);
     }
 
+    template<>
+    bool Store<ESM::Dialogue>::eraseStatic(const std::string &id)
+    {
+        auto it = mStatic.find(Misc::StringUtils::lowerCase(id));
+
+        if (it != mStatic.end() && Misc::StringUtils::ciEqual(it->second.mId, id)) {
+            mStatic.erase(it);
+        }
+
+        return true;
+    }
+
 }
 
 template class MWWorld::Store<ESM::Activator>;
