@@ -7,6 +7,8 @@
 
 #include <osg/Vec4f>
 
+#include <components/fallback/fallback.hpp>
+
 #include "../mwbase/soundmanager.hpp"
 
 #include "../mwrender/sky.hpp"
@@ -38,6 +40,20 @@ namespace MWWorld
 {
     class TimeStamp;
 
+    enum NightDayMode
+    {
+        Default = 0,
+        ExteriorNight = 1,
+        InteriorDay = 2
+    };
+
+    struct WeatherSetting
+    {
+        float mPreSunriseTime;
+        float mPostSunriseTime;
+        float mPreSunsetTime;
+        float mPostSunsetTime;
+    };
 
     struct TimeOfDaySettings
     {
@@ -45,7 +61,37 @@ namespace MWWorld
         float mNightEnd;
         float mDayStart;
         float mDayEnd;
-        float mSunriseTime;
+
+        std::map<std::string, WeatherSetting> mSunriseTransitions;
+
+        float mStarsPostSunsetStart;
+        float mStarsPreSunriseFinish;
+        float mStarsFadingDuration;
+
+        WeatherSetting getSetting(const std::string& type) const
+        {
+            std::map<std::string, WeatherSetting>::const_iterator it = mSunriseTransitions.find(type);
+            if (it != mSunriseTransitions.end())
+            {
+                return it->second;
+            }
+            else
+            {
+                return { 1.f, 1.f, 1.f, 1.f };
+            }
+        }
+
+        void addSetting(const std::string& type)
+        {
+            WeatherSetting setting = {
+                Fallback::Map::getFloat("Weather_" + type + "_Pre-Sunrise_Time"),
+                Fallback::Map::getFloat("Weather_" + type + "_Post-Sunrise_Time"),
+                Fallback::Map::getFloat("Weather_" + type + "_Pre-Sunset_Time"),
+                Fallback::Map::getFloat("Weather_" + type + "_Post-Sunset_Time")
+            };
+
+            mSunriseTransitions[type] = setting;
+        }
     };
 
     /// Interpolates between 4 data points (sunrise, day, sunset, night) based on the time of day.
@@ -59,7 +105,7 @@ namespace MWWorld
         {
         }
 
-        T getValue (const float gameHour, const TimeOfDaySettings& timeSettings) const;
+        T getValue (const float gameHour, const TimeOfDaySettings& timeSettings, const std::string& prefix) const;
 
     private:
         T mSunriseValue, mDayValue, mSunsetValue, mNightValue;
@@ -70,7 +116,6 @@ namespace MWWorld
     {
     public:
         Weather(const std::string& name,
-                const Fallback::Map& fallback,
                 float stormWindSpeed,
                 float rainSpeed,
                 float dlFactor,
@@ -125,7 +170,20 @@ namespace MWWorld
         float mRainSpeed;
 
         // How often does a new rain mesh spawn?
-        float mRainFrequency;
+        float mRainEntranceSpeed;
+
+        // Maximum count of rain particles
+        int mRainMaxRaindrops;
+
+        // Radius of rain effect
+        float mRainDiameter;
+
+        // Transition threshold to spawn rain
+        float mRainThreshold;
+
+        // Height of rain particles spawn
+        float mRainMinHeight;
+        float mRainMaxHeight;
 
         std::string mParticleEffect;
 
@@ -184,7 +242,7 @@ namespace MWWorld
     class MoonModel
     {
     public:
-        MoonModel(const std::string& name, const Fallback::Map& fallback);
+        MoonModel(const std::string& name);
 
         MWRender::MoonState calculateState(const TimeStamp& gameTime) const;
 
@@ -214,9 +272,7 @@ namespace MWWorld
     {
     public:
         // Have to pass fallback and Store, can't use singleton since World isn't fully constructed yet at the time
-        WeatherManager(MWRender::RenderingManager& rendering,
-                       const Fallback::Map& fallback,
-                       MWWorld::ESMStore& store);
+        WeatherManager(MWRender::RenderingManager& rendering, MWWorld::ESMStore& store);
         ~WeatherManager();
 
         /**
@@ -238,6 +294,7 @@ namespace MWWorld
         void stopSounds();
 
         float getWindSpeed() const;
+        NightDayMode getNightDayMode() const;
 
         /// Are we in an ash or blight storm?
         bool isInStorm() const;
@@ -281,6 +338,8 @@ namespace MWWorld
         MoonModel mSecunda;
 
         float mWindSpeed;
+        float mCurrentWindSpeed;
+        float mNextWindSpeed;
         bool mIsStorm;
         bool mPrecipitation;
         osg::Vec3f mStormDirection;
@@ -290,6 +349,7 @@ namespace MWWorld
         bool mFastForward;
         float mWeatherUpdateTime;
         float mTransitionFactor;
+        NightDayMode mNightDayMode;
         int mCurrentWeather;
         int mNextWeather;
         int mQueuedWeather;
@@ -300,7 +360,6 @@ namespace MWWorld
         std::string mPlayingSoundID;
 
         void addWeather(const std::string& name,
-                        const Fallback::Map& fallback,
                         float dlFactor, float dlOffset,
                         const std::string& particleEffect = "");
 
@@ -318,6 +377,7 @@ namespace MWWorld
         void calculateWeatherResult(const float gameHour, const float elapsedSeconds, const bool isPaused);
         void calculateResult(const int weatherID, const float gameHour);
         void calculateTransitionResult(const float factor, const float gameHour);
+        float calculateWindSpeed(int weatherId, float currentSpeed);
     };
 }
 
