@@ -62,10 +62,11 @@ Wizard::MainWizard::MainWizard(QWidget *parent) :
     setupInstallations();
     setupPages();
 
-    const boost::filesystem::path& installedPath = mCfgMgr.getInstallPath();
-    if (!installedPath.empty())
+    const boost::filesystem::path& installationPath = mCfgMgr.getInstallPath();
+    if (!installationPath.empty())
     {
-        addInstallation(toQString(installedPath));
+        const boost::filesystem::path& dataPath = installationPath / "Data Files";
+        addInstallation(toQString(dataPath));
     }
 }
 
@@ -161,7 +162,8 @@ void Wizard::MainWizard::setupGameSettings()
     paths.append(QLatin1String("openmw.cfg"));
     paths.append(globalPath + QLatin1String("openmw.cfg"));
 
-    foreach (const QString &path2, paths) {
+    for (const QString &path2 : paths)
+    {
         qDebug() << "Loading config file:" << path2.toUtf8().constData();
 
         file.setFileName(path2);
@@ -221,7 +223,8 @@ void Wizard::MainWizard::setupLauncherSettings()
 void Wizard::MainWizard::setupInstallations()
 {
     // Check if the paths actually contain a Morrowind installation
-    foreach (const QString path, mGameSettings.getDataDirs()) {
+    for (const QString& path : mGameSettings.getDataDirs())
+    {
 
         if (findFiles(QLatin1String("Morrowind"), path))
             addInstallation(path);
@@ -230,29 +233,13 @@ void Wizard::MainWizard::setupInstallations()
 
 void Wizard::MainWizard::runSettingsImporter()
 {
+    writeSettings();
+
     QString path(field(QLatin1String("installation.path")).toString());
 
-    // Create the file if it doesn't already exist, else the importer will fail
     QString userPath(toQString(mCfgMgr.getUserConfigPath()));
     QFile file(userPath + QLatin1String("openmw.cfg"));
 
-    if (!file.exists()) {
-        if (!file.open(QIODevice::ReadWrite)) {
-            // File cannot be created
-            QMessageBox msgBox;
-            msgBox.setWindowTitle(tr("Error writing OpenMW configuration file"));
-            msgBox.setIcon(QMessageBox::Critical);
-            msgBox.setStandardButtons(QMessageBox::Ok);
-            msgBox.setText(tr("<html><head/><body><p><b>Could not open or create %1 for writing</b></p> \
-                              <p>Please make sure you have the right permissions \
-                              and try again.</p></body></html>").arg(file.fileName()));
-            msgBox.exec();
-            return qApp->quit();
-        }
-
-        file.close();
-    }
-
     // Construct the arguments to run the importer
     QStringList arguments;
 
