@@ -2,7 +2,7 @@
   OpenMW - The completely unofficial reimplementation of Morrowind
   Copyright (C) 2008-2010  Nicolay Korslund
   Email: < korslund@gmail.com >
-  WWW: http://openmw.sourceforge.net/
+  WWW: https://openmw.org/
 
   This file (bsa_file.h) is part of the OpenMW package.
 
@@ -17,7 +17,7 @@
 
   You should have received a copy of the GNU General Public License
   version 3 along with this program. If not, see
-  http://www.gnu.org/licenses/ .
+  https://www.gnu.org/licenses/ .
 
  */
 
@@ -56,18 +56,18 @@ public:
     };
     typedef std::vector<FileStruct> FileList;
 
-private:
+protected:
     /// Table of files in this archive
-    FileList files;
+    FileList mFiles;
 
     /// Filename string buffer
-    std::vector<char> stringBuf;
+    std::vector<char> mStringBuf;
 
     /// True when an archive has been loaded
-    bool isLoaded;
+    bool mIsLoaded;
 
     /// Used for error messages
-    std::string filename;
+    std::string mFilename;
 
     /// Case insensitive string comparison
     struct iltstr
@@ -81,13 +81,16 @@ private:
         checks are case insensitive.
     */
     typedef std::map<const char*, int, iltstr> Lookup;
-    Lookup lookup;
+    Lookup mLookup;
 
     /// Error handling
     void fail(const std::string &msg);
 
     /// Read header information from the input source
-    void readHeader();
+    virtual void readHeader();
+
+    /// Read header information from the input source
+
 
     /// Get the index of a given file name, or -1 if not found
     /// @note Thread safe.
@@ -100,7 +103,10 @@ public:
      */
 
     BSAFile()
-      : isLoaded(false)
+      : mIsLoaded(false)
+    { }
+
+    virtual ~BSAFile()
     { }
 
     /// Open an archive file.
@@ -112,24 +118,24 @@ public:
      */
 
     /// Check if a file exists
-    bool exists(const char *file) const
+    virtual bool exists(const char *file) const
     { return getIndex(file) != -1; }
 
     /** Open a file contained in the archive. Throws an exception if the
         file doesn't exist.
      * @note Thread safe.
     */
-    Files::IStreamPtr getFile(const char *file);
+    virtual Files::IStreamPtr getFile(const char *file);
 
     /** Open a file contained in the archive.
      * @note Thread safe.
     */
-    Files::IStreamPtr getFile(const FileStruct* file);
+    virtual Files::IStreamPtr getFile(const FileStruct* file);
 
     /// Get a list of all files
     /// @note Thread safe.
     const FileList &getList() const
-    { return files; }
+    { return mFiles; }
 };
 
 }
