@@ -5,6 +5,7 @@
 #include <algorithm>
 #include <stack>
 #include <iterator>
+#include <sstream>
 
 #include <components/misc/stringops.hpp>
 
@@ -99,7 +100,7 @@ namespace Compiler
         else if (t1=='f' || t2=='f')
             mOperands.push_back ('f');
         else
-            throw std::logic_error ("failed to determine result operand type");
+            throw std::logic_error ("Failed to determine result operand type");
     }
 
     void ExprParser::pop()
@@ -158,7 +159,7 @@ namespace Compiler
 
             default:
 
-                throw std::logic_error ("unknown operator");
+                throw std::logic_error ("Unknown operator");
         }
     }
 
@@ -277,10 +278,18 @@ namespace Compiler
     {
         if (!mExplicit.empty())
         {
-            if (mMemberOp && handleMemberAccess (name))
-                return true;
+            if (!mRefOp)
+            {
+                if (mMemberOp && handleMemberAccess (name))
+                    return true;
 
-            return Parser::parseName (name, loc, scanner);
+                return Parser::parseName (name, loc, scanner);
+            }
+            else
+            {
+                mExplicit.clear();
+                getErrorHandler().warning ("Stray explicit reference", loc);
+            }
         }
 
         mFirst = false;
@@ -316,6 +325,21 @@ namespace Compiler
                 mExplicit = name2;
                 return true;
             }
+
+            // This is terrible, but of course we must have this for legacy content.
+            // Convert the string to a number even if it's impossible and use it as a number literal.
+            // Can't use stof/atof or to_string out of locale concerns.
+            float number;
+            std::stringstream stream(name2);
+            stream >> number;
+            stream.str(std::string());
+            stream.clear();
+            stream << number;
+
+            pushFloatLiteral(number);
+            mTokenLoc = loc;
+            getErrorHandler().warning ("Parsing a non-variable string as a number: " + stream.str(), loc);
+            return true;
         }
         else
         {
@@ -323,8 +347,6 @@ namespace Compiler
             scanner.putbackName (name, loc);
             return false;
         }
-
-        return Parser::parseName (name, loc, scanner);
     }
 
     bool ExprParser::parseKeyword (int keyword, const TokenLoc& loc, Scanner& scanner)
@@ -422,7 +444,7 @@ namespace Compiler
                     {
                         if (!hasExplicit)
                         {
-                            getErrorHandler().warning ("stray explicit reference (ignoring it)", loc);
+                            getErrorHandler().warning ("Stray explicit reference", loc);
                             mExplicit.clear();
                         }
 
@@ -481,7 +503,7 @@ namespace Compiler
                 parseArguments ("l", scanner);
 
                 Generator::random (mCode);
-                mOperands.push_back ('l');
+                mOperands.push_back ('f');
 
                 mNextOperand = false;
                 return true;
@@ -727,13 +749,13 @@ namespace Compiler
     {
         if (mOperands.empty() && mOperators.empty())
         {
-            getErrorHandler().error ("missing expression", mTokenLoc);
+            getErrorHandler().error ("Missing expression", mTokenLoc);
             return 'l';
         }
 
         if (mNextOperand || mOperands.empty())
         {
-            getErrorHandler().error ("syntax error in expression", mTokenLoc);
+            getErrorHandler().error ("Syntax error in expression", mTokenLoc);
             return 'l';
         }
 
@@ -791,7 +813,7 @@ namespace Compiler
                         ++optionalCount;
                 }
                 else
-                    getErrorHandler().warning ("Ignoring extra argument",
+                    getErrorHandler().warning ("Extra argument",
                         stringParser.getTokenLoc());
             }
             else if (*iter=='X')
@@ -805,7 +827,7 @@ namespace Compiler
                 if (parser.isEmpty())
                     break;
                 else
-                    getErrorHandler().warning("Ignoring extra argument", parser.getTokenLoc());
+                    getErrorHandler().warning("Extra argument", parser.getTokenLoc());
             }
             else if (*iter=='z')
             {
@@ -817,7 +839,7 @@ namespace Compiler
                 if (discardParser.isEmpty())
                     break;
                 else
-                    getErrorHandler().warning("Ignoring extra argument", discardParser.getTokenLoc());
+                    getErrorHandler().warning("Extra argument", discardParser.getTokenLoc());
             }
             else if (*iter=='j')
             {
