@@ -7,12 +7,14 @@
 
 namespace Compiler
 {
-    /// \brief Error handler implementation: Write errors into stream
+    class ContextOverride;
+    /// \brief Error handler implementation: Write errors into logging stream
 
     class StreamErrorHandler : public ErrorHandler
     {
-            std::ostream& mStream;
+            std::string mContext;
 
+            friend class ContextOverride;
         // not implemented
 
             StreamErrorHandler (const StreamErrorHandler&);
@@ -26,11 +28,26 @@ namespace Compiler
 
         public:
 
+            void setContext(const std::string& context);
+
         // constructors
 
-            StreamErrorHandler (std::ostream& ErrorStream);
+            StreamErrorHandler ();
             ///< constructor
     };
+
+    class ContextOverride
+    {
+            StreamErrorHandler& mHandler;
+            const std::string mContext;
+        public:
+            ContextOverride (StreamErrorHandler& handler, const std::string& context);
+
+            ContextOverride (const ContextOverride&) = delete;
+            ContextOverride& operator= (const ContextOverride&) = delete;
+
+            ~ContextOverride();
+    };
 }
 
 #endif
