@@ -1,6 +1,7 @@
 #ifndef OPENMW_ESM_CELLREF_H
 #define OPENMW_ESM_CELLREF_H
 
+#include <limits>
 #include <string>
 
 #include "defs.hpp"
@@ -10,6 +11,7 @@ namespace ESM
     class ESMWriter;
     class ESMReader;
 
+    const int UnbreakableLock = std::numeric_limits<int>::max();
 
     struct RefNum
     {
