@@ -14,6 +14,15 @@ namespace ESM
 
         virtual void load (ESMReader &esm);
         virtual void save (ESMWriter &esm, bool inInventory = false) const;
+
+        virtual CreatureLevListState& asCreatureLevListState()
+        {
+            return *this;
+        }
+        virtual const CreatureLevListState& asCreatureLevListState() const
+        {
+            return *this;
+        }
     };
 }
 
