@@ -78,18 +78,19 @@ struct Cell
 
   struct DATAstruct
   {
-    int mFlags;
-    int mX, mY;
+      int mFlags {0};
+      int mX {0}, mY {0};
   };
 
   struct AMBIstruct
   {
-    Color mAmbient, mSunlight, mFog;
-    float mFogDensity;
+      Color mAmbient {0}, mSunlight {0}, mFog {0};
+      float mFogDensity {0.f};
   };
 
   Cell() : mName(""),
            mRegion(""),
+           mHasAmbi(true),
            mWater(0),
            mWaterInt(false),
            mMapColor(0),
@@ -108,6 +109,7 @@ struct Cell
   CellId mCellId;
 
   AMBIstruct mAmbi;
+  bool mHasAmbi;
 
   float mWater; // Water level
   bool mWaterInt;
@@ -152,6 +154,16 @@ struct Cell
       return ((mData.mFlags&HasWater) != 0) || isExterior();
   }
 
+  bool hasAmbient() const
+  {
+      return mHasAmbi;
+  }
+
+  void setHasAmbient(bool hasAmbi)
+  {
+      mHasAmbi = hasAmbi;
+  }
+
   // Restore the given reader to the stored position. Will try to open
   // the file matching the stored file name. If you want to read from
   // somewhere other than the file system, you need to pre-open the
