@@ -12,6 +12,11 @@ namespace ESM
 {
     class ESMReader;
     class ESMWriter;
+    struct ContainerState;
+    struct CreatureLevListState;
+    struct CreatureState;
+    struct DoorState;
+    struct NpcState;
 
     // format 0, saved games only
 
@@ -34,7 +39,9 @@ namespace ESM
 
         ESM::AnimationState mAnimationState;
 
-        ObjectState() : mHasCustomState(true), mVersion(0)
+        ObjectState()
+        : mHasLocals(0), mEnabled(0), mCount(0)
+        , mFlags(0), mHasCustomState(true), mVersion(0)
         {}
 
         /// @note Does not load the CellRef ID, it should already be loaded before calling this method
@@ -46,6 +53,21 @@ namespace ESM
         void blank();
 
         virtual ~ObjectState();
+
+        virtual const NpcState& asNpcState() const;
+        virtual NpcState& asNpcState();
+
+        virtual const CreatureState& asCreatureState() const;
+        virtual CreatureState& asCreatureState();
+
+        virtual const ContainerState& asContainerState() const;
+        virtual ContainerState& asContainerState();
+
+        virtual const DoorState& asDoorState() const;
+        virtual DoorState& asDoorState();
+
+        virtual const CreatureLevListState& asCreatureLevListState() const;
+        virtual CreatureLevListState& asCreatureLevListState();
     };
 }
 
