@@ -34,6 +34,9 @@ namespace ESM
         StatState<int> mSaveAttributes[ESM::Attribute::Length];
         StatState<int> mSaveSkills[ESM::Skill::Length];
 
+        typedef std::map<std::string, std::string> PreviousItems; // previous equipped items, needed for bound spells
+        PreviousItems mPreviousItems;
+
         void load (ESMReader &esm);
         void save (ESMWriter &esm) const;
     };
