@@ -1,16 +1,15 @@
 #include "storage.hpp"
 
 #include <set>
-#include <iostream>
 
 #include <OpenThreads/ScopedLock>
 
 #include <osg/Image>
 #include <osg/Plane>
 
-#include <boost/algorithm/string.hpp>
-
+#include <components/debug/debuglog.hpp>
 #include <components/misc/resourcehelpers.hpp>
+#include <components/misc/stringops.hpp>
 #include <components/vfs/manager.hpp>
 
 namespace ESMTerrain
@@ -24,6 +23,8 @@ namespace ESMTerrain
     };
 
     LandObject::LandObject()
+        : mLand(nullptr)
+        , mLoadFlags(0)
     {
     }
 
@@ -35,6 +36,8 @@ namespace ESMTerrain
     }
 
     LandObject::LandObject(const LandObject &copy, const osg::CopyOp &copyop)
+        : mLand(nullptr)
+        , mLoadFlags(0)
     {
     }
 
@@ -42,19 +45,6 @@ namespace ESMTerrain
     {
     }
 
-    const ESM::Land::LandData *LandObject::getData(int flags) const
-    {
-        if ((mData.mDataLoaded & flags) != flags)
-            return NULL;
-        return &mData;
-    }
-
-    int LandObject::getPlugin() const
-    {
-        return mLand->mPlugin;
-    }
-
-
     const float defaultHeight = ESM::Land::DEFAULT_HEIGHT;
 
     Storage::Storage(const VFS::Manager *vfs, const std::string& normalMapPattern, const std::string& normalHeightMapPattern, bool autoUseNormalMaps, const std::string& specularMapPattern, bool autoUseSpecularMaps)
@@ -154,7 +144,7 @@ namespace ESMTerrain
         normal.normalize();
     }
 
-    void Storage::fixColour (osg::Vec4f& color, int cellX, int cellY, int col, int row, LandCache& cache)
+    void Storage::fixColour (osg::Vec4ub& color, int cellX, int cellY, int col, int row, LandCache& cache)
     {
         if (col == ESM::Land::LAND_SIZE-1)
         {
@@ -171,22 +161,22 @@ namespace ESMTerrain
         const ESM::Land::LandData* data = land ? land->getData(ESM::Land::DATA_VCLR) : 0;
         if (data)
         {
-            color.r() = data->mColours[col*ESM::Land::LAND_SIZE*3+row*3] / 255.f;
-            color.g() = data->mColours[col*ESM::Land::LAND_SIZE*3+row*3+1] / 255.f;
-            color.b() = data->mColours[col*ESM::Land::LAND_SIZE*3+row*3+2] / 255.f;
+            color.r() = data->mColours[col*ESM::Land::LAND_SIZE*3+row*3];
+            color.g() = data->mColours[col*ESM::Land::LAND_SIZE*3+row*3+1];
+            color.b() = data->mColours[col*ESM::Land::LAND_SIZE*3+row*3+2];
         }
         else
         {
-            color.r() = 1;
-            color.g() = 1;
-            color.b() = 1;
+            color.r() = 255;
+            color.g() = 255;
+            color.b() = 255;
         }
     }
 
     void Storage::fillVertexBuffers (int lodLevel, float size, const osg::Vec2f& center,
                                             osg::ref_ptr<osg::Vec3Array> positions,
                                             osg::ref_ptr<osg::Vec3Array> normals,
-                                            osg::ref_ptr<osg::Vec4Array> colours)
+                                            osg::ref_ptr<osg::Vec4ubArray> colours)
     {
         // LOD level n means every 2^n-th vertex is kept
         size_t increment = static_cast<size_t>(1) << lodLevel;
@@ -203,13 +193,15 @@ namespace ESMTerrain
         colours->resize(numVerts*numVerts);
 
         osg::Vec3f normal;
-        osg::Vec4f color;
+        osg::Vec4ub color;
 
         float vertY = 0;
         float vertX = 0;
 
         LandCache cache;
 
+        bool alteration = useAlteration();
+
         float vertY_ = 0; // of current cell corner
         for (int cellY = startCellY; cellY < startCellY + std::ceil(size); ++cellY)
         {
@@ -260,10 +252,11 @@ namespace ESMTerrain
                         float height = defaultHeight;
                         if (heightData)
                             height = heightData->mHeights[col*ESM::Land::LAND_SIZE + row];
-
+                        if (alteration)
+                            height += getAlteredHeight(col, row);
                         (*positions)[static_cast<unsigned int>(vertX*numVerts + vertY)]
-                            = osg::Vec3f((vertX / float(numVerts - 1) - 0.5f) * size * 8192,
-                                         (vertY / float(numVerts - 1) - 0.5f) * size * 8192,
+                            = osg::Vec3f((vertX / float(numVerts - 1) - 0.5f) * size * Constants::CellSizeInUnits,
+                                         (vertY / float(numVerts - 1) - 0.5f) * size * Constants::CellSizeInUnits,
                                          height);
 
                         if (normalData)
@@ -291,20 +284,22 @@ namespace ESMTerrain
                         if (colourData)
                         {
                             for (int i=0; i<3; ++i)
-                                color[i] = colourData->mColours[srcArrayIndex+i] / 255.f;
+                                color[i] = colourData->mColours[srcArrayIndex+i];
                         }
                         else
                         {
-                            color.r() = 1;
-                            color.g() = 1;
-                            color.b() = 1;
+                            color.r() = 255;
+                            color.g() = 255;
+                            color.b() = 255;
                         }
+                        if (alteration)
+                            adjustColor(col, row, heightData, color); //Does nothing by default, override in OpenMW-CS
 
                         // Unlike normals, colors mostly connect seamlessly between cells, but not always...
                         if (col == ESM::Land::LAND_SIZE-1 || row == ESM::Land::LAND_SIZE-1)
                             fixColour(color, cellX, cellY, col, row, cache);
 
-                        color.a() = 1;
+                        color.a() = 255;
 
                         (*colours)[static_cast<unsigned int>(vertX*numVerts + vertY)] = color;
 
@@ -361,7 +356,7 @@ namespace ESMTerrain
 
     std::string Storage::getTextureName(UniqueTextureId id)
     {
-        static const std::string defaultTexture = "textures\\_land_default.dds";
+        static constexpr char defaultTexture[] = "textures\\_land_default.dds";
         if (id.first == 0)
             return defaultTexture; // Not sure if the default texture really is hardcoded?
 
@@ -369,7 +364,7 @@ namespace ESMTerrain
         const ESM::LandTexture* ltex = getLandTexture(id.first-1, id.second);
         if (!ltex)
         {
-            std::cerr << "Warning: Unable to find land texture index " << id.first-1 << " in plugin " << id.second << ", using default texture instead" << std::endl;
+            Log(Debug::Warning) << "Warning: Unable to find land texture index " << id.first-1 << " in plugin " << id.second << ", using default texture instead";
             return defaultTexture;
         }
 
@@ -379,8 +374,7 @@ namespace ESMTerrain
         return texture;
     }
 
-    void Storage::getBlendmaps(float chunkSize, const osg::Vec2f &chunkCenter,
-        bool pack, ImageVector &blendmaps, std::vector<Terrain::LayerInfo> &layerList)
+    void Storage::getBlendmaps(float chunkSize, const osg::Vec2f &chunkCenter, ImageVector &blendmaps, std::vector<Terrain::LayerInfo> &layerList)
     {
         osg::Vec2f origin = chunkCenter - osg::Vec2f(chunkSize/2.f, chunkSize/2.f);
         int cellX = static_cast<int>(std::floor(origin.x()));
@@ -390,80 +384,67 @@ namespace ESMTerrain
 
         int rowStart = (origin.x() - cellX) * realTextureSize;
         int colStart = (origin.y() - cellY) * realTextureSize;
-        int rowEnd = rowStart + chunkSize * (realTextureSize-1) + 1;
-        int colEnd = colStart + chunkSize * (realTextureSize-1) + 1;
 
-        // Save the used texture indices so we know the total number of textures
-        // and number of required blend maps
-        std::set<UniqueTextureId> textureIndices;
-        // Due to the way the blending works, the base layer will always shine through in between
-        // blend transitions (eg halfway between two texels, both blend values will be 0.5, so 25% of base layer visible).
-        // To get a consistent look, we need to make sure to use the same base layer in all cells.
-        // So we're always adding _land_default.dds as the base layer here, even if it's not referenced in this cell.
-        textureIndices.insert(std::make_pair(0,0));
+        const int blendmapSize = (realTextureSize-1) * chunkSize + 1;
+        // We need to upscale the blendmap 2x with nearest neighbor sampling to look like Vanilla
+        const int imageScaleFactor = 2;
+        const int blendmapImageSize = blendmapSize * imageScaleFactor;
 
         LandCache cache;
+        std::map<UniqueTextureId, unsigned int> textureIndicesMap;
 
-        for (int y=colStart; y<colEnd; ++y)
-            for (int x=rowStart; x<rowEnd; ++x)
-            {
-                UniqueTextureId id = getVtexIndexAt(cellX, cellY, x, y, cache);
-                textureIndices.insert(id);
-            }
-
-        // Makes sure the indices are sorted, or rather,
-        // retrieved as sorted. This is important to keep the splatting order
-        // consistent across cells.
-        std::map<UniqueTextureId, int> textureIndicesMap;
-        for (std::set<UniqueTextureId>::iterator it = textureIndices.begin(); it != textureIndices.end(); ++it)
+        for (int y=0; y<blendmapSize; y++)
         {
-            int size = textureIndicesMap.size();
-            textureIndicesMap[*it] = size;
-            layerList.push_back(getLayerInfo(getTextureName(*it)));
-        }
-
-        int numTextures = textureIndices.size();
-        // numTextures-1 since the base layer doesn't need blending
-        int numBlendmaps = pack ? static_cast<int>(std::ceil((numTextures - 1) / 4.f)) : (numTextures - 1);
-
-        int channels = pack ? 4 : 1;
-
-        // Second iteration - create and fill in the blend maps
-        const int blendmapSize = (realTextureSize-1) * chunkSize + 1;
-
-        for (int i=0; i<numBlendmaps; ++i)
-        {
-            GLenum format = pack ? GL_RGBA : GL_ALPHA;
-
-            osg::ref_ptr<osg::Image> image (new osg::Image);
-            image->allocateImage(blendmapSize, blendmapSize, 1, format, GL_UNSIGNED_BYTE);
-            unsigned char* pData = image->data();
-
-            for (int y=0; y<blendmapSize; ++y)
+            for (int x=0; x<blendmapSize; x++)
             {
-                for (int x=0; x<blendmapSize; ++x)
+                UniqueTextureId id = getVtexIndexAt(cellX, cellY, x+rowStart, y+colStart, cache);
+                std::map<UniqueTextureId, unsigned int>::iterator found = textureIndicesMap.find(id);
+                if (found == textureIndicesMap.end())
                 {
-                    UniqueTextureId id = getVtexIndexAt(cellX, cellY, x+rowStart, y+colStart, cache);
-                    assert(textureIndicesMap.find(id) != textureIndicesMap.end());
-                    int layerIndex = textureIndicesMap.find(id)->second;
-                    int blendIndex = (pack ? static_cast<int>(std::floor((layerIndex - 1) / 4.f)) : layerIndex - 1);
-                    int channel = pack ? std::max(0, (layerIndex-1) % 4) : 0;
+                    unsigned int layerIndex = layerList.size();
+                    Terrain::LayerInfo info = getLayerInfo(getTextureName(id));
 
-                    if (blendIndex == i)
-                        pData[(blendmapSize - y - 1)*blendmapSize*channels + x*channels + channel] = 255;
-                    else
-                        pData[(blendmapSize - y - 1)*blendmapSize*channels + x*channels + channel] = 0;
+                    // look for existing diffuse map, which may be present when several plugins use the same texture
+                    for (unsigned int i=0; i<layerList.size(); ++i)
+                    {
+                        if (layerList[i].mDiffuseMap == info.mDiffuseMap)
+                        {
+                            layerIndex = i;
+                            break;
+                        }
+                    }
+
+                    found = textureIndicesMap.emplace(id, layerIndex).first;
+
+                    if (layerIndex >= layerList.size())
+                    {
+                        osg::ref_ptr<osg::Image> image (new osg::Image);
+                        image->allocateImage(blendmapImageSize, blendmapImageSize, 1, GL_ALPHA, GL_UNSIGNED_BYTE);
+                        unsigned char* pData = image->data();
+                        memset(pData, 0, image->getTotalDataSize());
+                        blendmaps.emplace_back(image);
+                        layerList.emplace_back(info);
+                    }
                 }
+                unsigned int layerIndex = found->second;
+                unsigned char* pData = blendmaps[layerIndex]->data();
+                int realY = (blendmapSize - y - 1)*imageScaleFactor;
+                int realX = x*imageScaleFactor;
+                pData[((realY+0)*blendmapImageSize + realX + 0)] = 255;
+                pData[((realY+1)*blendmapImageSize + realX + 0)] = 255;
+                pData[((realY+0)*blendmapImageSize + realX + 1)] = 255;
+                pData[((realY+1)*blendmapImageSize + realX + 1)] = 255;
             }
-
-            blendmaps.push_back(image);
         }
+
+        if (blendmaps.size() == 1)
+            blendmaps.clear(); // If a single texture fills the whole terrain, there is no need to blend
     }
 
     float Storage::getHeightAt(const osg::Vec3f &worldPos)
     {
-        int cellX = static_cast<int>(std::floor(worldPos.x() / 8192.f));
-        int cellY = static_cast<int>(std::floor(worldPos.y() / 8192.f));
+        int cellX = static_cast<int>(std::floor(worldPos.x() / float(Constants::CellSizeInUnits)));
+        int cellY = static_cast<int>(std::floor(worldPos.y() / float(Constants::CellSizeInUnits)));
 
         osg::ref_ptr<const LandObject> land = getLand(cellX, cellY);
         if (!land)
@@ -476,8 +457,8 @@ namespace ESMTerrain
         // Mostly lifted from Ogre::Terrain::getHeightAtTerrainPosition
 
         // Normalized position in the cell
-        float nX = (worldPos.x() - (cellX * 8192))/8192.f;
-        float nY = (worldPos.y() - (cellY * 8192))/8192.f;
+        float nX = (worldPos.x() - (cellX * Constants::CellSizeInUnits)) / float(Constants::CellSizeInUnits);
+        float nY = (worldPos.y() - (cellY * Constants::CellSizeInUnits)) / float(Constants::CellSizeInUnits);
 
         // get left / bottom points (rounded down)
         float factor = ESM::Land::LAND_SIZE - 1.0f;
@@ -509,10 +490,10 @@ namespace ESMTerrain
         */
 
         // Build all 4 positions in normalized cell space, using point-sampled height
-        osg::Vec3f v0 (startXTS, startYTS, getVertexHeight(data, startX, startY) / 8192.f);
-        osg::Vec3f v1 (endXTS, startYTS, getVertexHeight(data, endX, startY) / 8192.f);
-        osg::Vec3f v2 (endXTS, endYTS, getVertexHeight(data, endX, endY) / 8192.f);
-        osg::Vec3f v3 (startXTS, endYTS, getVertexHeight(data, startX, endY) / 8192.f);
+        osg::Vec3f v0 (startXTS, startYTS, getVertexHeight(data, startX, startY) / float(Constants::CellSizeInUnits));
+        osg::Vec3f v1 (endXTS, startYTS, getVertexHeight(data, endX, startY) / float(Constants::CellSizeInUnits));
+        osg::Vec3f v2 (endXTS, endYTS, getVertexHeight(data, endX, endY) / float(Constants::CellSizeInUnits));
+        osg::Vec3f v3 (startXTS, endYTS, getVertexHeight(data, startX, endY) / float(Constants::CellSizeInUnits));
         // define this plane in terrain space
         osg::Plane plane;
         // FIXME: deal with differing triangle alignment
@@ -540,17 +521,10 @@ namespace ESMTerrain
         // Solve plane equation for z
         return (-plane.getNormal().x() * nX
                 -plane.getNormal().y() * nY
-                - plane[3]) / plane.getNormal().z() * 8192;
+                - plane[3]) / plane.getNormal().z() * Constants::CellSizeInUnits;
 
     }
 
-    float Storage::getVertexHeight(const ESM::Land::LandData* data, int x, int y)
-    {
-        assert(x < ESM::Land::LAND_SIZE);
-        assert(y < ESM::Land::LAND_SIZE);
-        return data->mHeights[y * ESM::Land::LAND_SIZE + x];
-    }
-
     const LandObject* Storage::getLand(int cellX, int cellY, LandCache& cache)
     {
         LandCache::Map::iterator found = cache.mMap.find(std::make_pair(cellX, cellY));
@@ -563,6 +537,15 @@ namespace ESMTerrain
         }
     }
 
+    void Storage::adjustColor(int col, int row, const ESM::Land::LandData *heightData, osg::Vec4ub& color) const
+    {
+    }
+
+    float Storage::getAlteredHeight(int col, int row) const
+    {
+        return 0;
+    }
+
     Terrain::LayerInfo Storage::getLayerInfo(const std::string& texture)
     {
         OpenThreads::ScopedLock<OpenThreads::Mutex> lock(mLayerInfoMutex);
@@ -580,7 +563,7 @@ namespace ESMTerrain
         if (mAutoUseNormalMaps)
         {
             std::string texture_ = texture;
-            boost::replace_last(texture_, ".", mNormalHeightMapPattern + ".");
+            Misc::StringUtils::replaceLast(texture_, ".", mNormalHeightMapPattern + ".");
             if (mVFS->exists(texture_))
             {
                 info.mNormalMap = texture_;
@@ -589,7 +572,7 @@ namespace ESMTerrain
             else
             {
                 texture_ = texture;
-                boost::replace_last(texture_, ".", mNormalMapPattern + ".");
+                Misc::StringUtils::replaceLast(texture_, ".", mNormalMapPattern + ".");
                 if (mVFS->exists(texture_))
                     info.mNormalMap = texture_;
             }
@@ -598,7 +581,7 @@ namespace ESMTerrain
         if (mAutoUseSpecularMaps)
         {
             std::string texture_ = texture;
-            boost::replace_last(texture_, ".", mSpecularMapPattern + ".");
+            Misc::StringUtils::replaceLast(texture_, ".", mSpecularMapPattern + ".");
             if (mVFS->exists(texture_))
             {
                 info.mDiffuseMap = texture_;
@@ -611,15 +594,6 @@ namespace ESMTerrain
         return info;
     }
 
-    Terrain::LayerInfo Storage::getDefaultLayer()
-    {
-        Terrain::LayerInfo info;
-        info.mDiffuseMap = "textures\\_land_default.dds";
-        info.mParallax = false;
-        info.mSpecular = false;
-        return info;
-    }
-
     float Storage::getCellWorldSize()
     {
         return static_cast<float>(ESM::Land::REAL_SIZE);
