@@ -1,9 +1,9 @@
 #include "multidircollection.hpp"
 
-#include <iostream>
-
 #include <boost/filesystem.hpp>
 
+#include <components/debug/debuglog.hpp>
+
 namespace Files
 {
     struct NameEqual
@@ -46,7 +46,7 @@ namespace Files
         {
             if (!boost::filesystem::is_directory(*iter))
             {
-                std::cout << "Skipping invalid directory: " << (*iter).string() << std::endl;
+                Log(Debug::Info) << "Skipping invalid directory: " << (*iter).string();
                 continue;
             }
 
