@@ -3,7 +3,6 @@
 #include <stdexcept>
 
 #include <osg/Image>
-#include <osg/Texture2D>
 
 #include <osgDB/WriteFile>
 
@@ -14,6 +13,8 @@
 #include <MyGUI_FactoryManager.h>
 #include <MyGUI_RenderManager.h>
 
+#include <components/debug/debuglog.hpp>
+
 #include <components/vfs/manager.hpp>
 
 #include <components/misc/stringops.hpp>
@@ -143,8 +144,9 @@ namespace
 namespace Gui
 {
 
-    FontLoader::FontLoader(ToUTF8::FromType encoding, const VFS::Manager* vfs)
+    FontLoader::FontLoader(ToUTF8::FromType encoding, const VFS::Manager* vfs, const std::string& userDataPath)
         : mVFS(vfs)
+        , mUserDataPath(userDataPath)
     {
         if (encoding == ToUTF8::WINDOWS_1252)
             mEncoding = ToUTF8::CP437;
@@ -159,11 +161,21 @@ namespace Gui
         mTextures.clear();
 
         for (std::vector<MyGUI::ResourceManualFont*>::iterator it = mFonts.begin(); it != mFonts.end(); ++it)
-            MyGUI::ResourceManager::getInstance().removeByName((*it)->getResourceName());
+        {
+            try
+            {
+                MyGUI::ResourceManager::getInstance().removeByName((*it)->getResourceName());
+            }
+            catch(const MyGUI::Exception& e)
+            {
+                Log(Debug::Error) << "Error in the destructor: " << e.what();
+            }
+        }
+
         mFonts.clear();
     }
 
-    void FontLoader::loadAllFonts(bool exportToFile)
+    void FontLoader::loadBitmapFonts(bool exportToFile)
     {
         const std::map<std::string, VFS::File*>& index = mVFS->getIndex();
 
@@ -186,6 +198,25 @@ namespace Gui
         }
     }
 
+    void FontLoader::loadTrueTypeFonts()
+    {
+        osgMyGUI::DataManager* dataManager = dynamic_cast<osgMyGUI::DataManager*>(&osgMyGUI::DataManager::getInstance());
+        if (!dataManager)
+        {
+            Log(Debug::Error) << "Can not load TrueType fonts: osgMyGUI::DataManager is not available.";
+            return;
+        }
+
+        const std::string cfg = dataManager->getDataPath("");
+        const std::string fontFile = mUserDataPath + "/" + "Fonts" + "/" + "openmw_font.xml";
+        if (!boost::filesystem::exists(fontFile))
+            return;
+
+        dataManager->setResourcePath(mUserDataPath + "/" + "Fonts");
+        MyGUI::ResourceManager::getInstance().load("openmw_font.xml");
+        dataManager->setResourcePath(cfg);
+    }
+
 
     typedef struct
     {
@@ -281,7 +312,7 @@ namespace Gui
             assert (image->isDataContiguous());
             memcpy(image->data(), &textureData[0], textureData.size());
 
-            std::cout << "Writing " << resourceName + ".png" << std::endl;
+            Log(Debug::Info) << "Writing " << resourceName + ".png";
             osgDB::writeImageFile(*image, resourceName + ".png");
         }
 
@@ -461,13 +492,21 @@ namespace Gui
 
         if (exportToFile)
         {
-            std::cout << "Writing " << resourceName + ".xml" << std::endl;
+            Log(Debug::Info) << "Writing " << resourceName + ".xml";
             xmlDocument.createDeclaration();
             xmlDocument.save(resourceName + ".xml");
         }
 
         font->deserialization(root, MyGUI::Version(3,2,0));
 
+        // Setup "book" version of font as fallback if we will not use TrueType fonts
+        MyGUI::ResourceManualFont* bookFont = static_cast<MyGUI::ResourceManualFont*>(
+                    MyGUI::FactoryManager::getInstance().createObject("Resource", "ResourceManualFont"));
+        mFonts.push_back(bookFont);
+        bookFont->deserialization(root, MyGUI::Version(3,2,0));
+        bookFont->setResourceName("Journalbook " + resourceName);
+
+        // Remove automatically registered fonts
         for (std::vector<MyGUI::ResourceManualFont*>::iterator it = mFonts.begin(); it != mFonts.end();)
         {
             if ((*it)->getResourceName() == font->getResourceName())
@@ -475,10 +514,17 @@ namespace Gui
                 MyGUI::ResourceManager::getInstance().removeByName(font->getResourceName());
                 it = mFonts.erase(it);
             }
+            else if ((*it)->getResourceName() == bookFont->getResourceName())
+            {
+                MyGUI::ResourceManager::getInstance().removeByName(bookFont->getResourceName());
+                it = mFonts.erase(it);
+            }
             else
                 ++it;
         }
+
         MyGUI::ResourceManager::getInstance().addResource(font);
+        MyGUI::ResourceManager::getInstance().addResource(bookFont);
     }
 
 }
