@@ -1,6 +1,9 @@
 #ifndef OPENMW_COMPONENTS_FONTLOADER_H
 #define OPENMW_COMPONENTS_FONTLOADER_H
 
+#include "boost/filesystem/operations.hpp"
+
+#include <components/myguiplatform/myguidatamanager.hpp>
 #include <components/to_utf8/to_utf8.hpp>
 
 namespace VFS
@@ -23,15 +26,17 @@ namespace Gui
     class FontLoader
     {
     public:
-        FontLoader (ToUTF8::FromType encoding, const VFS::Manager* vfs);
+        FontLoader (ToUTF8::FromType encoding, const VFS::Manager* vfs, const std::string& userDataPath);
         ~FontLoader();
 
         /// @param exportToFile export the converted fonts (Images and XML with glyph metrics) to files?
-        void loadAllFonts (bool exportToFile);
+        void loadBitmapFonts (bool exportToFile);
+        void loadTrueTypeFonts ();
 
     private:
         ToUTF8::FromType mEncoding;
         const VFS::Manager* mVFS;
+        std::string mUserDataPath;
 
         std::vector<MyGUI::ITexture*> mTextures;
         std::vector<MyGUI::ResourceManualFont*> mFonts;
