@@ -2,6 +2,7 @@
 #define OPENMW_COMPONENTS_MISC_RNG_H
 
 #include <cassert>
+#include <random>
 
 namespace Misc
 {
@@ -13,8 +14,11 @@ class Rng
 {
 public:
 
+    /// create a RNG
+    static std::mt19937 generator;
+
     /// seed the RNG
-    static void init();
+    static void init(unsigned int seed = generateDefaultSeed());
 
     /// return value in range [0.0f, 1.0f)  <- note open upper range.
     static float rollProbability();
@@ -27,6 +31,9 @@ public:
 
     /// return value in range [0, 99]
     static int roll0to99() { return rollDice(100); }
+
+    /// returns default seed for RNG
+    static unsigned int generateDefaultSeed();
 };
 
 }
