@@ -2,7 +2,6 @@
 #define MISC_STRINGOPS_H
 
 #include <cctype>
-#include <cstring>
 #include <string>
 #include <algorithm>
 
@@ -19,6 +18,19 @@ class StringUtils
         }
     };
 
+    // Allow to convert complex arguments to C-style strings for format() function
+    template <typename T>
+    static T argument(T value) noexcept
+    {
+        return value;
+    }
+
+    template <typename T>
+    static T const * argument(std::basic_string<T> const & value) noexcept
+    {
+        return value.c_str();
+    }
+
 public:
 
     /// Plain and simple locale-unaware toLower. Anything from A to Z is lower-cased, multibyte characters are unchanged.
@@ -26,36 +38,7 @@ public:
     /// Don't use tolower(int) because that depends on global locale.
     static char toLower(char c)
     {
-        switch(c)
-        {
-        case 'A':return 'a';
-        case 'B':return 'b';
-        case 'C':return 'c';
-        case 'D':return 'd';
-        case 'E':return 'e';
-        case 'F':return 'f';
-        case 'G':return 'g';
-        case 'H':return 'h';
-        case 'I':return 'i';
-        case 'J':return 'j';
-        case 'K':return 'k';
-        case 'L':return 'l';
-        case 'M':return 'm';
-        case 'N':return 'n';
-        case 'O':return 'o';
-        case 'P':return 'p';
-        case 'Q':return 'q';
-        case 'R':return 'r';
-        case 'S':return 's';
-        case 'T':return 't';
-        case 'U':return 'u';
-        case 'V':return 'v';
-        case 'W':return 'w';
-        case 'X':return 'x';
-        case 'Y':return 'y';
-        case 'Z':return 'z';
-        default:return c;
-        };
+        return (c >= 'A' && c <= 'Z') ? c + 'a' - 'A' : c;
     }
 
     static Utf8Stream::UnicodeChar toLowerUtf8(Utf8Stream::UnicodeChar ch)
@@ -146,8 +129,15 @@ public:
         std::string::const_iterator yit = y.begin();
         for(;xit != x.end() && yit != y.end() && len > 0;++xit,++yit,--len)
         {
-            int res = *xit - *yit;
-            if(res != 0 && toLower(*xit) != toLower(*yit))
+            char left = *xit;
+            char right = *yit;
+            if (left == right)
+                continue;
+
+            left = toLower(left);
+            right = toLower(right);
+            int res = left - right;
+            if(res != 0)
                 return (res > 0) ? 1 : -1;
         }
         if(len > 0)
@@ -234,6 +224,80 @@ public:
         }
         return str;
     }
+
+    // Requires some C++11 features:
+    // 1. std::string needs to be contiguous
+    // 2. std::snprintf with zero size (second argument) returns an output string size
+    // 3. variadic templates support
+    template <typename ... Args>
+    static std::string format(const char* fmt, Args const & ... args)
+    {
+        auto size = std::snprintf(nullptr, 0, fmt, argument(args) ...);
+        // Note: sprintf also writes a trailing null character. We should remove it.
+        std::string ret(size+1, '\0');
+        std::sprintf(&ret[0], fmt, argument(args) ...);
+        ret.erase(size);
+
+        return ret;
+    }
+
+    template <typename ... Args>
+    static std::string format(const std::string& fmt, Args const & ... args)
+    {
+        return format(fmt.c_str(), args ...);
+    }
+
+    static inline void trim(std::string &s)
+    {
+        // left trim
+        s.erase(s.begin(), std::find_if(s.begin(), s.end(), [](int ch)
+        {
+            return !std::isspace(ch);
+        }));
+
+        // right trim
+        s.erase(std::find_if(s.rbegin(), s.rend(), [](int ch)
+        {
+            return !std::isspace(ch);
+        }).base(), s.end());
+    }
+
+    template <class Container>
+    static inline void split(const std::string& str, Container& cont, const std::string& delims = " ")
+    {
+        std::size_t current, previous = 0;
+        current = str.find_first_of(delims);
+        while (current != std::string::npos)
+        {
+            cont.push_back(str.substr(previous, current - previous));
+            previous = current + 1;
+            current = str.find_first_of(delims, previous);
+        }
+        cont.push_back(str.substr(previous, current - previous));
+    }
+
+    // TODO: use the std::string_view once we will use the C++17.
+    // It should allow us to avoid data copying while we still will support both string and literal arguments.
+
+    static inline void replaceAll(std::string& data, std::string toSearch, std::string replaceStr)
+    {
+        size_t pos = data.find(toSearch);
+
+        while( pos != std::string::npos)
+        {
+            data.replace(pos, toSearch.size(), replaceStr);
+            pos = data.find(toSearch, pos + replaceStr.size());
+        }
+    }
+
+     static inline void replaceLast(std::string& str, std::string substr, std::string with)
+     {
+         size_t pos = str.rfind(substr);
+         if (pos == std::string::npos)
+             return;
+
+         str.replace(pos, substr.size(), with);
+     }
 };
 
 }
