@@ -8,13 +8,21 @@
 #include <sstream>
 #include <map>
 
-#include <boost/shared_ptr.hpp>
-
 #include "niffile.hpp"
 
 namespace Nif
 {
 
+enum InterpolationType
+{
+    InterpolationType_Unknown = 0,
+    InterpolationType_Linear = 1,
+    InterpolationType_Quadratic = 2,
+    InterpolationType_TBC = 3,
+    InterpolationType_XYZ = 4,
+    InterpolationType_Constant = 5
+};
+
 template<typename T>
 struct KeyT {
     T mValue;
@@ -28,34 +36,27 @@ struct KeyT {
     float mContinuity; // Only for TBC interpolation
     */
 };
-typedef KeyT<float> FloatKey;
-typedef KeyT<osg::Vec3f> Vector3Key;
-typedef KeyT<osg::Vec4f> Vector4Key;
-typedef KeyT<osg::Quat> QuaternionKey;
+using FloatKey = KeyT<float>;
+using Vector3Key = KeyT<osg::Vec3f>;
+using Vector4Key = KeyT<osg::Vec4f>;
+using QuaternionKey = KeyT<osg::Quat>;
 
 template<typename T, T (NIFStream::*getValue)()>
 struct KeyMapT {
-    typedef std::map< float, KeyT<T> > MapType;
+    using MapType = std::map<float, KeyT<T>>;
 
-    typedef T ValueType;
-    typedef KeyT<T> KeyType;
+    using ValueType = T;
+    using KeyType = KeyT<T>;
 
-    static const unsigned int sLinearInterpolation = 1;
-    static const unsigned int sQuadraticInterpolation = 2;
-    static const unsigned int sTBCInterpolation = 3;
-    static const unsigned int sXYZInterpolation = 4;
-
-    unsigned int mInterpolationType;
+    unsigned int mInterpolationType = InterpolationType_Linear;
     MapType mKeys;
 
-    KeyMapT() : mInterpolationType(sLinearInterpolation) {}
-
     //Read in a KeyGroup (see http://niftools.sourceforge.net/doc/nif/NiKeyframeData.html)
     void read(NIFStream *nif, bool force=false)
     {
         assert(nif);
 
-        mInterpolationType = 0;
+        mInterpolationType = InterpolationType_Unknown;
 
         size_t count = nif->getUInt();
         if(count == 0 && !force)
@@ -68,7 +69,8 @@ struct KeyMapT {
         KeyT<T> key;
         NIFStream &nifReference = *nif;
 
-        if(mInterpolationType == sLinearInterpolation)
+        if (mInterpolationType == InterpolationType_Linear
+         || mInterpolationType == InterpolationType_Constant)
         {
             for(size_t i = 0;i < count;i++)
             {
@@ -77,7 +79,7 @@ struct KeyMapT {
                 mKeys[time] = key;
             }
         }
-        else if(mInterpolationType == sQuadraticInterpolation)
+        else if (mInterpolationType == InterpolationType_Quadratic)
         {
             for(size_t i = 0;i < count;i++)
             {
@@ -86,7 +88,7 @@ struct KeyMapT {
                 mKeys[time] = key;
             }
         }
-        else if(mInterpolationType == sTBCInterpolation)
+        else if (mInterpolationType == InterpolationType_TBC)
         {
             for(size_t i = 0;i < count;i++)
             {
@@ -96,11 +98,11 @@ struct KeyMapT {
             }
         }
         //XYZ keys aren't actually read here.
-        //data.hpp sees that the last type read was sXYZInterpolation and:
+        //data.hpp sees that the last type read was InterpolationType_XYZ and:
         //    Eats a floating point number, then
         //    Re-runs the read function 3 more times.
-        //        When it does that it's reading in a bunch of sLinearInterpolation keys, not sXYZInterpolation.
-        else if(mInterpolationType == sXYZInterpolation)
+        //        When it does that it's reading in a bunch of InterpolationType_Linear keys, not InterpolationType_XYZ.
+        else if(mInterpolationType == InterpolationType_XYZ)
         {
             //Don't try to read XYZ keys into the wrong part
             if ( count != 1 )
@@ -111,7 +113,7 @@ struct KeyMapT {
                 nif->file->fail(error.str());
             }
         }
-        else if (0 == mInterpolationType)
+        else if (mInterpolationType == InterpolationType_Unknown)
         {
             if (count != 0)
                 nif->file->fail("Interpolation type 0 doesn't work with keys");
@@ -151,15 +153,17 @@ private:
         /*key.mContinuity = */nif.getFloat();
     }
 };
-typedef KeyMapT<float,&NIFStream::getFloat> FloatKeyMap;
-typedef KeyMapT<osg::Vec3f,&NIFStream::getVector3> Vector3KeyMap;
-typedef KeyMapT<osg::Vec4f,&NIFStream::getVector4> Vector4KeyMap;
-typedef KeyMapT<osg::Quat,&NIFStream::getQuaternion> QuaternionKeyMap;
+using FloatKeyMap = KeyMapT<float,&NIFStream::getFloat>;
+using Vector3KeyMap = KeyMapT<osg::Vec3f,&NIFStream::getVector3>;
+using Vector4KeyMap = KeyMapT<osg::Vec4f,&NIFStream::getVector4>;
+using QuaternionKeyMap = KeyMapT<osg::Quat,&NIFStream::getQuaternion>;
+using ByteKeyMap = KeyMapT<char,&NIFStream::getChar>;
 
-typedef boost::shared_ptr<FloatKeyMap> FloatKeyMapPtr;
-typedef boost::shared_ptr<Vector3KeyMap> Vector3KeyMapPtr;
-typedef boost::shared_ptr<Vector4KeyMap> Vector4KeyMapPtr;
-typedef boost::shared_ptr<QuaternionKeyMap> QuaternionKeyMapPtr;
+using FloatKeyMapPtr = std::shared_ptr<FloatKeyMap>;
+using Vector3KeyMapPtr = std::shared_ptr<Vector3KeyMap>;
+using Vector4KeyMapPtr = std::shared_ptr<Vector4KeyMap>;
+using QuaternionKeyMapPtr = std::shared_ptr<QuaternionKeyMap>;
+using ByteKeyMapPtr = std::shared_ptr<ByteKeyMap>;
 
 } // Namespace
 #endif //#ifndef OPENMW_COMPONENTS_NIF_NIFKEY_HPP
