@@ -4,7 +4,6 @@
 #include <string>
 
 #include <BulletCollision/CollisionShapes/btBoxShape.h>
-#include <BulletCollision/CollisionShapes/btTriangleMesh.h>
 #include <BulletCollision/CollisionShapes/btScaledBvhTriangleMeshShape.h>
 #include <BulletCollision/CollisionShapes/btCompoundShape.h>
 
@@ -12,13 +11,15 @@ namespace Resource
 {
 
 BulletShape::BulletShape()
-    : mCollisionShape(NULL)
+    : mCollisionShape(nullptr)
+    , mAvoidCollisionShape(nullptr)
 {
 
 }
 
 BulletShape::BulletShape(const BulletShape &copy, const osg::CopyOp &copyop)
     : mCollisionShape(duplicateCollisionShape(copy.mCollisionShape))
+    , mAvoidCollisionShape(duplicateCollisionShape(copy.mAvoidCollisionShape))
     , mCollisionBoxHalfExtents(copy.mCollisionBoxHalfExtents)
     , mCollisionBoxTranslate(copy.mCollisionBoxTranslate)
     , mAnimatedShapes(copy.mAnimatedShapes)
@@ -27,12 +28,13 @@ BulletShape::BulletShape(const BulletShape &copy, const osg::CopyOp &copyop)
 
 BulletShape::~BulletShape()
 {
+    deleteShape(mAvoidCollisionShape);
     deleteShape(mCollisionShape);
 }
 
 void BulletShape::deleteShape(btCollisionShape* shape)
 {
-    if(shape!=NULL)
+    if(shape!=nullptr)
     {
         if(shape->isCompound())
         {
@@ -77,11 +79,23 @@ btCollisionShape* BulletShape::duplicateCollisionShape(const btCollisionShape *s
     throw std::logic_error(std::string("Unhandled Bullet shape duplication: ")+shape->getName());
 }
 
-btCollisionShape *BulletShape::getCollisionShape()
+btCollisionShape *BulletShape::getCollisionShape() const
 {
     return mCollisionShape;
 }
 
+btCollisionShape *BulletShape::getAvoidCollisionShape() const
+{
+    return mAvoidCollisionShape;
+}
+
+void BulletShape::setLocalScaling(const btVector3& scale)
+{
+    mCollisionShape->setLocalScaling(scale);
+    if (mAvoidCollisionShape)
+        mAvoidCollisionShape->setLocalScaling(scale);
+}
+
 osg::ref_ptr<BulletShapeInstance> BulletShape::makeInstance() const
 {
     osg::ref_ptr<BulletShapeInstance> instance (new BulletShapeInstance(this));
@@ -99,6 +113,9 @@ BulletShapeInstance::BulletShapeInstance(osg::ref_ptr<const BulletShape> source)
 
     if (source->mCollisionShape)
         mCollisionShape = duplicateCollisionShape(source->mCollisionShape);
+
+    if (source->mAvoidCollisionShape)
+        mAvoidCollisionShape = duplicateCollisionShape(source->mAvoidCollisionShape);
 }
 
 }
