@@ -102,7 +102,7 @@ namespace SceneUtil
             if (LightManager* lightManager = dynamic_cast<LightManager*>(path[i]))
                 return lightManager;
         }
-        return NULL;
+        return nullptr;
     }
 
     // Set on a LightSource. Adds the light source to its light manager for the current frame.
@@ -165,6 +165,8 @@ namespace SceneUtil
         , mLightingMask(~0u)
     {
         setUpdateCallback(new LightManagerUpdateCallback);
+        for (unsigned int i=0; i<8; ++i)
+            mDummies.push_back(new LightStateAttribute(i, std::vector<osg::ref_ptr<osg::Light> >()));
     }
 
     LightManager::LightManager(const LightManager &copy, const osg::CopyOp &copyop)
@@ -219,7 +221,6 @@ namespace SceneUtil
 
     osg::ref_ptr<osg::StateSet> LightManager::getLightListStateSet(const LightList &lightList, unsigned int frameNum)
     {
-
         // possible optimization: return a StateSet containing all requested lights plus some extra lights (if a suitable one exists)
         size_t hash = 0;
         for (unsigned int i=0; i<lightList.size();++i)
@@ -234,27 +235,24 @@ namespace SceneUtil
         {
             osg::ref_ptr<osg::StateSet> stateset = new osg::StateSet;
             std::vector<osg::ref_ptr<osg::Light> > lights;
+            lights.reserve(lightList.size());
             for (unsigned int i=0; i<lightList.size();++i)
-            {
                 lights.push_back(lightList[i]->mLightSource->getLight(frameNum));
-            }
 
             // the first light state attribute handles the actual state setting for all lights
             // it's best to batch these up so that we don't need to touch the modelView matrix more than necessary
-            osg::ref_ptr<LightStateAttribute> attr = new LightStateAttribute(mStartLight, lights);
             // don't use setAttributeAndModes, that does not support light indices!
-            stateset->setAttribute(attr, osg::StateAttribute::ON);
-            stateset->setAssociatedModes(attr, osg::StateAttribute::ON);
+            stateset->setAttribute(new LightStateAttribute(mStartLight, std::move(lights)), osg::StateAttribute::ON);
+
+            for (unsigned int i=0; i<lightList.size(); ++i)
+                stateset->setMode(GL_LIGHT0 + mStartLight + i, osg::StateAttribute::ON);
 
             // need to push some dummy attributes to ensure proper state tracking
             // lights need to reset to their default when the StateSet is popped
             for (unsigned int i=1; i<lightList.size(); ++i)
-            {
-                osg::ref_ptr<LightStateAttribute> dummy = new LightStateAttribute(mStartLight+i, std::vector<osg::ref_ptr<osg::Light> >());
-                stateset->setAttribute(dummy, osg::StateAttribute::ON);
-            }
+                stateset->setAttribute(mDummies[i+mStartLight].get(), osg::StateAttribute::ON);
 
-            stateSetCache.insert(std::make_pair(hash, stateset));
+            stateSetCache.emplace(hash, stateset);
             return stateset;
         }
     }
@@ -299,7 +297,7 @@ namespace SceneUtil
 
         virtual osg::Object* cloneType() const { return new DisableLight(mIndex); }
         virtual osg::Object* clone(const osg::CopyOp& copyop) const { return new DisableLight(*this,copyop); }
-        virtual bool isSameKindAs(const osg::Object* obj) const { return dynamic_cast<const DisableLight *>(obj)!=NULL; }
+        virtual bool isSameKindAs(const osg::Object* obj) const { return dynamic_cast<const DisableLight *>(obj)!=nullptr; }
         virtual const char* libraryName() const { return "SceneUtil"; }
         virtual const char* className() const { return "DisableLight"; }
         virtual Type getType() const { return LIGHT; }
@@ -323,17 +321,17 @@ namespace SceneUtil
         virtual void apply(osg::State& state) const
         {
             int lightNum = GL_LIGHT0 + mIndex;
-            glLightfv( lightNum, GL_AMBIENT,               mNull.ptr() );
-            glLightfv( lightNum, GL_DIFFUSE,               mNull.ptr() );
-            glLightfv( lightNum, GL_SPECULAR,              mNull.ptr() );
+            glLightfv( lightNum, GL_AMBIENT,               mnullptr.ptr() );
+            glLightfv( lightNum, GL_DIFFUSE,               mnullptr.ptr() );
+            glLightfv( lightNum, GL_SPECULAR,              mnullptr.ptr() );
 
             LightStateCache* cache = getLightStateCache(state.getContextID());
-            cache->lastAppliedLight[mIndex] = NULL;
+            cache->lastAppliedLight[mIndex] = nullptr;
         }
 
     private:
         unsigned int mIndex;
-        osg::Vec4f mNull;
+        osg::Vec4f mnullptr;
     };
 
     void LightManager::setStartLight(int start)
@@ -399,7 +397,7 @@ namespace SceneUtil
                 return false;
         }
 
-        if (!(cv->getCurrentCamera()->getCullMask() & mLightManager->getLightingMask()))
+        if (!(cv->getTraversalMask() & mLightManager->getLightingMask()))
             return false;
 
         // Possible optimizations:
@@ -447,7 +445,7 @@ namespace SceneUtil
         {
             unsigned int maxLights = static_cast<unsigned int> (8 - mLightManager->getStartLight());
 
-            osg::StateSet* stateset = NULL;
+            osg::StateSet* stateset = nullptr;
 
             if (mLightList.size() > maxLights)
             {
