@@ -1,28 +1,15 @@
 #include "unrefqueue.hpp"
 
-#include <deque>
-
 //#include <osg/Timer>
-//#include <iostream>
 
-#include <components/sceneutil/workqueue.hpp>
+//#include <components/debug/debuglog.hpp>
 
 namespace SceneUtil
 {
-
-    class UnrefWorkItem : public SceneUtil::WorkItem
+    void UnrefWorkItem::doWork()
     {
-    public:
-        std::deque<osg::ref_ptr<const osg::Referenced> > mObjects;
-
-        virtual void doWork()
-        {
-            //osg::Timer timer;
-            //size_t objcount = mObjects.size();
-            mObjects.clear();
-            //std::cout << "cleared " << objcount << " objects in " << timer.time_m() << std::endl;
-        }
-    };
+        mObjects.clear();
+    }
 
     UnrefQueue::UnrefQueue()
     {
