@@ -1,6 +1,7 @@
 #include "sdlinputwrapper.hpp"
 
-#include <iostream>
+#include <components/debug/debuglog.hpp>
+#include <components/settings/settings.hpp>
 
 #include <osgViewer/Viewer>
 
@@ -10,10 +11,11 @@ namespace SDLUtil
 InputWrapper::InputWrapper(SDL_Window* window, osg::ref_ptr<osgViewer::Viewer> viewer, bool grab) :
         mSDLWindow(window),
         mViewer(viewer),
-        mMouseListener(NULL),
-        mKeyboardListener(NULL),
-        mWindowListener(NULL),
-        mConListener(NULL),
+        mMouseListener(nullptr),
+        mSensorListener(nullptr),
+        mKeyboardListener(nullptr),
+        mWindowListener(nullptr),
+        mConListener(nullptr),
         mWarpX(0),
         mWarpY(0),
         mWarpCompensate(false),
@@ -77,6 +79,10 @@ InputWrapper::InputWrapper(SDL_Window* window, osg::ref_ptr<osgViewer::Viewer> v
                     break;
                 case SDL_MOUSEWHEEL:
                     mMouseListener->mouseMoved(_packageMouseMotion(evt));
+                    mMouseListener->mouseWheelMoved(evt.wheel);
+                    break;
+                case SDL_SENSORUPDATE:
+                    mSensorListener->sensorUpdated(evt.sensor);
                     break;
                 case SDL_MOUSEBUTTONDOWN:
                     mMouseListener->mousePressed(evt.button, evt.button.button);
@@ -108,11 +114,8 @@ InputWrapper::InputWrapper(SDL_Window* window, osg::ref_ptr<osgViewer::Viewer> v
                 case SDL_TEXTINPUT:
                     mKeyboardListener->textInput(evt.text);
                     break;
-
-#if SDL_VERSION_ATLEAST(2, 0, 4)
                 case SDL_KEYMAPCHANGED:
                     break;
-#endif
                 case SDL_JOYHATMOTION: //As we manage everything with GameController, don't even bother with these.
                 case SDL_JOYAXISMOTION:
                 case SDL_JOYBUTTONDOWN:
@@ -147,6 +150,17 @@ InputWrapper::InputWrapper(SDL_Window* window, osg::ref_ptr<osgViewer::Viewer> v
                     if (mWindowListener)
                         mWindowListener->windowClosed();
                     break;
+                case SDL_DISPLAYEVENT:
+                    switch (evt.display.event)
+                    {
+                        case SDL_DISPLAYEVENT_ORIENTATION:
+                            if (mSensorListener && evt.display.display == (unsigned int) Settings::Manager::getInt("screen", "Video"))
+                                mSensorListener->displayOrientationChanged();
+                            break;
+                        default:
+                            break;
+                    }
+                    break;
                 case SDL_CLIPBOARDUPDATE:
                     break; // We don't need this event, clipboard is retrieved on demand
 
@@ -159,10 +173,23 @@ InputWrapper::InputWrapper(SDL_Window* window, osg::ref_ptr<osgViewer::Viewer> v
                     // No use for touch & gesture events
                     break;
 
+                case SDL_APP_WILLENTERBACKGROUND:
+                case SDL_APP_WILLENTERFOREGROUND:
+                case SDL_APP_DIDENTERBACKGROUND:
+                case SDL_APP_DIDENTERFOREGROUND:
+                    // We do not need background/foreground switch event for mobile devices so far
+                    break;
+
+                case SDL_APP_TERMINATING:
+                    // There is nothing we can do here.
+                    break;
+
+                case SDL_APP_LOWMEMORY:
+                    Log(Debug::Warning) << "System reports that free RAM on device is running low. You may encounter an unexpected behaviour.";
+                    break;
+
                 default:
-                    std::ios::fmtflags f(std::cerr.flags());
-                    std::cerr << "Unhandled SDL event of type 0x" << std::hex << evt.type << std::endl;
-                    std::cerr.flags(f);
+                    Log(Debug::Info) << "Unhandled SDL event of type 0x" << std::hex << evt.type;
                     break;
             }
         }
@@ -236,7 +263,7 @@ InputWrapper::InputWrapper(SDL_Window* window, osg::ref_ptr<osgViewer::Viewer> v
 
     bool InputWrapper::isKeyDown(SDL_Scancode key)
     {
-        return (SDL_GetKeyboardState(NULL)[key]) != 0;
+        return (SDL_GetKeyboardState(nullptr)[key]) != 0;
     }
 
     /// \brief Moves the mouse to the specified point within the viewport
@@ -370,7 +397,7 @@ InputWrapper::InputWrapper(SDL_Window* window, osg::ref_ptr<osgViewer::Viewer> v
         }
         else
         {
-            throw new std::runtime_error("Tried to package non-motion event!");
+            throw std::runtime_error("Tried to package non-motion event!");
         }
 
         return pack_evt;
