@@ -14,7 +14,7 @@ namespace Terrain
     class TerrainGrid : public Terrain::World
     {
     public:
-        TerrainGrid(osg::Group* parent, osg::Group* compileRoot, Resource::ResourceSystem* resourceSystem, Storage* storage, int nodeMask, int preCompileMask=~0);
+        TerrainGrid(osg::Group* parent, osg::Group* compileRoot, Resource::ResourceSystem* resourceSystem, Storage* storage);
         ~TerrainGrid();
 
         virtual void cacheCell(View* view, int x, int y);
@@ -33,10 +33,8 @@ namespace Terrain
         // split each ESM::Cell into mNumSplits*mNumSplits terrain chunks
         unsigned int mNumSplits;
 
-        typedef std::map<std::pair<int, int>, osg::ref_ptr<osg::Node> > Grid;
-        Grid mGrid;
+        CellBorder::CellGrid mGrid;
     };
-
 }
 
 #endif
