@@ -5,6 +5,7 @@
 #include <osg/Camera>
 
 #include <components/resource/resourcesystem.hpp>
+#include <components/sceneutil/vismask.hpp>
 
 #include "storage.hpp"
 #include "texturemanager.hpp"
@@ -14,13 +15,14 @@
 namespace Terrain
 {
 
-World::World(osg::Group* parent, osg::Group* compileRoot, Resource::ResourceSystem* resourceSystem, Storage* storage, int nodeMask, int preCompileMask)
+World::World(osg::Group* parent, osg::Group* compileRoot, Resource::ResourceSystem* resourceSystem, Storage* storage)
     : mStorage(storage)
     , mParent(parent)
     , mResourceSystem(resourceSystem)
+    , mBorderVisible(false)
 {
     mTerrainRoot = new osg::Group;
-    mTerrainRoot->setNodeMask(nodeMask);
+    mTerrainRoot->setNodeMask(SceneUtil::Mask_Terrain);
     mTerrainRoot->getOrCreateStateSet()->setRenderingHint(osg::StateSet::OPAQUE_BIN);
     osg::ref_ptr<osg::Material> material (new osg::Material);
     material->setColorMode(osg::Material::AMBIENT_AND_DIFFUSE);
@@ -33,13 +35,12 @@ World::World(osg::Group* parent, osg::Group* compileRoot, Resource::ResourceSyst
     compositeCam->setProjectionMatrix(osg::Matrix::identity());
     compositeCam->setViewMatrix(osg::Matrix::identity());
     compositeCam->setReferenceFrame(osg::Camera::ABSOLUTE_RF);
-    compositeCam->setClearMask(0);
-    compositeCam->setNodeMask(preCompileMask);
+    compositeCam->setClearMask(SceneUtil::Mask_Disabled);
+    compositeCam->setNodeMask(SceneUtil::Mask_PreCompile);
     mCompositeMapCamera = compositeCam;
 
     compileRoot->addChild(compositeCam);
 
-
     mCompositeMapRenderer = new CompositeMapRenderer;
     compositeCam->addChild(mCompositeMapRenderer);
 
@@ -47,6 +48,7 @@ World::World(osg::Group* parent, osg::Group* compileRoot, Resource::ResourceSyst
 
     mTextureManager.reset(new TextureManager(mResourceSystem->getSceneManager()));
     mChunkManager.reset(new ChunkManager(mStorage, mResourceSystem->getSceneManager(), mTextureManager.get(), mCompositeMapRenderer));
+    mCellBorder.reset(new CellBorder(this,mTerrainRoot.get()));
 
     mResourceSystem->addResourceManager(mChunkManager.get());
     mResourceSystem->addResourceManager(mTextureManager.get());
@@ -65,6 +67,40 @@ World::~World()
     delete mStorage;
 }
 
+void World::setWorkQueue(SceneUtil::WorkQueue* workQueue)
+{
+    mCompositeMapRenderer->setWorkQueue(workQueue);
+}
+
+void World::setBordersVisible(bool visible)
+{
+    mBorderVisible = visible;
+
+    if (visible)
+    {
+        for (std::set<std::pair<int,int>>::iterator it = mLoadedCells.begin(); it != mLoadedCells.end(); ++it)
+            mCellBorder->createCellBorderGeometry(it->first,it->second);
+    }
+    else
+        mCellBorder->destroyCellBorderGeometry();
+}
+
+void World::loadCell(int x, int y)
+{
+    if (mBorderVisible)
+        mCellBorder->createCellBorderGeometry(x,y);
+
+    mLoadedCells.insert(std::pair<int,int>(x,y));
+}
+
+void World::unloadCell(int x, int y)
+{
+    if (mBorderVisible)
+        mCellBorder->destroyCellBorderGeometry(x,y);
+
+    mLoadedCells.erase(std::pair<int,int>(x,y));
+}
+
 void World::setTargetFrameRate(float rate)
 {
     mCompositeMapRenderer->setTargetFrameRate(rate);
