@@ -1,5 +1,7 @@
 #include "box.hpp"
 
+#include <MyGUI_EditText.h>
+
 namespace Gui
 {
 
@@ -48,26 +50,55 @@ namespace Gui
         }
         else
         {
-            TextBox::setPropertyOverride (_key, _value);
+            Gui::TextBox::setPropertyOverride (_key, _value);
         }
     }
 
+    int AutoSizedEditBox::getWidth()
+    {
+        // If the widget has the one short text line, we can shrink widget to avoid a lot of empty space.
+        int textWidth = mMaxWidth;
+
+        if (mShrink)
+        {
+            // MyGUI needs to know the widget size for wordwrapping, but we will know the widget size only after wordwrapping.
+            // To solve this issue, use the maximum tooltip width first for wordwrapping, then resize widget to its actual used space.
+            if (mWasResized)
+            {
+                int maxLineWidth = 0;
+                const MyGUI::VectorLineInfo & lines = getSubWidgetText()->castType<MyGUI::EditText>()->getLineInfo();
+                for (unsigned int i = 0; i < lines.size(); ++i)
+                    maxLineWidth = std::max(maxLineWidth, lines[i].width);
+
+                textWidth = std::min(maxLineWidth, textWidth);
+            }
+            else
+            {
+                mWasResized = true;
+            }
+        }
+
+        return textWidth;
+    }
+
     MyGUI::IntSize AutoSizedEditBox::getRequestedSize()
     {
         if (getAlign().isHStretch())
             throw std::runtime_error("AutoSizedEditBox can't have HStretch align (" + getName() + ")");
-        return MyGUI::IntSize(getSize().width, getTextSize().height);
+        return MyGUI::IntSize(getWidth(), getTextSize().height);
     }
 
     void AutoSizedEditBox::setCaption(const MyGUI::UString& _value)
     {
         EditBox::setCaption(_value);
+        mWasResized = false;
 
         notifySizeChange (this);
     }
 
     void AutoSizedEditBox::initialiseOverride()
     {
+        mMaxWidth = getSize().width;
         Base::initialiseOverride();
         setNeedKeyFocus(false);
         setEditStatic(true);
@@ -79,13 +110,16 @@ namespace Gui
         {
             mExpandDirection = MyGUI::Align::parse (_value);
         }
+        else if (_key == "Shrink")
+        {
+            mShrink = MyGUI::utility::parseValue<bool>(_value);
+        }
         else
         {
-            EditBox::setPropertyOverride (_key, _value);
+            Gui::EditBox::setPropertyOverride (_key, _value);
         }
     }
 
-
     MyGUI::IntSize AutoSizedButton::getRequestedSize()
     {
         MyGUI::IntSize padding(24, 8);
@@ -111,16 +145,14 @@ namespace Gui
         }
         else
         {
-            Button::setPropertyOverride (_key, _value);
+            Gui::Button::setPropertyOverride (_key, _value);
         }
     }
-
     Box::Box()
         : mSpacing(4)
         , mPadding(0)
         , mAutoResize(false)
     {
-
     }
 
     void Box::notifyChildrenSizeChanged ()
