@@ -2,6 +2,8 @@
 
 #include <MyGUI_RenderManager.h>
 
+#include <components/debug/debuglog.hpp>
+
 namespace Gui
 {
 
@@ -12,6 +14,8 @@ namespace Gui
         , mMouseFocus(false)
         , mMousePress(false)
         , mKeyFocus(false)
+        , mUseWholeTexture(true)
+        , mTextureRect(MyGUI::IntCoord(0, 0, 0, 0))
     {
         setNeedKeyFocus(sDefaultNeedKeyFocus);
     }
@@ -21,6 +25,13 @@ namespace Gui
         sDefaultNeedKeyFocus = enabled;
     }
 
+    void ImageButton::setTextureRect(MyGUI::IntCoord coord)
+    {
+        mTextureRect = coord;
+        mUseWholeTexture = (coord == MyGUI::IntCoord(0, 0, 0, 0));
+        updateImage();
+    }
+
     void ImageButton::setPropertyOverride(const std::string &_key, const std::string &_value)
     {
         if (_key == "ImageHighlighted")
@@ -35,6 +46,11 @@ namespace Gui
             }
             mImageNormal = _value;
         }
+        else if (_key == "TextureRect")
+        {
+            mTextureRect = MyGUI::IntCoord::parse(_value);
+            mUseWholeTexture = (mTextureRect == MyGUI::IntCoord(0, 0, 0, 0));
+        }
         else
             ImageBox::setPropertyOverride(_key, _value);
     }
@@ -64,12 +80,25 @@ namespace Gui
 
     void ImageButton::updateImage()
     {
+        std::string textureName = mImageNormal;
         if (mMousePress)
-            setImageTexture(mImagePushed);
+            textureName = mImagePushed;
         else if (mMouseFocus || mKeyFocus)
-            setImageTexture(mImageHighlighted);
-        else
-            setImageTexture(mImageNormal);
+            textureName = mImageHighlighted;
+
+        if (!mUseWholeTexture)
+        {
+            int scale = 1.f;
+            MyGUI::ITexture* texture = MyGUI::RenderManager::getInstance().getTexture(textureName);
+            if (texture && getHeight() != 0)
+                scale = texture->getHeight() / getHeight();
+
+            setImageTile(MyGUI::IntSize(mTextureRect.width * scale, mTextureRect.height * scale));
+            MyGUI::IntCoord scaledSize(mTextureRect.left * scale, mTextureRect.top * scale, mTextureRect.width * scale, mTextureRect.height * scale);
+            setImageCoord(scaledSize);
+        }
+
+        setImageTexture(textureName);
     }
 
     MyGUI::IntSize ImageButton::getRequestedSize()
@@ -77,10 +106,14 @@ namespace Gui
         MyGUI::ITexture* texture = MyGUI::RenderManager::getInstance().getTexture(mImageNormal);
         if (!texture)
         {
-            std::cerr << "ImageButton: can't find " << mImageNormal << std::endl;
+            Log(Debug::Error) << "ImageButton: can't find image " << mImageNormal;
             return MyGUI::IntSize(0,0);
         }
-        return MyGUI::IntSize (texture->getWidth(), texture->getHeight());
+
+        if (mUseWholeTexture)
+            return MyGUI::IntSize(texture->getWidth(), texture->getHeight());
+
+        return MyGUI::IntSize(mTextureRect.width, mTextureRect.height);
     }
 
     void ImageButton::setImage(const std::string &image)
@@ -94,7 +127,7 @@ namespace Gui
         mImageHighlighted = imageNoExt + "_over" + ext;
         mImagePushed = imageNoExt + "_pressed" + ext;
 
-        setImageTexture(mImageNormal);
+        updateImage();
     }
 
     void ImageButton::onMouseButtonReleased(int _left, int _top, MyGUI::MouseButton _id)
