@@ -2,10 +2,10 @@
 #define VIDEOPLAYER_VIDEOSTATE_H
 
 #include <stdint.h>
+#include <atomic>
 #include <vector>
 #include <memory>
-
-#include <boost/shared_ptr.hpp>
+#include <string>
 
 #include <OpenThreads/Thread>
 #include <OpenThreads/Mutex>
@@ -17,6 +17,19 @@ namespace osg
     class Texture2D;
 }
 
+extern "C"
+{
+#include <libavcodec/avcodec.h>
+#include <libavformat/avformat.h>
+#include <libavutil/imgutils.h>
+#include <libavutil/channel_layout.h>
+
+// From version 54.56 binkaudio encoding format changed from S16 to FLTP. See:
+// https://gitorious.org/ffmpeg/ffmpeg/commit/7bfd1766d1c18f07b0a2dd042418a874d49ea60d
+// https://ffmpeg.zeranoe.com/forum/viewtopic.php?f=15&t=872
+#include <libswresample/swresample.h>
+}
+
 #include "videodefs.hpp"
 
 #define VIDEO_PICTURE_QUEUE_SIZE 50
@@ -66,9 +79,9 @@ struct PacketQueue {
     { clear(); }
 
     AVPacketList *first_pkt, *last_pkt;
-    volatile bool flushing;
-    int nb_packets;
-    int size;
+    std::atomic<bool> flushing;
+    std::atomic<int> nb_packets;
+    std::atomic<int> size;
 
     OpenThreads::Mutex mutex;
     OpenThreads::Condition cond;
@@ -94,7 +107,7 @@ struct VideoState {
 
     void setAudioFactory(MovieAudioFactory* factory);
 
-    void init(boost::shared_ptr<std::istream> inputstream, const std::string& name);
+    void init(std::shared_ptr<std::istream> inputstream, const std::string& name);
     void deinit();
 
     void setPaused(bool isPaused);
@@ -127,12 +140,14 @@ struct VideoState {
     osg::ref_ptr<osg::Texture2D> mTexture;
 
     MovieAudioFactory* mAudioFactory;
-    boost::shared_ptr<MovieAudioDecoder> mAudioDecoder;
+    std::shared_ptr<MovieAudioDecoder> mAudioDecoder;
 
     ExternalClock mExternalClock;
 
-    boost::shared_ptr<std::istream> stream;
+    std::shared_ptr<std::istream> stream;
     AVFormatContext* format_ctx;
+    AVCodecContext* video_ctx;
+    AVCodecContext* audio_ctx;
 
     int av_sync_type;
 
@@ -152,15 +167,15 @@ struct VideoState {
     OpenThreads::Mutex pictq_mutex;
     OpenThreads::Condition pictq_cond;
 
-    std::auto_ptr<ParseThread> parse_thread;
-    std::auto_ptr<VideoThread> video_thread;
+    std::unique_ptr<ParseThread> parse_thread;
+    std::unique_ptr<VideoThread> video_thread;
 
-    volatile bool mSeekRequested;
+    std::atomic<bool> mSeekRequested;
     uint64_t mSeekPos;
 
-    volatile bool mVideoEnded;
-    volatile bool mPaused;
-    volatile bool mQuit;
+    std::atomic<bool> mVideoEnded;
+    std::atomic<bool> mPaused;
+    std::atomic<bool> mQuit;
 };
 
 }
